@@ -8,10 +8,12 @@ from dagster.core.definitions import (
     AssetSensorDefinition,
     CompositeSolidDefinition,
     ConfigMapping,
+    DailyPartitionsDefinition,
     DependencyDefinition,
     DynamicOut,
     DynamicOutput,
     DynamicOutputDefinition,
+    DynamicPartitionsDefinition,
     EventMetadata,
     EventMetadataEntry,
     ExecutorDefinition,
@@ -20,19 +22,23 @@ from dagster.core.definitions import (
     Failure,
     FloatMetadataEntryData,
     GraphDefinition,
+    GraphIn,
+    GraphOut,
     HookDefinition,
+    HourlyPartitionsDefinition,
     In,
     InputDefinition,
     InputMapping,
     IntMetadataEntryData,
-    IntermediateStorageDefinition,
     JobDefinition,
     JsonMetadataEntryData,
     LoggerDefinition,
     MarkdownMetadataEntryData,
     Materialization,
     ModeDefinition,
+    MonthlyPartitionsDefinition,
     MultiDependencyDefinition,
+    NodeInvocation,
     OpDefinition,
     Out,
     Output,
@@ -41,6 +47,8 @@ from dagster.core.definitions import (
     Partition,
     PartitionScheduleDefinition,
     PartitionSetDefinition,
+    PartitionedConfig,
+    PartitionsDefinition,
     PathMetadataEntryData,
     PipelineDefinition,
     PipelineFailureSensorContext,
@@ -63,23 +71,27 @@ from dagster.core.definitions import (
     SkipReason,
     SolidDefinition,
     SolidInvocation,
+    StaticPartitionsDefinition,
     TextMetadataEntryData,
     TypeCheck,
     UrlMetadataEntryData,
+    WeeklyPartitionsDefinition,
     asset_sensor,
     build_init_logger_context,
+    build_schedule_from_partitioned_job,
     composite_solid,
     config_mapping,
     daily_partitioned_config,
     daily_schedule,
     default_executors,
+    dynamic_partitioned_config,
     executor,
     failure_hook,
     graph,
     hourly_partitioned_config,
     hourly_schedule,
     in_process_executor,
-    intermediate_storage,
+    job,
     lambda_solid,
     logger,
     make_values_resource,
@@ -99,20 +111,21 @@ from dagster.core.definitions import (
     schedule_from_partitions,
     sensor,
     solid,
+    static_partitioned_config,
     success_hook,
     weekly_partitioned_config,
     weekly_schedule,
 )
 from dagster.core.definitions.configurable import configured
 from dagster.core.definitions.policy import Backoff, Jitter, RetryPolicy
-from dagster.core.definitions.schedule import build_schedule_context
-from dagster.core.definitions.sensor import build_sensor_context
+from dagster.core.definitions.schedule_definition import build_schedule_context
+from dagster.core.definitions.sensor_definition import build_sensor_context
 from dagster.core.definitions.utils import (
     config_from_files,
     config_from_pkg_resources,
     config_from_yaml_strings,
 )
-from dagster.core.definitions.version_strategy import VersionStrategy
+from dagster.core.definitions.version_strategy import SourceHashVersionStrategy, VersionStrategy
 from dagster.core.errors import (
     DagsterConfigMappingFunctionError,
     DagsterError,
@@ -129,6 +142,7 @@ from dagster.core.errors import (
     DagsterSubprocessError,
     DagsterTypeCheckDidNotPass,
     DagsterTypeCheckError,
+    DagsterUnknownPartitionError,
     DagsterUnknownResourceError,
     DagsterUnmetExecutorRequirementsError,
     DagsterUserCodeExecutionError,
@@ -140,7 +154,7 @@ from dagster.core.execution.api import (
     reexecute_pipeline,
     reexecute_pipeline_iterator,
 )
-from dagster.core.execution.context.compute import SolidExecutionContext
+from dagster.core.execution.context.compute import OpExecutionContext, SolidExecutionContext
 from dagster.core.execution.context.hook import HookContext, build_hook_context
 from dagster.core.execution.context.init import InitResourceContext, build_init_resource_context
 from dagster.core.execution.context.input import InputContext, build_input_context
@@ -148,6 +162,7 @@ from dagster.core.execution.context.invocation import build_op_context, build_so
 from dagster.core.execution.context.logger import InitLoggerContext
 from dagster.core.execution.context.output import OutputContext, build_output_context
 from dagster.core.execution.context.system import TypeCheckContext
+from dagster.core.execution.execute_in_process_result import ExecuteInProcessResult
 from dagster.core.execution.results import (
     CompositeSolidExecutionResult,
     PipelineExecutionResult,
@@ -167,23 +182,15 @@ from dagster.core.storage.event_log import (
 )
 from dagster.core.storage.file_manager import FileHandle, LocalFileHandle, local_file_manager
 from dagster.core.storage.fs_io_manager import custom_path_fs_io_manager, fs_io_manager
-from dagster.core.storage.init import InitIntermediateStorageContext
 from dagster.core.storage.io_manager import IOManager, IOManagerDefinition, io_manager
 from dagster.core.storage.mem_io_manager import mem_io_manager
 from dagster.core.storage.memoizable_io_manager import MemoizableIOManager
-from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
+from dagster.core.storage.pipeline_run import DagsterRun, PipelineRun, PipelineRunStatus
 from dagster.core.storage.root_input_manager import (
     RootInputManager,
     RootInputManagerDefinition,
     root_input_manager,
 )
-from dagster.core.storage.system_storage import (
-    build_intermediate_storage_from_object_store,
-    default_intermediate_storage_defs,
-    fs_intermediate_storage,
-    io_manager_from_intermediate_storage,
-    mem_intermediate_storage,
-)
 from dagster.core.storage.tags import MEMOIZED_RUN_TAG
 from dagster.core.types.config_schema import (
     DagsterTypeLoader,
@@ -196,12 +203,12 @@ from dagster.core.types.decorator import (
     make_python_type_usable_as_dagster_type,
     usable_as_dagster_type,
 )
-from dagster.core.types.marshal import SerializationStrategy
 from dagster.core.types.python_dict import Dict
 from dagster.core.types.python_set import Set
 from dagster.core.types.python_tuple import Tuple
 from dagster.utils import file_relative_path
 from dagster.utils.backcompat import ExperimentalWarning
+from dagster.utils.log import get_dagster_logger
 from dagster.utils.partitions import (
     create_offset_partition_selector,
     date_partition_range,
@@ -235,12 +242,13 @@ __all__ = [
     "Failure",
     "Field",
     "GraphDefinition",
+    "GraphIn",
+    "GraphOut",
     "HookDefinition",
     "JobDefinition",
     "In",
     "InputDefinition",
     "InputMapping",
-    "IntermediateStorageDefinition",
     "JsonMetadataEntryData",
     "LoggerDefinition",
     "build_init_logger_context",
@@ -263,6 +271,7 @@ __all__ = [
     "RepositoryDefinition",
     "ResourceDefinition",
     "SolidDefinition",
+    "NodeInvocation",
     "SolidInvocation",
     "TextMetadataEntryData",
     "UrlMetadataEntryData",
@@ -280,7 +289,7 @@ __all__ = [
     "config_mapping",
     "executor",
     "graph",
-    "intermediate_storage",
+    "job",
     "lambda_solid",
     "logger",
     "op",
@@ -306,8 +315,9 @@ __all__ = [
     "InitExecutorContext",
     "InitLoggerContext",
     "InitResourceContext",
+    "ExecuteInProcessResult",
     "build_init_resource_context",
-    "InitIntermediateStorageContext",
+    "OpExecutionContext",
     "PipelineExecutionResult",
     "RetryRequested",
     "SolidExecutionResult",
@@ -322,17 +332,14 @@ __all__ = [
     "OutputContext",
     "build_output_context",
     "PipelineRun",
+    "DagsterRun",
     "PipelineRunStatus",
     "default_executors",
-    "default_intermediate_storage_defs",
     "execute_pipeline_iterator",
     "execute_pipeline",
     "validate_run_config",
     "execute_solid_within_pipeline",
-    "fs_intermediate_storage",
     "in_process_executor",
-    "mem_intermediate_storage",
-    "io_manager_from_intermediate_storage",
     "multiprocess_executor",
     "multiple_process_executor_requirements",
     "reconstructable",
@@ -354,13 +361,14 @@ __all__ = [
     "DagsterSubprocessError",
     "DagsterTypeCheckDidNotPass",
     "DagsterTypeCheckError",
+    "DagsterUnknownPartitionError",
     "DagsterUnknownResourceError",
     "DagsterUnmetExecutorRequirementsError",
     "DagsterUserCodeExecutionError",
     # Logging
     "DagsterLogManager",
+    "get_dagster_logger",
     # Utilities
-    "build_intermediate_storage_from_object_store",
     "check_dagster_type",
     "execute_solid",
     "execute_solids_within_pipeline",
@@ -380,7 +388,6 @@ __all__ = [
     "List",
     "Nothing",
     "Optional",
-    "SerializationStrategy",
     "Set",
     "String",
     "Tuple",
@@ -415,12 +422,23 @@ __all__ = [
     "EventRecordsFilter",
     "RunShardedEventsCursor",
     # partitions and schedules
+    "build_schedule_from_partitioned_job",
     "schedule_from_partitions",
+    "dynamic_partitioned_config",
+    "static_partitioned_config",
     "daily_partitioned_config",
     "hourly_partitioned_config",
     "monthly_partitioned_config",
     "weekly_partitioned_config",
+    "DynamicPartitionsDefinition",
+    "StaticPartitionsDefinition",
+    "DailyPartitionsDefinition",
+    "HourlyPartitionsDefinition",
+    "MonthlyPartitionsDefinition",
+    "WeeklyPartitionsDefinition",
     "Partition",
+    "PartitionedConfig",
+    "PartitionsDefinition",
     "PartitionScheduleDefinition",
     "PartitionSetDefinition",
     "RunRequest",
@@ -459,4 +477,5 @@ __all__ = [
     "VersionStrategy",
     "MEMOIZED_RUN_TAG",
     "MemoizableIOManager",
+    "SourceHashVersionStrategy",
 ]
