@@ -28,17 +28,19 @@ def get_ephemeral_repository_name(pipeline_name):
 
 @whitelist_for_serdes
 class ReconstructableRepository(
-    namedtuple("_ReconstructableRepository", "pointer container_image")
+    namedtuple("_ReconstructableRepository", "pointer container_image executable_path")
 ):
     def __new__(
         cls,
         pointer,
         container_image=None,
+        executable_path=None,
     ):
         return super(ReconstructableRepository, cls).__new__(
             cls,
             pointer=check.inst_param(pointer, "pointer", CodePointer),
             container_image=check.opt_str_param(container_image, "container_image"),
+            executable_path=check.opt_str_param(executable_path, "executable_path"),
         )
 
     @lru_cache(maxsize=1)
@@ -66,7 +68,7 @@ class ReconstructableRepository(
 
     def get_python_origin(self):
         return RepositoryPythonOrigin(
-            executable_path=sys.executable,
+            executable_path=self.executable_path if self.executable_path else sys.executable,
             code_pointer=self.pointer,
             container_image=self.container_image,
         )
@@ -105,21 +107,37 @@ class ReconstructablePipeline(
 
     @lru_cache(maxsize=1)
     def get_definition(self):
-        return (
-            self.repository.get_definition()
-            .get_pipeline(self.pipeline_name)
-            .get_pipeline_subset_def(self.solids_to_execute)
-        )
+        from dagster.core.definitions.job_definition import JobDefinition
+
+        defn = self.repository.get_definition().get_pipeline(self.pipeline_name)
+
+        if isinstance(defn, JobDefinition):
+            return (
+                self.repository.get_definition()
+                .get_pipeline(self.pipeline_name)
+                .get_job_def_for_op_selection(
+                    list(self.solids_to_execute) if self.solids_to_execute else None
+                )
+            )
+        else:
+            return (
+                self.repository.get_definition()
+                .get_pipeline(self.pipeline_name)
+                .get_pipeline_subset_def(self.solids_to_execute)
+            )
 
     def _resolve_solid_selection(self, solid_selection):
         # resolve a list of solid selection queries to a frozenset of qualified solid names
         # e.g. ['foo_solid+'] to {'foo_solid', 'bar_solid'}
         check.list_param(solid_selection, "solid_selection", of_type=str)
-        solids_to_execute = parse_solid_selection(self.get_definition(), solid_selection)
+        pipeline_def = self.get_definition()
+        solids_to_execute = parse_solid_selection(pipeline_def, solid_selection)
         if len(solids_to_execute) == 0:
             raise DagsterInvalidSubsetError(
-                "No qualified solids to execute found for solid_selection={requested}".format(
-                    requested=solid_selection
+                "No qualified {node_type} to execute found for {selection_type}={requested}".format(
+                    requested=solid_selection,
+                    node_type="ops" if pipeline_def.is_job else "solids",
+                    selection_type="op_selection" if pipeline_def.is_job else "solid_selection",
                 )
             )
         return solids_to_execute
@@ -141,7 +159,6 @@ class ReconstructablePipeline(
                 pipeline_name=self.pipeline_name,
             )
 
-        pipe.get_definition()  # verify the subset is correct
         return pipe
 
     def subset_for_execution(self, solid_selection):
@@ -150,7 +167,6 @@ class ReconstructablePipeline(
         solids_to_execute = (
             self._resolve_solid_selection(solid_selection) if solid_selection else None
         )
-
         return self._subset_for_execution(solids_to_execute, solid_selection)
 
     def subset_for_execution_from_existing_pipeline(self, solids_to_execute):
@@ -230,44 +246,73 @@ class ReconstructableSchedule(
 def reconstructable(target):
     """
     Create a :py:class:`~dagster.core.definitions.reconstructable.ReconstructablePipeline` from a
-    function that returns a :py:class:`~dagster.PipelineDefinition`, or a function decorated with
-    :py:func:`@pipeline <dagster.pipeline>`
+    function that returns a :py:class:`~dagster.PipelineDefinition`/:py:class:`~dagster.JobDefinition`,
+    or a function decorated with :py:func:`@pipeline <dagster.pipeline>`/:py:func:`@job <dagster.job>`.
 
-    When your pipeline must cross process boundaries, e.g., for execution on multiple nodes or
-    in different systems (like ``dagstermill``), Dagster must know how to reconstruct the pipeline
+    When your pipeline/job must cross process boundaries, e.g., for execution on multiple nodes or
+    in different systems (like ``dagstermill``), Dagster must know how to reconstruct the pipeline/job
     on the other side of the process boundary.
 
-    This function implements a very conservative strategy for reconstructing pipelines, so that
-    its behavior is easy to predict, but as a consequence it is not able to reconstruct certain
-    kinds of pipelines, such as those defined by lambdas, in nested scopes (e.g., dynamically
-    within a method call), or in interactive environments such as the Python REPL or Jupyter
-    notebooks.
+    Passing a job created with ``~dagster.GraphDefinition.to_job`` to ``reconstructable()``,
+    requires you to wrap that job's definition in a module-scoped function, and pass that function
+    instead:
 
-    If you need to reconstruct pipelines constructed in these ways, you should use
+    .. code-block:: python
+
+        from dagster import graph, reconstructable
+
+        @graph
+        def my_graph():
+            ...
+
+        def define_my_job():
+            return my_graph.to_job()
+
+        reconstructable(define_my_job)
+
+    This function implements a very conservative strategy for reconstruction, so that its behavior
+    is easy to predict, but as a consequence it is not able to reconstruct certain kinds of pipelines
+    or jobs, such as those defined by lambdas, in nested scopes (e.g., dynamically within a method
+    call), or in interactive environments such as the Python REPL or Jupyter notebooks.
+
+    If you need to reconstruct objects constructed in these ways, you should use
     :py:func:`~dagster.core.definitions.reconstructable.build_reconstructable_pipeline` instead,
-    which allows you to specify your own strategy for reconstructing a pipeline.
+    which allows you to specify your own reconstruction strategy.
 
     Examples:
 
     .. code-block:: python
 
-        from dagster import PipelineDefinition, pipeline, reconstructable
+        from dagster import job, reconstructable
 
-        @pipeline
-        def foo_pipeline():
+        @job
+        def foo_job():
             ...
 
-        reconstructable_foo_pipeline = reconstructable(foo_pipeline)
+        reconstructable_foo_job = reconstructable(foo_job)
 
 
-        def make_bar_pipeline():
-            return PipelineDefinition(...)
+        @graph
+        def foo():
+            ...
 
-        reconstructable_bar_pipeline = reconstructable(bar_pipeline)
+        def make_bar_job():
+            return foo.to_job()
+
+        reconstructable_bar_job = reconstructable(make_bar_job)
     """
-    from dagster.core.definitions import PipelineDefinition
+    from dagster.core.definitions import PipelineDefinition, JobDefinition
 
     if not seven.is_function_or_decorator_instance_of(target, PipelineDefinition):
+        if isinstance(target, JobDefinition):
+            raise DagsterInvariantViolationError(
+                "Reconstructable target was not a function returning a job definition, or a job "
+                "definition produced by a decorated function. If your job was constructed using "
+                "``GraphDefinition.to_job``, you must wrap the ``to_job`` call in a function at "
+                "module scope, ie not within any other functions. "
+                "To learn more, check out the docs on ``reconstructable``: "
+                "https://docs.dagster.io/_apidocs/execution#dagster.reconstructable"
+            )
         raise DagsterInvariantViolationError(
             "Reconstructable target should be a function or definition produced "
             "by a decorated function, got {type}.".format(type=type(target)),
@@ -277,7 +322,7 @@ def reconstructable(target):
         raise DagsterInvariantViolationError(
             "Reconstructable target can not be a lambda. Use a function or "
             "decorated function defined at module scope instead, or use "
-            "build_reconstructable_pipeline."
+            "build_reconstructable_target."
         )
 
     if seven.qualname_differs(target):
@@ -297,16 +342,16 @@ def reconstructable(target):
             and inspect.getmodule(target).__name__ != "__main__"
         ):
             return ReconstructablePipeline.for_module(target.__module__, target.__name__)
-    except:  # pylint: disable=bare-except
+    except:
         pass
 
     python_file = get_python_file_from_target(target)
     if not python_file:
         raise DagsterInvariantViolationError(
-            "reconstructable() can not reconstruct pipelines defined in interactive environments "
+            "reconstructable() can not reconstruct jobs or pipelines defined in interactive environments "
             "like <stdin>, IPython, or Jupyter notebooks. "
             "Use a pipeline defined in a module or file instead, or "
-            "use build_reconstructable_pipeline."
+            "use build_reconstructable_target."
         )
 
     pointer = FileCodePointer(
@@ -317,7 +362,7 @@ def reconstructable(target):
 
 
 @experimental
-def build_reconstructable_pipeline(
+def build_reconstructable_target(
     reconstructor_module_name,
     reconstructor_function_name,
     reconstructable_args=None,
@@ -412,6 +457,9 @@ def build_reconstructable_pipeline(
     )
 
 
+build_reconstructable_pipeline = build_reconstructable_target
+
+
 def bootstrap_standalone_recon_pipeline(pointer):
     # So this actually straps the the pipeline for the sole
     # purpose of getting the pipeline name. If we changed ReconstructablePipeline
@@ -424,13 +472,14 @@ def bootstrap_standalone_recon_pipeline(pointer):
 
 
 def _check_is_loadable(definition):
-    from .pipeline import PipelineDefinition
-    from .repository import RepositoryDefinition
+    from .pipeline_definition import PipelineDefinition
+    from .repository_definition import RepositoryDefinition
+    from .graph_definition import GraphDefinition
 
-    if not isinstance(definition, (PipelineDefinition, RepositoryDefinition)):
+    if not isinstance(definition, (PipelineDefinition, RepositoryDefinition, GraphDefinition)):
         raise DagsterInvariantViolationError(
             (
-                "Loadable attributes must be either a PipelineDefinition or a "
+                "Loadable attributes must be either a JobDefinition, GraphDefinition, PipelineDefinition, or a "
                 "RepositoryDefinition. Got {definition}."
             ).format(definition=repr(definition))
         )
@@ -452,10 +501,13 @@ def load_def_in_python_file(python_file, attribute, working_directory):
 def def_from_pointer(pointer):
     target = pointer.load_target()
 
-    from .pipeline import PipelineDefinition
-    from .repository import RepositoryDefinition
+    from .pipeline_definition import PipelineDefinition
+    from .repository_definition import RepositoryDefinition
+    from .graph_definition import GraphDefinition
 
-    if isinstance(target, (PipelineDefinition, RepositoryDefinition)) or not callable(target):
+    if isinstance(
+        target, (PipelineDefinition, RepositoryDefinition, GraphDefinition)
+    ) or not callable(target):
         return _check_is_loadable(target)
 
     # if its a function invoke it - otherwise we are pointing to a
@@ -473,7 +525,7 @@ def def_from_pointer(pointer):
 
 
 def pipeline_def_from_pointer(pointer):
-    from .pipeline import PipelineDefinition
+    from .pipeline_definition import PipelineDefinition
 
     target = def_from_pointer(pointer)
 
@@ -481,21 +533,22 @@ def pipeline_def_from_pointer(pointer):
         return target
 
     raise DagsterInvariantViolationError(
-        "CodePointer ({str}) must resolve to a PipelineDefinition. "
+        "CodePointer ({str}) must resolve to a JobDefinition (or PipelineDefinition for legacy code). "
         "Received a {type}".format(str=pointer.describe(), type=type(target))
     )
 
 
 def repository_def_from_target_def(target):
-    from .pipeline import PipelineDefinition
-    from .repository import RepositoryData, RepositoryDefinition
+    from .pipeline_definition import PipelineDefinition
+    from .graph_definition import GraphDefinition
+    from .repository_definition import CachingRepositoryData, RepositoryDefinition
 
     # special case - we can wrap a single pipeline in a repository
-    if isinstance(target, PipelineDefinition):
+    if isinstance(target, (PipelineDefinition, GraphDefinition)):
         # consider including pipeline name in generated repo name
         return RepositoryDefinition(
             name=get_ephemeral_repository_name(target.name),
-            repository_data=RepositoryData.from_list([target]),
+            repository_data=CachingRepositoryData.from_list([target]),
         )
     elif isinstance(target, RepositoryDefinition):
         return target
@@ -509,7 +562,7 @@ def repository_def_from_pointer(pointer):
     if not repo_def:
         raise DagsterInvariantViolationError(
             "CodePointer ({str}) must resolve to a "
-            "RepositoryDefinition or a PipelineDefinition. "
+            "RepositoryDefinition, JobDefinition, or PipelineDefinition. "
             "Received a {type}".format(str=pointer.describe(), type=type(target))
         )
     return repo_def
