@@ -4,9 +4,8 @@ from typing import Any, Optional
 from dagster import check
 from dagster.core.definitions.dependency import Node, NodeHandle
 from dagster.core.definitions.mode import ModeDefinition
-from dagster.core.definitions.pipeline import PipelineDefinition
-from dagster.core.definitions.resource import Resources
-from dagster.core.definitions.solid import SolidDefinition
+from dagster.core.definitions.pipeline_definition import PipelineDefinition
+from dagster.core.definitions.solid_definition import SolidDefinition
 from dagster.core.definitions.step_launcher import StepLauncher
 from dagster.core.errors import DagsterInvalidPropertyError
 from dagster.core.instance import DagsterInstance
@@ -111,14 +110,14 @@ class SolidExecutionContext(AbstractComputeExecutionContext):
 
     @property
     def pdb(self) -> ForkedPdb:
-        """dagster.utils.forked_pdb.ForkedPdb: Gives access to pdb debugging from within the solid.
+        """dagster.utils.forked_pdb.ForkedPdb: Gives access to pdb debugging from within the op.
 
         Example:
 
         .. code-block:: python
 
-            @solid
-            def debug_solid(context):
+            @op
+            def debug(context):
                 context.pdb.set_trace()
 
         """
@@ -139,7 +138,7 @@ class SolidExecutionContext(AbstractComputeExecutionContext):
         )
 
     @property
-    def resources(self) -> Resources:
+    def resources(self) -> Any:
         """Resources: The currently available resources."""
         return self._step_execution_context.resources
 
@@ -200,6 +199,19 @@ class SolidExecutionContext(AbstractComputeExecutionContext):
         """SolidDefinition: The current solid definition."""
         return self._step_execution_context.pipeline_def.get_solid(self.solid_handle).definition
 
+    @property
+    def has_partition_key(self) -> bool:
+        """Whether the current run is a partitioned run"""
+        return self._step_execution_context.has_partition_key
+
+    @property
+    def partition_key(self) -> str:
+        """The partition key for the current run.
+
+        Raises an error if the current run is not a partitioned run.
+        """
+        return self._step_execution_context.partition_key
+
     def has_tag(self, key: str) -> bool:
         """Check if a logging tag is set.
 
@@ -241,8 +253,15 @@ class SolidExecutionContext(AbstractComputeExecutionContext):
 
         return self._step_execution_context.previous_attempt_count
 
+    def describe_op(self):
+        return self._step_execution_context.describe_op()
+
     def get_mapping_key(self) -> Optional[str]:
         """
         Which mapping_key this execution is for if downstream of a DynamicOutput, otherwise None.
         """
         return self._step_execution_context.step.get_mapping_key()
+
+
+class OpExecutionContext(SolidExecutionContext):
+    pass
