@@ -1,3 +1,4 @@
+import inspect
 from collections import defaultdict
 from typing import Any, Dict, Iterator, List, Optional, Set, Tuple, Union, cast
 
@@ -32,7 +33,6 @@ from dagster.core.execution.plan.inputs import StepInputData
 from dagster.core.execution.plan.objects import StepSuccessData, TypeCheckData
 from dagster.core.execution.plan.outputs import StepOutputData, StepOutputHandle
 from dagster.core.execution.resolve_versions import resolve_step_output_versions
-from dagster.core.storage.intermediate_storage import IntermediateStorageAdapter
 from dagster.core.storage.io_manager import IOManager
 from dagster.core.storage.tags import MEMOIZED_RUN_TAG
 from dagster.core.types.dagster_type import DagsterType, DagsterTypeKind
@@ -58,6 +58,7 @@ def _step_output_error_checked_user_event_sequence(
     check.generator_param(user_event_sequence, "user_event_sequence")
 
     step = step_context.step
+    op_label = step_context.describe_op()
     output_names = list([output_def.name for output_def in step.step_outputs])
     seen_outputs: Set[str] = set()
     seen_mapping_keys: Dict[str, Set[str]] = defaultdict(set)
@@ -69,16 +70,13 @@ def _step_output_error_checked_user_event_sequence(
 
         # do additional processing on Outputs
         output = user_event
-        if not step.has_step_output(output.output_name):
+        if not step.has_step_output(cast(str, output.output_name)):
             raise DagsterInvariantViolationError(
-                'Core compute for solid "{handle}" returned an output '
-                '"{output.output_name}" that does not exist. The available '
-                "outputs are {output_names}".format(
-                    handle=str(step.solid_handle), output=output, output_names=output_names
-                )
+                f'Core compute for {op_label} returned an output "{output.output_name}" that does '
+                f"not exist. The available outputs are {output_names}"
             )
 
-        step_output = step.step_output_named(output.output_name)
+        step_output = step.step_output_named(cast(str, output.output_name))
         output_def = step_context.pipeline_def.get_solid(step_output.solid_handle).output_def_named(
             step_output.name
         )
@@ -86,27 +84,25 @@ def _step_output_error_checked_user_event_sequence(
         if isinstance(output, Output):
             if output.output_name in seen_outputs:
                 raise DagsterInvariantViolationError(
-                    'Compute for solid "{handle}" returned an output '
-                    '"{output.output_name}" multiple times'.format(
-                        handle=str(step.solid_handle), output=output
-                    )
+                    f'Compute for {op_label} returned an output "{output.output_name}" multiple '
+                    "times"
                 )
 
             if output_def.is_dynamic:
                 raise DagsterInvariantViolationError(
-                    f'Compute for solid "{step.solid_handle}" for output "{output.output_name}" '
-                    "defined as dynamic must yield DynamicOutput, got Output."
+                    f'Compute for {op_label} for output "{output.output_name}" defined as dynamic '
+                    "must yield DynamicOutput, got Output."
                 )
         else:
             if not output_def.is_dynamic:
                 raise DagsterInvariantViolationError(
-                    f'Compute for solid "{step.solid_handle}" yielded a DynamicOutput, '
-                    "but did not use DynamicOutputDefinition."
+                    f"Compute for {op_label} yielded a DynamicOutput, but did not use "
+                    "DynamicOutputDefinition."
                 )
             if output.mapping_key in seen_mapping_keys[output.output_name]:
                 raise DagsterInvariantViolationError(
-                    f'Compute for solid "{step.solid_handle}" yielded a DynamicOutput with '
-                    f'mapping_key "{output.mapping_key}" multiple times.'
+                    f"Compute for {op_label} yielded a DynamicOutput with mapping_key "
+                    f'"{output.mapping_key}" multiple times.'
                 )
             seen_mapping_keys[output.output_name].add(output.mapping_key)
 
@@ -118,16 +114,14 @@ def _step_output_error_checked_user_event_sequence(
         if not step_output_def.name in seen_outputs and not step_output_def.optional:
             if step_output_def.dagster_type.kind == DagsterTypeKind.NOTHING:
                 step_context.log.info(
-                    'Emitting implicit Nothing for output "{output}" on solid {solid}'.format(
-                        output=step_output_def.name, solid={str(step.solid_handle)}
-                    )
+                    f'Emitting implicit Nothing for output "{step_output_def.name}" on {op_label}'
                 )
                 yield Output(output_name=step_output_def.name, value=None)
             elif not step_output_def.is_dynamic:
                 raise DagsterStepOutputNotFoundError(
-                    'Core compute for solid "{handle}" did not return an output '
-                    'for non-optional output "{step_output_def.name}"'.format(
-                        handle=str(step.solid_handle), step_output_def=step_output_def
+                    (
+                        f"Core compute for {op_label} did not return an output for non-optional "
+                        f'output "{step_output_def.name}"'
                     ),
                     step_key=step.key,
                     output_name=step_output_def.name,
@@ -177,15 +171,19 @@ def _type_checked_event_sequence_for_input(
     step_input = step_context.step.step_input_named(input_name)
     input_def = step_input.source.get_input_def(step_context.pipeline_def)
     dagster_type = input_def.dagster_type
+    type_check_context = step_context.for_type(dagster_type)
+    input_type = type(input_value)
+    op_label = step_context.describe_op()
+
     with user_code_error_boundary(
         DagsterTypeCheckError,
         lambda: (
-            f'Error occurred while type-checking input "{input_name}" of solid '
-            f'"{str(step_context.step.solid_handle)}", with Python type {type(input_value)} and '
-            f"Dagster type {dagster_type.display_name}"
+            f'Error occurred while type-checking input "{input_name}" of {op_label}, with Python '
+            f"type {input_type} and Dagster type {dagster_type.display_name}"
         ),
+        log_manager=type_check_context.log,
     ):
-        type_check = do_type_check(step_context.for_type(dagster_type), dagster_type, input_value)
+        type_check = do_type_check(type_check_context, dagster_type, input_value)
 
     yield _create_step_input_event(
         step_context, input_name, type_check=type_check, success=type_check.success
@@ -216,15 +214,19 @@ def _type_check_output(
     step_output_def = step_context.solid_def.output_def_named(step_output.name)
 
     dagster_type = step_output_def.dagster_type
+    type_check_context = step_context.for_type(dagster_type)
+    op_label = step_context.describe_op()
+    output_type = type(output.value)
+
     with user_code_error_boundary(
         DagsterTypeCheckError,
         lambda: (
-            f'Error occurred while type-checking output "{output.output_name}" of solid '
-            f'"{str(step_context.step.solid_handle)}", with Python type {type(output.value)} and '
-            f"Dagster type {dagster_type.display_name}"
+            f'Error occurred while type-checking output "{output.output_name}" of {op_label}, with '
+            f"Python type {output_type} and Dagster type {dagster_type.display_name}"
         ),
+        log_manager=type_check_context.log,
     ):
-        type_check = do_type_check(step_context.for_type(dagster_type), dagster_type, output.value)
+        type_check = do_type_check(type_check_context, dagster_type, output.value)
 
     yield DagsterEvent.step_output_event(
         step_context=step_context,
@@ -441,7 +443,7 @@ def _get_output_asset_materializations(
     all_metadata = output.metadata_entries + io_manager_metadata_entries
 
     if asset_partitions:
-        metadata_mapping: Dict[str, List[Union[str, "EventMetadataEntry"]]] = {
+        metadata_mapping: Dict[str, List["EventMetadataEntry"]] = {
             partition: [] for partition in asset_partitions
         }
         for entry in all_metadata:
@@ -471,7 +473,9 @@ def _get_output_asset_materializations(
                     f"Output {output_def.name} got a PartitionMetadataEntry ({entry}), but "
                     "is not associated with any specific partitions."
                 )
-        yield AssetMaterialization(asset_key=asset_key, metadata_entries=all_metadata)
+        yield AssetMaterialization(
+            asset_key=asset_key, metadata_entries=cast(List["EventMetadataEntry"], all_metadata)
+        )
 
 
 def _store_output(
@@ -485,37 +489,51 @@ def _store_output(
     output_manager = step_context.get_io_manager(step_output_handle)
     output_context = step_context.get_output_context(step_output_handle)
 
-    handle_output_res = output_manager.handle_output(output_context, output.value)
-
     manager_materializations = []
     manager_metadata_entries = []
-    if handle_output_res is not None:
-        for elt in iterate_with_context(
-            lambda: solid_execution_error_boundary(
-                DagsterExecutionHandleOutputError,
-                msg_fn=lambda: (
-                    f'Error occurred while handling output "{output_context.name}" of '
-                    f'step "{step_context.step.key}":'
-                ),
-                step_context=step_context,
-                step_key=step_context.step.key,
-                output_name=output_context.name,
+
+    # output_manager.handle_output is either a generator function, or a normal function with or
+    # without a return value. In the case that handle_output is a normal function, we need to
+    # catch errors should they be raised before a return value. We can do this by wrapping
+    # handle_output in a generator so that errors will be caught within iterate_with_context.
+
+    if not inspect.isgeneratorfunction(output_manager.handle_output):
+
+        def _gen_fn():
+            gen_output = output_manager.handle_output(output_context, output.value)
+            if gen_output:
+                yield gen_output
+
+        handle_output_gen = _gen_fn()
+    else:
+        handle_output_gen = output_manager.handle_output(output_context, output.value)
+
+    for elt in iterate_with_context(
+        lambda: solid_execution_error_boundary(
+            DagsterExecutionHandleOutputError,
+            msg_fn=lambda: (
+                f'Error occurred while handling output "{output_context.name}" of '
+                f'step "{step_context.step.key}":'
             ),
-            ensure_gen(handle_output_res),
-        ):
-            if isinstance(elt, AssetMaterialization):
-                manager_materializations.append(elt)
-            elif isinstance(elt, (EventMetadataEntry, PartitionMetadataEntry)):
-                experimental_functionality_warning(
-                    "Yielding metadata from an IOManager's handle_output() function"
-                )
-                manager_metadata_entries.append(elt)
-            else:
-                raise DagsterInvariantViolationError(
-                    f"IO manager on output {output_def.name} has returned "
-                    f"value {elt} of type {type(elt).__name__}. The return type can only be "
-                    "one of AssetMaterialization, EventMetadataEntry, PartitionMetadataEntry."
-                )
+            step_context=step_context,
+            step_key=step_context.step.key,
+            output_name=output_context.name,
+        ),
+        handle_output_gen,
+    ):
+        if isinstance(elt, AssetMaterialization):
+            manager_materializations.append(elt)
+        elif isinstance(elt, (EventMetadataEntry, PartitionMetadataEntry)):
+            experimental_functionality_warning(
+                "Yielding metadata from an IOManager's handle_output() function"
+            )
+            manager_metadata_entries.append(elt)
+        else:
+            raise DagsterInvariantViolationError(
+                f"IO manager on output {output_def.name} has returned "
+                f"value {elt} of type {type(elt).__name__}. The return type can only be "
+                "one of AssetMaterialization, EventMetadataEntry, PartitionMetadataEntry."
+            )
 
     # do not alter explicitly created AssetMaterializations
     for materialization in manager_materializations:
@@ -538,9 +556,6 @@ def _store_output(
         step_context,
         output_name=step_output_handle.output_name,
         manager_key=output_def.io_manager_key,
-        message_override=f'Handled input "{step_output_handle.output_name}" using intermediate storage'
-        if isinstance(output_manager, IntermediateStorageAdapter)
-        else None,
         metadata_entries=[
             entry for entry in manager_metadata_entries if isinstance(entry, EventMetadataEntry)
         ],
@@ -576,6 +591,7 @@ def _create_type_materializations(
                         f'\n    solid invocation: "{step_context.solid.name}"'
                         f'\n    solid definition: "{step_context.solid_def.name}"'
                     ),
+                    log_manager=step_context.log,
                 ):
                     output_def = step_context.solid_def.output_def_named(step_output.name)
                     dagster_type = output_def.dagster_type
