@@ -1,5 +1,6 @@
 import inspect
 import logging
+import logging.config
 import os
 import sys
 import tempfile
@@ -25,8 +26,11 @@ from typing import (
 import yaml
 from dagster import check
 from dagster.core.definitions.events import AssetKey
-from dagster.core.definitions.pipeline import PipelineDefinition, PipelineSubsetDefinition
 from dagster.core.definitions.pipeline_base import InMemoryPipeline
+from dagster.core.definitions.pipeline_definition import (
+    PipelineDefinition,
+    PipelineSubsetDefinition,
+)
 from dagster.core.errors import (
     DagsterHomeNotSetError,
     DagsterInvariantViolationError,
@@ -34,6 +38,8 @@ from dagster.core.errors import (
     DagsterRunConflict,
 )
 from dagster.core.storage.pipeline_run import (
+    IN_PROGRESS_RUN_STATUSES,
+    DagsterRun,
     PipelineRun,
     PipelineRunStatsSnapshot,
     PipelineRunStatus,
@@ -45,6 +51,7 @@ from dagster.core.system_config.objects import ResolvedRunConfig
 from dagster.core.utils import str_format_list
 from dagster.serdes import ConfigurableClass
 from dagster.seven import get_current_datetime_in_utc
+from dagster.utils.backcompat import experimental_functionality_warning
 from dagster.utils.error import serializable_error_info_from_exc_info
 
 from .config import DAGSTER_CONFIG_YAML_FILENAME, is_dagster_home_set
@@ -185,7 +192,7 @@ class DagsterInstance:
     For example, to use Postgres for run and event log storage, you can write a ``dagster.yaml``
     such as the following:
 
-    .. literalinclude:: ../../../../docs/sections/deploying/postgres_dagster.yaml
+    .. literalinclude:: ../../../../../examples/docs_snippets/docs_snippets/deploying/postgres_dagster.yaml
        :caption: dagster.yaml
        :language: YAML
 
@@ -306,6 +313,12 @@ class DagsterInstance:
 
         self._subscribers: Dict[str, List[Callable]] = defaultdict(list)
 
+        if self.run_monitoring_enabled:
+            check.invariant(
+                self.run_launcher.supports_check_run_worker_health,
+                "Run monitoring only supports select RunLaunchers",
+            )
+
     # ctors
 
     @staticmethod
@@ -345,6 +358,10 @@ class DagsterInstance:
                     "file which can configure storing metadata in an external database.\n"
                     "You can resolve this error by exporting the environment variable. For example, you can run the following command in your shell or include it in your shell configuration file:\n"
                     '\texport DAGSTER_HOME="~/dagster_home"\n'
+                    "or PowerShell\n"
+                    "$env:DAGSTER_HOME = ($home + '\\dagster_home')"
+                    "or batch"
+                    "set DAGSTER_HOME=%UserProfile%/dagster_home"
                     "Alternatively, DagsterInstance.ephemeral() can be used for a transient instance.\n"
                 )
             )
@@ -545,9 +562,65 @@ class DagsterInstance:
 
         if "enabled" in telemetry_settings:
             return telemetry_settings["enabled"]
+        elif "experimental_dagit" in telemetry_settings:
+            return telemetry_settings["experimental_dagit"]
         else:
             return dagster_telemetry_enabled_default
 
+    # run monitoring
+
+    @property
+    def run_monitoring_enabled(self) -> bool:
+        if self.is_ephemeral:
+            return False
+
+        run_monitoring_enabled_default = False
+
+        run_monitoring_settings = self.get_settings("run_monitoring")
+
+        if not run_monitoring_settings:
+            return run_monitoring_enabled_default
+
+        if "enabled" in run_monitoring_settings:
+            return run_monitoring_settings["enabled"]
+        else:
+            return run_monitoring_enabled_default
+
+    @property
+    def run_monitoring_settings(self) -> Dict:
+        check.invariant(self.run_monitoring_enabled, "run_monitoring is not enabled")
+        return self.get_settings("run_monitoring")
+
+    @property
+    def run_monitoring_start_timeout_seconds(self) -> int:
+        return self.run_monitoring_settings.get("start_timeout_seconds", 180)
+
+    @property
+    def run_monitoring_max_resume_run_attempts(self) -> int:
+        return self.run_monitoring_settings.get("max_resume_run_attempts", 3)
+
+    @property
+    def run_monitoring_poll_interval_seconds(self) -> int:
+        return self.run_monitoring_settings.get("poll_interval_seconds", 120)
+
+    @property
+    def cancellation_thread_poll_interval_seconds(self) -> int:
+        return self.get_settings("run_monitoring").get(
+            "cancellation_thread_poll_interval_seconds", 10
+        )
+
+    # python logs
+
+    @property
+    def managed_python_loggers(self) -> List[str]:
+        python_log_settings = self.get_settings("python_logs") or {}
+        return python_log_settings.get("managed_python_loggers", [])
+
+    @property
+    def python_log_level(self) -> Optional[str]:
+        python_log_settings = self.get_settings("python_logs") or {}
+        return python_log_settings.get("python_log_level")
+
     def upgrade(self, print_fn=None):
         from dagster.core.storage.migration.utils import upgrading_instance
 
@@ -569,7 +642,6 @@ class DagsterInstance:
 
     def optimize_for_dagit(self, statement_timeout):
         if self._schedule_storage:
-            self._schedule_storage.validate_stored_schedules(self.scheduler_class)
             self._schedule_storage.optimize_for_dagit(statement_timeout=statement_timeout)
         self._run_storage.optimize_for_dagit(statement_timeout=statement_timeout)
         self._event_storage.optimize_for_dagit(statement_timeout=statement_timeout)
@@ -598,6 +670,9 @@ class DagsterInstance:
     def has_pipeline_snapshot(self, snapshot_id: str) -> bool:
         return self._run_storage.has_pipeline_snapshot(snapshot_id)
 
+    def has_snapshot(self, snapshot_id: str) -> bool:
+        return self._run_storage.has_snapshot(snapshot_id)
+
     def get_historical_pipeline(self, snapshot_id: str) -> "HistoricalPipeline":
         from dagster.core.host_representation import HistoricalPipeline
 
@@ -607,9 +682,7 @@ class DagsterInstance:
             if snapshot.lineage_snapshot
             else None
         )
-        return HistoricalPipeline(
-            self._run_storage.get_pipeline_snapshot(snapshot_id), snapshot_id, parent_snapshot
-        )
+        return HistoricalPipeline(snapshot, snapshot_id, parent_snapshot)
 
     def has_historical_pipeline(self, snapshot_id: str) -> bool:
         return self._run_storage.has_pipeline_snapshot(snapshot_id)
@@ -646,6 +719,7 @@ class DagsterInstance:
         pipeline_code_origin=None,
     ):
         from dagster.core.execution.plan.plan import ExecutionPlan
+        from dagster.core.execution.api import create_execution_plan
         from dagster.core.snap import snapshot_from_execution_plan
 
         check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)
@@ -682,10 +756,11 @@ class DagsterInstance:
             step_keys_to_execute = execution_plan.step_keys_to_execute
 
         else:
-            resolved_run_config = ResolvedRunConfig.build(pipeline_def, run_config, mode)
-            execution_plan = ExecutionPlan.build(
-                InMemoryPipeline(pipeline_def),
-                resolved_run_config,
+            execution_plan = create_execution_plan(
+                pipeline=InMemoryPipeline(pipeline_def),
+                run_config=run_config,
+                mode=mode,
+                instance_ref=self.get_ref() if self.is_persistent else None,
                 tags=tags,
             )
 
@@ -757,7 +832,7 @@ class DagsterInstance:
             else None
         )
 
-        return PipelineRun(
+        return DagsterRun(
             pipeline_name=pipeline_name,
             run_id=run_id,
             run_config=run_config,
@@ -953,6 +1028,9 @@ class DagsterInstance:
     def add_run(self, pipeline_run: PipelineRun):
         return self._run_storage.add_run(pipeline_run)
 
+    def add_snapshot(self, snapshot, snapshot_id=None):
+        return self._run_storage.add_snapshot(snapshot, snapshot_id)
+
     def handle_run_event(self, run_id: str, event: "DagsterEvent"):
         return self._run_storage.handle_run_event(run_id, event)
 
@@ -1006,8 +1084,19 @@ class DagsterInstance:
 
     # event storage
 
-    def logs_after(self, run_id, cursor, of_type: "DagsterEventType" = None):
-        return self._event_storage.get_logs_for_run(run_id, cursor=cursor, of_type=of_type)
+    def logs_after(
+        self,
+        run_id,
+        cursor,
+        of_type: "DagsterEventType" = None,
+        limit: Optional[int] = None,
+    ):
+        return self._event_storage.get_logs_for_run(
+            run_id,
+            cursor=cursor,
+            of_type=of_type,
+            limit=limit,
+        )
 
     def all_logs(self, run_id, of_type: "DagsterEventType" = None):
         return self._event_storage.get_logs_for_run(run_id, of_type=of_type)
@@ -1023,6 +1112,9 @@ class DagsterInstance:
     def all_asset_keys(self):
         return self._event_storage.all_asset_keys()
 
+    def get_asset_keys(self, prefix=None, limit=None, cursor=None):
+        return self._event_storage.get_asset_keys(prefix=prefix, limit=limit, cursor=cursor)
+
     def has_asset_key(self, asset_key: AssetKey) -> bool:
         return self._event_storage.has_asset_key(asset_key)
 
@@ -1094,11 +1186,10 @@ records = instance.get_event_records(
         return self._event_storage.get_asset_run_ids(asset_key)
 
     def all_asset_tags(self):
-        return self._event_storage.all_asset_tags()
+        return {}
 
-    def get_asset_tags(self, asset_key):
-        check.inst_param(asset_key, "asset_key", AssetKey)
-        return self._event_storage.get_asset_tags(asset_key)
+    def get_asset_tags(self, asset_key):  # pylint: disable=unused-argument
+        return {}
 
     def wipe_assets(self, asset_keys):
         check.list_param(asset_keys, "asset_keys", of_type=AssetKey)
@@ -1107,11 +1198,45 @@ records = instance.get_event_records(
 
     # event subscriptions
 
-    def get_logger(self):
-        logger = logging.Logger("__event_listener")
-        logger.addHandler(_EventListenerLogHandler(self))
-        logger.setLevel(10)
-        return logger
+    def _get_yaml_python_handlers(self):
+        if self._settings:
+            logging_config = self.get_settings("python_logs").get("dagster_handler_config", {})
+
+            if logging_config:
+                experimental_functionality_warning("Handling yaml-defined logging configuration")
+
+            # Handlers can only be retrieved from dictConfig configuration if they are attached
+            # to a logger. We add a dummy logger to the configuration that allows us to access user
+            # defined handlers.
+            handler_names = logging_config.get("handlers", {}).keys()
+
+            dagster_dummy_logger_name = "dagster_dummy_logger"
+
+            processed_dict_conf = {
+                "version": 1,
+                "disable_existing_loggers": False,
+                "loggers": {dagster_dummy_logger_name: {"handlers": handler_names}},
+            }
+            processed_dict_conf.update(logging_config)
+
+            logging.config.dictConfig(processed_dict_conf)
+
+            dummy_logger = logging.getLogger(dagster_dummy_logger_name)
+            return dummy_logger.handlers
+        return []
+
+    def _get_event_log_handler(self):
+        event_log_handler = _EventListenerLogHandler(self)
+        event_log_handler.setLevel(10)
+        return event_log_handler
+
+    def get_handlers(self):
+        handlers = [self._get_event_log_handler()]
+        handlers.extend(self._get_yaml_python_handlers())
+        return handlers
+
+    def store_event(self, event):
+        self._event_storage.store_event(event)
 
     def handle_new_event(self, event):
         run_id = event.run_id
@@ -1163,6 +1288,7 @@ records = instance.get_event_records(
             pipeline_name=pipeline_run.pipeline_name,
             message=message,
             event_specific_data=engine_event_data,
+            step_key=step_key,
         )
         event_record = EventLogEntry(
             message=message,
@@ -1188,7 +1314,7 @@ records = instance.get_event_records(
         message = check.opt_str_param(
             message,
             "message",
-            "Sending pipeline termination request.",
+            "Sending run termination request.",
         )
         canceling_event = DagsterEvent(
             event_type_value=DagsterEventType.PIPELINE_CANCELING.value,
@@ -1222,7 +1348,7 @@ records = instance.get_event_records(
         message = check.opt_str_param(
             message,
             "mesage",
-            "This pipeline run has been marked as canceled from outside the execution context.",
+            "This run has been marked as canceled from outside the execution context.",
         )
 
         dagster_event = DagsterEvent(
@@ -1253,7 +1379,7 @@ records = instance.get_event_records(
         message = check.opt_str_param(
             message,
             "message",
-            "This pipeline run has been marked as failed from outside the execution context.",
+            "This run has been marked as failed from outside the execution context.",
         )
 
         dagster_event = DagsterEvent(
@@ -1280,9 +1406,6 @@ records = instance.get_event_records(
     def file_manager_directory(self, run_id):
         return self._local_artifact_storage.file_manager_dir(run_id)
 
-    def intermediates_directory(self, run_id):
-        return self._local_artifact_storage.intermediates_dir(run_id)
-
     def storage_directory(self):
         return self._local_artifact_storage.storage_dir
 
@@ -1405,10 +1528,67 @@ records = instance.get_event_records(
 
         return run
 
-    # Scheduler
+    def resume_run(self, run_id: str, workspace: "IWorkspace", attempt_number: int):
+        """Resume a pipeline run.
 
-    def reconcile_scheduler_state(self, external_repository):
-        return self._scheduler.reconcile_scheduler_state(self, external_repository)
+        This method should be called on runs which have already been launched, but whose run workers
+        have died.
+
+        Args:
+            run_id (str): The id of the run the launch.
+        """
+        from dagster.core.launcher import ResumeRunContext
+        from dagster.core.events import EngineEventData
+        from dagster.daemon.monitoring import RESUME_RUN_LOG_MESSAGE
+
+        run = self.get_run_by_id(run_id)
+        if run is None:
+            raise DagsterInvariantViolationError(
+                f"Could not load run {run_id} that was passed to resume_run"
+            )
+        if run.status not in IN_PROGRESS_RUN_STATUSES:
+            raise DagsterInvariantViolationError(
+                f"Run {run_id} is not in a state that can be resumed"
+            )
+
+        self.report_engine_event(
+            RESUME_RUN_LOG_MESSAGE,
+            run,
+        )
+
+        try:
+            self._run_launcher.resume_run(
+                ResumeRunContext(
+                    pipeline_run=run,
+                    workspace=workspace,
+                    resume_attempt_number=attempt_number,
+                )
+            )
+        except:
+            error = serializable_error_info_from_exc_info(sys.exc_info())
+            self.report_engine_event(
+                error.message,
+                run,
+                EngineEventData.engine_error(error),
+            )
+            self.report_run_failed(run)
+            raise
+
+        return run
+
+    def count_resume_run_attempts(self, run_id: str):
+        from dagster.daemon.monitoring import RESUME_RUN_LOG_MESSAGE
+        from dagster.core.events import DagsterEventType
+
+        events = self.all_logs(run_id, of_type=DagsterEventType.ENGINE_EVENT)
+        return len([event for event in events if event.message == RESUME_RUN_LOG_MESSAGE])
+
+    def run_will_resume(self, run_id: str):
+        if not self.run_monitoring_enabled:
+            return False
+        return self.count_resume_run_attempts(run_id) < self.run_monitoring_max_resume_run_attempts
+
+    # Scheduler
 
     def start_schedule_and_update_storage_state(self, external_schedule):
         return self._scheduler.start_schedule_and_update_storage_state(self, external_schedule)
@@ -1515,6 +1695,12 @@ records = instance.get_event_records(
     def delete_job_state(self, job_origin_id):
         return self._schedule_storage.delete_job_state(job_origin_id)
 
+    def get_job_tick(self, job_origin_id, timestamp):
+        matches = self._schedule_storage.get_job_ticks(
+            job_origin_id, before=timestamp + 1, after=timestamp - 1, limit=1
+        )
+        return matches[0] if len(matches) else None
+
     def get_job_ticks(self, job_origin_id, before=None, after=None, limit=None):
         return self._schedule_storage.get_job_ticks(
             job_origin_id, before=before, after=after, limit=limit
@@ -1580,7 +1766,12 @@ records = instance.get_event_records(
     def get_required_daemon_types(self):
         from dagster.core.run_coordinator import QueuedRunCoordinator
         from dagster.core.scheduler import DagsterDaemonScheduler
-        from dagster.daemon.daemon import SchedulerDaemon, SensorDaemon, BackfillDaemon
+        from dagster.daemon.daemon import (
+            SchedulerDaemon,
+            SensorDaemon,
+            BackfillDaemon,
+            MonitoringDaemon,
+        )
         from dagster.daemon.run_coordinator.queued_run_coordinator_daemon import (
             QueuedRunCoordinatorDaemon,
         )
@@ -1593,6 +1784,8 @@ records = instance.get_event_records(
             daemons.append(SchedulerDaemon.daemon_type())
         if isinstance(self.run_coordinator, QueuedRunCoordinator):
             daemons.append(QueuedRunCoordinatorDaemon.daemon_type())
+        if self.run_monitoring_enabled:
+            daemons.append(MonitoringDaemon.daemon_type())
         return daemons
 
     # backfill
@@ -1607,3 +1800,21 @@ records = instance.get_event_records(
 
     def update_backfill(self, partition_backfill):
         return self._run_storage.update_backfill(partition_backfill)
+
+    @property
+    def should_start_background_run_thread(self) -> bool:
+        """
+        Gate on an experimental feature to start a thread that monitors for if the run should be canceled.
+        """
+        return False
+
+
+def is_dagit_telemetry_enabled(instance):
+    telemetry_settings = instance.get_settings("telemetry")
+    if not telemetry_settings:
+        return False
+
+    if "experimental_dagit" in telemetry_settings:
+        return telemetry_settings["experimental_dagit"]
+    else:
+        return False
