@@ -1,6 +1,7 @@
+import logging
 import time
 from collections import OrderedDict, defaultdict
-from typing import Dict
+from typing import Iterable, Optional
 
 from dagster import check
 from dagster.core.definitions.events import AssetKey
@@ -8,7 +9,13 @@ from dagster.core.events import DagsterEventType
 from dagster.core.events.log import EventLogEntry
 from dagster.serdes import ConfigurableClass
 
-from .base import EventLogRecord, EventLogStorage, EventsCursor, extract_asset_events_cursor
+from .base import (
+    EventLogRecord,
+    EventLogStorage,
+    EventRecordsFilter,
+    RunShardedEventsCursor,
+    extract_asset_events_cursor,
+)
 
 
 class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
@@ -42,7 +49,13 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
     def from_config_value(cls, inst_data, config_value):
         return cls(inst_data)
 
-    def get_logs_for_run(self, run_id, cursor=-1, of_type=None):
+    def get_logs_for_run(
+        self,
+        run_id,
+        cursor=-1,
+        of_type=None,
+        limit=None,
+    ):
         check.str_param(run_id, "run_id")
         check.int_param(cursor, "cursor")
         check.invariant(
@@ -53,7 +66,7 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
 
         cursor = cursor + 1
         if of_type:
-            return list(
+            events = list(
                 filter(
                     lambda r: r.is_dagster_event
                     and r.dagster_event.event_type_value == of_type.value,
@@ -61,7 +74,12 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
                 )
             )
         else:
-            return self._logs[run_id][cursor:]
+            events = self._logs[run_id][cursor:]
+
+        if limit:
+            events = events[:limit]
+
+        return events
 
     def store_event(self, event):
         check.inst_param(event, "event", EventLogEntry)
@@ -72,8 +90,14 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
             materialization = event.dagster_event.step_materialization_data.materialization
             self._asset_tags[event.dagster_event.asset_key] = materialization.tags or {}
 
-        for handler in self._handlers[run_id]:
-            handler(event)
+        # snapshot handlers
+        handlers = list(self._handlers[run_id])
+
+        for handler in handlers:
+            try:
+                handler(event)
+            except Exception:
+                logging.exception("Exception in callback for event watch on run %s.", run_id)
 
     def delete_events(self, run_id):
         del self._logs[run_id]
@@ -81,7 +105,10 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
     def upgrade(self):
         pass
 
-    def reindex(self, print_fn=lambda _: None, force=False):
+    def reindex_events(self, print_fn=None, force=False):
+        pass
+
+    def reindex_assets(self, print_fn=None, force=False):
         pass
 
     def wipe(self):
@@ -100,29 +127,80 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
 
     def get_event_records(
         self,
-        after_cursor=None,
-        limit=None,
-        ascending=False,
-        of_type=None,
-    ):
-        check.opt_inst_param(after_cursor, "after_cursor", EventsCursor)
-        after_id = after_cursor.id if after_cursor else None
+        event_records_filter: Optional[EventRecordsFilter] = None,
+        limit: Optional[int] = None,
+        ascending: bool = False,
+    ) -> Iterable[EventLogRecord]:
+        after_id = (
+            (
+                event_records_filter.after_cursor.id
+                if isinstance(event_records_filter.after_cursor, RunShardedEventsCursor)
+                else event_records_filter.after_cursor
+            )
+            if event_records_filter
+            else None
+        )
+        before_id = (
+            (
+                event_records_filter.before_cursor.id
+                if isinstance(event_records_filter.before_cursor, RunShardedEventsCursor)
+                else event_records_filter.before_cursor
+            )
+            if event_records_filter
+            else None
+        )
 
         filtered_events = []
 
+        def _apply_filters(record):
+            if not event_records_filter:
+                return True
+
+            if (
+                event_records_filter.event_type
+                and record.dagster_event.event_type_value != event_records_filter.event_type.value
+            ):
+                return False
+
+            if (
+                event_records_filter.asset_key
+                and record.dagster_event.asset_key != event_records_filter.asset_key
+            ):
+                return False
+
+            if (
+                event_records_filter.asset_key
+                and self._wiped_asset_keys[event_records_filter.asset_key] > record.timestamp
+            ):
+                return False
+
+            if (
+                event_records_filter.asset_partitions
+                and record.dagster_event.partition not in event_records_filter.asset_partitions
+            ):
+                return False
+
+            if (
+                event_records_filter.after_timestamp
+                and record.timestamp >= event_records_filter.after_timestamp
+            ):
+                return False
+
+            if (
+                event_records_filter.before_timestamp
+                and record.timestamp >= event_records_filter.before_timestamp
+            ):
+                return False
+            return True
+
         for records in self._logs.values():
-            filtered_events += list(
-                filter(
-                    lambda r: of_type is None
-                    or (r.is_dagster_event and r.dagster_event.event_type_value == of_type.value),
-                    records,
-                )
-            )
+            filtered_events += list(filter(_apply_filters, records))
 
         event_records = [
             EventLogRecord(storage_id=event_id, event_log_entry=event)
             for event_id, event in enumerate(filtered_events)
-            if after_id is None or event_id > after_id
+            if (after_id is None or event_id > after_id)
+            and (before_id is None or event_id < before_id)
         ]
 
         event_records = sorted(event_records, key=lambda x: x.storage_id, reverse=not ascending)
@@ -175,40 +253,24 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
         before_timestamp=None,
         cursor=None,
     ):
-        asset_events = []
-        for records in self._logs.values():
-            asset_events += [
-                record
-                for record in records
-                if record.is_dagster_event
-                and record.dagster_event.asset_key == asset_key
-                and (not partitions or record.dagster_event.partition in partitions)
-                and self._wiped_asset_keys[record.dagster_event.asset_key] < record.timestamp
-            ]
-
         before_cursor, after_cursor = extract_asset_events_cursor(
             cursor, before_cursor, after_cursor, ascending
         )
-
-        events_with_ids = [
-            tuple([event_id, event])
-            for event_id, event in enumerate(asset_events)
-            if (after_cursor is None or event_id > after_cursor)
-            and (before_cursor is None or event_id < before_cursor)
-            and (before_timestamp is None or event.timestamp < before_timestamp)
-        ]
-
-        events_with_ids = sorted(
-            events_with_ids, key=lambda x: x[1].timestamp, reverse=not ascending
+        event_records = self.get_event_records(
+            EventRecordsFilter(
+                asset_key=asset_key,
+                asset_partitions=partitions,
+                before_cursor=before_cursor,
+                after_cursor=after_cursor,
+                before_timestamp=before_timestamp,
+            ),
+            limit=limit,
+            ascending=ascending,
         )
-
-        if limit:
-            events_with_ids = events_with_ids[:limit]
-
         if include_cursor:
-            return events_with_ids
-
-        return [event for _id, event in events_with_ids]
+            return [tuple([record.storage_id, record.event_log_entry]) for record in event_records]
+        else:
+            return [record.event_log_entry for record in event_records]
 
     def get_asset_run_ids(self, asset_key):
         asset_run_ids = set()
@@ -229,9 +291,3 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
         self._wiped_asset_keys[asset_key] = time.time()
         if asset_key in self._asset_tags:
             del self._asset_tags[asset_key]
-
-    def all_asset_tags(self) -> Dict[AssetKey, Dict[str, str]]:
-        return {asset_key: tags for asset_key, tags in self._asset_tags.items()}
-
-    def get_asset_tags(self, asset_key: AssetKey) -> Dict[str, str]:
-        return self._asset_tags[asset_key]
