@@ -1,8 +1,8 @@
 import logging
 from abc import abstractmethod
-from collections import defaultdict
+from collections import OrderedDict
 from datetime import datetime
-from typing import Iterable, Optional
+from typing import Iterable, List, Optional, cast
 
 import pendulum
 import sqlalchemy as db
@@ -12,7 +12,7 @@ from dagster.core.definitions.events import AssetKey, AssetMaterialization
 from dagster.core.errors import DagsterEventLogInvalidForRun
 from dagster.core.events import DagsterEventType
 from dagster.core.events.log import EventLogEntry
-from dagster.core.execution.stats import RunStepKeyStatsSnapshot, StepEventStatus
+from dagster.core.execution.stats import build_run_step_stats_from_events
 from dagster.serdes import deserialize_json_to_dagster_namedtuple, serialize_dagster_namedtuple
 from dagster.serdes.errors import DeserializationError
 from dagster.utils import datetime_as_float, utc_datetime_from_naive, utc_datetime_from_timestamp
@@ -28,6 +28,8 @@ from .base import (
 from .migration import ASSET_DATA_MIGRATIONS, ASSET_KEY_INDEX_COLS, EVENT_LOG_DATA_MIGRATIONS
 from .schema import AssetKeyTable, SecondaryIndexMigrationTable, SqlEventLogStorageTable
 
+MIN_ASSET_ROWS = 25
+
 
 class SqlEventLogStorage(EventLogStorage):
     """Base class for SQL backed event log storages.
@@ -306,73 +308,18 @@ class SqlEventLogStorage(EventLogStorage):
         check.str_param(run_id, "run_id")
         check.opt_list_param(step_keys, "step_keys", of_type=str)
 
-        STEP_STATS_EVENT_TYPES = [
-            DagsterEventType.STEP_START.value,
-            DagsterEventType.STEP_SUCCESS.value,
-            DagsterEventType.STEP_SKIPPED.value,
-            DagsterEventType.STEP_FAILURE.value,
-            DagsterEventType.STEP_RESTARTED.value,
-        ]
-
-        by_step_query = (
-            db.select(
-                [
-                    SqlEventLogStorageTable.c.step_key,
-                    SqlEventLogStorageTable.c.dagster_event_type,
-                    db.func.max(SqlEventLogStorageTable.c.timestamp).label("timestamp"),
-                    db.func.count(SqlEventLogStorageTable.c.id).label("n_events_of_type_for_step"),
-                ]
-            )
-            .where(SqlEventLogStorageTable.c.run_id == run_id)
-            .where(SqlEventLogStorageTable.c.step_key != None)
-            .where(SqlEventLogStorageTable.c.dagster_event_type.in_(STEP_STATS_EVENT_TYPES))
-        )
-
-        if step_keys:
-            by_step_query = by_step_query.where(SqlEventLogStorageTable.c.step_key.in_(step_keys))
-
-        by_step_query = by_step_query.group_by(
-            SqlEventLogStorageTable.c.step_key,
-            SqlEventLogStorageTable.c.dagster_event_type,
-        )
-
-        with self.run_connection(run_id) as conn:
-            results = conn.execute(by_step_query).fetchall()
-
-        by_step_key = defaultdict(dict)
-        for result in results:
-            step_key = result.step_key
-            if result.dagster_event_type == DagsterEventType.STEP_START.value:
-                by_step_key[step_key]["start_time"] = (
-                    datetime_as_float(result.timestamp) if result.timestamp else None
-                )
-                by_step_key[step_key]["attempts"] = by_step_key[step_key].get("attempts", 0) + 1
-            if result.dagster_event_type == DagsterEventType.STEP_RESTARTED.value:
-                by_step_key[step_key]["attempts"] = (
-                    # In case we see step retarted events but not a step started event, we want to
-                    # only count the restarted events, since the attempt count represents
-                    # the number of times we have successfully started runnning the step
-                    by_step_key[step_key].get("attempts", 0)
-                    + result.n_events_of_type_for_step
-                )
-            if result.dagster_event_type == DagsterEventType.STEP_FAILURE.value:
-                by_step_key[step_key]["end_time"] = (
-                    datetime_as_float(result.timestamp) if result.timestamp else None
-                )
-                by_step_key[step_key]["status"] = StepEventStatus.FAILURE
-            if result.dagster_event_type == DagsterEventType.STEP_SUCCESS.value:
-                by_step_key[step_key]["end_time"] = (
-                    datetime_as_float(result.timestamp) if result.timestamp else None
-                )
-                by_step_key[step_key]["status"] = StepEventStatus.SUCCESS
-            if result.dagster_event_type == DagsterEventType.STEP_SKIPPED.value:
-                by_step_key[step_key]["end_time"] = (
-                    datetime_as_float(result.timestamp) if result.timestamp else None
-                )
-                by_step_key[step_key]["status"] = StepEventStatus.SKIPPED
-
-        materializations = defaultdict(list)
-        expectation_results = defaultdict(list)
+        # Originally, this was two different queries:
+        # 1) one query which aggregated top-level step stats by grouping by event type / step_key in
+        #    a single query, using pure SQL (e.g. start_time, end_time, status, attempt counts).
+        # 2) one query which fetched all the raw events for a specific event type and then inspected
+        #    the deserialized event object to aggregate stats derived from sequences of events.
+        #    (e.g. marker events, materializations, expectations resuls, attempts timing, etc.)
+        #
+        # For simplicity, we now just do the second type of query and derive the stats in Python
+        # from the raw events.  This has the benefit of being easier to read and also the benefit of
+        # being able to share code with the in-memory event log storage implementation.  We may
+        # choose to revisit this in the future, especially if we are able to do JSON-column queries
+        # in SQL as a way of bypassing the serdes layer in all cases.
         raw_event_query = (
             db.select([SqlEventLogStorageTable.c.event])
             .where(SqlEventLogStorageTable.c.run_id == run_id)
@@ -380,14 +327,21 @@ class SqlEventLogStorage(EventLogStorage):
             .where(
                 SqlEventLogStorageTable.c.dagster_event_type.in_(
                     [
+                        DagsterEventType.STEP_START.value,
+                        DagsterEventType.STEP_SUCCESS.value,
+                        DagsterEventType.STEP_SKIPPED.value,
+                        DagsterEventType.STEP_FAILURE.value,
+                        DagsterEventType.STEP_RESTARTED.value,
                         DagsterEventType.ASSET_MATERIALIZATION.value,
                         DagsterEventType.STEP_EXPECTATION_RESULT.value,
+                        DagsterEventType.STEP_RESTARTED.value,
+                        DagsterEventType.STEP_UP_FOR_RETRY.value,
+                        DagsterEventType.ENGINE_EVENT.value,
                     ]
                 )
             )
             .order_by(SqlEventLogStorageTable.c.id.asc())
         )
-
         if step_keys:
             raw_event_query = raw_event_query.where(
                 SqlEventLogStorageTable.c.step_key.in_(step_keys)
@@ -397,35 +351,16 @@ class SqlEventLogStorage(EventLogStorage):
             results = conn.execute(raw_event_query).fetchall()
 
         try:
-            for (json_str,) in results:
-                event = check.inst_param(
+            records = [
+                check.inst_param(
                     deserialize_json_to_dagster_namedtuple(json_str), "event", EventLogEntry
                 )
-                if event.dagster_event.event_type == DagsterEventType.ASSET_MATERIALIZATION:
-                    materializations[event.step_key].append(
-                        event.dagster_event.event_specific_data.materialization
-                    )
-                elif event.dagster_event.event_type == DagsterEventType.STEP_EXPECTATION_RESULT:
-                    expectation_results[event.step_key].append(
-                        event.dagster_event.event_specific_data.expectation_result
-                    )
+                for (json_str,) in results
+            ]
+            return build_run_step_stats_from_events(run_id, records)
         except (seven.JSONDecodeError, DeserializationError) as err:
             raise DagsterEventLogInvalidForRun(run_id=run_id) from err
 
-        return [
-            RunStepKeyStatsSnapshot(
-                run_id=run_id,
-                step_key=step_key,
-                status=value.get("status"),
-                start_time=value.get("start_time"),
-                end_time=value.get("end_time"),
-                materializations=materializations.get(step_key),
-                expectation_results=expectation_results.get(step_key),
-                attempts=value.get("attempts"),
-            )
-            for step_key, value in by_step_key.items()
-        ]
-
     def _apply_migration(self, migration_name, migration_fn, print_fn, force):
         if self.has_secondary_index(migration_name):
             if not force:
@@ -720,135 +655,170 @@ class SqlEventLogStorage(EventLogStorage):
 
     def has_asset_key(self, asset_key: AssetKey) -> bool:
         check.inst_param(asset_key, "asset_key", AssetKey)
-        if self.has_secondary_index(ASSET_KEY_INDEX_COLS):
-            query = (
-                db.select([AssetKeyTable.c.asset_key])
-                .where(
-                    db.or_(
-                        AssetKeyTable.c.asset_key == asset_key.to_string(),
-                        AssetKeyTable.c.asset_key == asset_key.to_string(legacy=True),
-                    )
-                )
-                .where(
-                    db.or_(
-                        AssetKeyTable.c.wipe_timestamp == None,
-                        AssetKeyTable.c.last_materialization_timestamp
-                        > AssetKeyTable.c.wipe_timestamp,
-                    )
-                )
-                .limit(1)
-            )
-            with self.index_connection() as conn:
-                row = conn.execute(query).fetchone()
-                return bool(row)
-
-        # has not migrated, need to pull asset_details to get wipe status
-        query = (
-            db.select([AssetKeyTable.c.asset_key, AssetKeyTable.c.asset_details])
-            .where(
-                db.or_(
-                    AssetKeyTable.c.asset_key == asset_key.to_string(),
-                    AssetKeyTable.c.asset_key == asset_key.to_string(legacy=True),
-                )
-            )
-            .limit(1)
-        )
-
-        with self.index_connection() as conn:
-            row = conn.execute(query).fetchone()
-            if not row:
-                return False
-
-            asset_details: Optional[AssetDetails] = AssetDetails.from_db_string(row[1])
-            if not asset_details or not asset_details.last_wipe_timestamp:
-                return True
-
-            materialization_row = conn.execute(
-                db.select([SqlEventLogStorageTable.c.timestamp])
-                .where(
-                    db.or_(
-                        AssetKeyTable.c.asset_key == asset_key.to_string(),
-                        AssetKeyTable.c.asset_key == asset_key.to_string(legacy=True),
-                    )
-                )
-                .order_by(SqlEventLogStorageTable.c.timestamp.desc())
-                .limit(1)
-            ).fetchone()
-            if not materialization_row:
-                return False
-
-            return utc_datetime_from_naive(materialization_row[0]) > utc_datetime_from_timestamp(
-                asset_details.last_wipe_timestamp
-            )
+        rows = self._fetch_asset_rows(asset_keys=[asset_key])
+        return bool(rows)
 
     def all_asset_keys(self):
+        rows = self._fetch_asset_rows()
+        asset_keys = [AssetKey.from_db_string(row[0]) for row in sorted(rows, key=lambda x: x[0])]
+        return [asset_key for asset_key in asset_keys if asset_key]
+
+    def get_asset_keys(
+        self,
+        prefix: Optional[List[str]] = None,
+        limit: Optional[int] = None,
+        cursor: Optional[str] = None,
+    ) -> Iterable[AssetKey]:
+        rows = self._fetch_asset_rows(prefix=prefix, limit=limit, cursor=cursor)
+        asset_keys = [AssetKey.from_db_string(row[0]) for row in sorted(rows, key=lambda x: x[0])]
+        return [asset_key for asset_key in asset_keys if asset_key]
+
+    def _fetch_asset_rows(self, asset_keys=None, prefix=None, limit=None, cursor=None):
+        # fetches rows containing asset_key, last_materialization, and asset_details from the DB,
+        # applying the filters specified in the arguments.
+        #
+        # Differs from _fetch_raw_asset_rows, in that it loops through to make sure enough rows are
+        # returned to satisfy the limit.
+        #
+        # returns a list of rows where each row is a tuple of serialized asset_key, materialization,
+        # and asset_details
+        should_query = True
+        current_cursor = cursor
         if self.has_secondary_index(ASSET_KEY_INDEX_COLS):
+            # if we have migrated, we can limit using SQL
+            fetch_limit = limit
+        else:
+            # if we haven't migrated, overfetch in case the first N results are wiped
+            fetch_limit = max(limit, MIN_ASSET_ROWS) if limit else None
+        result = []
+
+        while should_query:
+            rows, has_more, new_cursor = self._fetch_raw_asset_rows(
+                asset_keys=asset_keys, prefix=prefix, limit=fetch_limit, cursor=current_cursor
+            )
+            result.extend(rows)
+            should_query = bool(has_more) and bool(limit) and len(result) < cast(int, limit)
+            current_cursor = new_cursor
+
+        return result[:limit] if limit else result
+
+    def _fetch_raw_asset_rows(self, asset_keys=None, prefix=None, limit=None, cursor=None):
+        # fetches rows containing asset_key, last_materialization, and asset_details from the DB,
+        # applying the filters specified in the arguments.  Does not guarantee that the number of
+        # rows returned will match the limit specified.  This helper function is used to fetch a
+        # chunk of asset key rows, which may or may not be wiped.
+        #
+        # Returns a tuple of (rows, has_more, cursor), where each row is a tuple of serialized
+        # asset_key, materialization, and asset_details
+
+        query = db.select(
+            [
+                AssetKeyTable.c.asset_key,
+                AssetKeyTable.c.last_materialization,
+                AssetKeyTable.c.asset_details,
+            ]
+        ).order_by(AssetKeyTable.c.asset_key.asc())
+        query = self._apply_asset_filter_to_query(query, asset_keys, prefix, limit, cursor)
+
+        if self.has_secondary_index(ASSET_KEY_INDEX_COLS):
+            query = query.where(
+                db.or_(
+                    AssetKeyTable.c.wipe_timestamp == None,
+                    AssetKeyTable.c.last_materialization_timestamp > AssetKeyTable.c.wipe_timestamp,
+                )
+            )
             with self.index_connection() as conn:
-                results = conn.execute(
-                    db.select([AssetKeyTable.c.asset_key]).where(
-                        db.or_(
-                            AssetKeyTable.c.wipe_timestamp == None,
-                            AssetKeyTable.c.last_materialization_timestamp
-                            > AssetKeyTable.c.wipe_timestamp,
-                        )
-                    )
-                ).fetchall()
-                return list(set([AssetKey.from_db_string(result[0]) for result in results]))
+                rows = conn.execute(query).fetchall()
+
+            return rows, False, None
 
-        # has not migrated, need to fetch the wipe_timestamp from asset_details
         with self.index_connection() as conn:
-            results = conn.execute(
-                db.select([AssetKeyTable.c.asset_key, AssetKeyTable.c.asset_details])
-            ).fetchall()
+            rows = conn.execute(query).fetchall()
 
-            asset_keys = set()
-            wiped = set()
-            wiped_timestamps = {}
-            for result in results:
-                asset_key = AssetKey.from_db_string(result[0])
-                asset_details: Optional[AssetDetails] = AssetDetails.from_db_string(result[1])
-                asset_keys.add(asset_key)
-                if asset_details and asset_details.last_wipe_timestamp:
-                    wiped_timestamps[asset_key] = asset_details.last_wipe_timestamp
+        wiped_timestamps_by_asset_key = {}
+        row_by_asset_key = OrderedDict()
 
-            if wiped_timestamps:
-                materialized_timestamps = {}
+        for row in rows:
+            asset_key = AssetKey.from_db_string(row[0])
+            if not asset_key:
+                continue
+            asset_details = AssetDetails.from_db_string(row[2])
+            if not asset_details or not asset_details.last_wipe_timestamp:
+                row_by_asset_key[asset_key] = row
+                continue
+            materialization_or_event = (
+                deserialize_json_to_dagster_namedtuple(row[1]) if row[1] else None
+            )
+            if isinstance(materialization_or_event, EventLogEntry):
+                if asset_details.last_wipe_timestamp > materialization_or_event.timestamp:
+                    # this asset has not been materialized since being wiped, skip
+                    continue
+                else:
+                    # add the key
+                    row_by_asset_key[asset_key] = row
+            else:
+                row_by_asset_key[asset_key] = row
+                wiped_timestamps_by_asset_key[asset_key] = asset_details.last_wipe_timestamp
 
-                # fetch the last materialization timestamp per asset key
-                materialization_results = conn.execute(
-                    db.select(
-                        [
-                            SqlEventLogStorageTable.c.asset_key,
-                            db.func.max(SqlEventLogStorageTable.c.timestamp),
-                        ]
-                    )
-                    .where(
-                        SqlEventLogStorageTable.c.asset_key.in_(
-                            [asset_key.to_string() for asset_key in wiped_timestamps.keys()]
-                        )
-                    )
-                    .group_by(SqlEventLogStorageTable.c.asset_key)
-                    .order_by(db.func.max(SqlEventLogStorageTable.c.timestamp).asc())
-                ).fetchall()
-
-                for result in materialization_results:
-                    asset_key = AssetKey.from_db_string(result[0])
-                    last_materialized_timestamp = result[1]
-                    materialized_timestamps[asset_key] = last_materialized_timestamp
-
-                # calculate the set of wiped asset keys that have not had a materialization since
-                # the wipe timestamp
-                wiped = set(
+        if wiped_timestamps_by_asset_key:
+            backcompat_query = (
+                db.select(
                     [
-                        asset_key
-                        for asset_key in wiped_timestamps.keys()
-                        if not materialized_timestamps.get(asset_key)
-                        or utc_datetime_from_naive(materialized_timestamps.get(asset_key))
-                        < utc_datetime_from_timestamp(wiped_timestamps[asset_key])
+                        SqlEventLogStorageTable.c.asset_key,
+                        db.func.max(SqlEventLogStorageTable.c.timestamp),
                     ]
                 )
+                .where(
+                    SqlEventLogStorageTable.c.asset_key.in_(
+                        [
+                            asset_key.to_string()
+                            for asset_key in wiped_timestamps_by_asset_key.keys()
+                        ]
+                    )
+                )
+                .group_by(SqlEventLogStorageTable.c.asset_key)
+                .order_by(db.func.max(SqlEventLogStorageTable.c.timestamp).asc())
+            )
+            with self.index_connection() as conn:
+                backcompat_rows = conn.execute(backcompat_query).fetchall()
+            materialization_times = {
+                AssetKey.from_db_string(row[0]): row[1] for row in backcompat_rows
+            }
+            for asset_key, wiped_timestamp in wiped_timestamps_by_asset_key.items():
+                materialization_time = materialization_times.get(asset_key)
+                if not materialization_time or utc_datetime_from_naive(
+                    materialization_time
+                ) < utc_datetime_from_timestamp(wiped_timestamp):
+                    # remove rows that have not been materialized since being wiped
+                    row_by_asset_key.pop(asset_key)
 
-        return list(asset_keys.difference(wiped))
+        has_more = limit and len(rows) == limit
+        new_cursor = rows[-1][0] if rows else None
+        return row_by_asset_key.values(), has_more, new_cursor
+
+    def _apply_asset_filter_to_query(
+        self,
+        query,
+        asset_keys=None,
+        prefix=None,
+        limit=None,
+        cursor=None,
+    ):
+        if asset_keys:
+            query = query.where(
+                AssetKeyTable.c.asset_key.in_([asset_key.to_string() for asset_key in asset_keys])
+            )
+
+        if prefix:
+            prefix_str = seven.dumps(prefix)[:-1]
+            query = query.where(AssetKeyTable.c.asset_key.startswith(prefix_str))
+
+        if cursor:
+            query = query.where(AssetKeyTable.c.asset_key > cursor)
+
+        if limit:
+            query = query.limit(limit)
+        return query
 
     def _get_asset_details(self, asset_key):
         check.inst_param(asset_key, "asset_key", AssetKey)
@@ -961,72 +931,6 @@ class SqlEventLogStorage(EventLogStorage):
 
         return event_or_materialization.dagster_event.step_materialization_data.materialization
 
-    def all_asset_tags(self):
-        tags_by_asset_key = defaultdict(dict)
-        if self.has_secondary_index(ASSET_KEY_INDEX_COLS):
-            query = (
-                db.select([AssetKeyTable.c.asset_key, AssetKeyTable.c.tags])
-                .where(AssetKeyTable.c.tags != None)
-                .where(
-                    db.or_(
-                        AssetKeyTable.c.wipe_timestamp == None,
-                        AssetKeyTable.c.last_materialization_timestamp
-                        > AssetKeyTable.c.wipe_timestamp,
-                    )
-                )
-            )
-            with self.index_connection() as conn:
-                rows = conn.execute(query).fetchall()
-                for asset_key, tags_json in rows:
-                    tags = seven.json.loads(tags_json)
-                    if tags:
-                        tags_by_asset_key[AssetKey.from_db_string(asset_key)] = tags
-
-        else:
-            query = db.select([AssetKeyTable.c.asset_key, AssetKeyTable.c.last_materialization])
-            with self.index_connection() as conn:
-                rows = conn.execute(query).fetchall()
-                for asset_key, json_str in rows:
-                    materialization = self._asset_materialization_from_json_column(json_str)
-                    if materialization and materialization.tags:
-                        tags_by_asset_key[AssetKey.from_db_string(asset_key)] = {
-                            k: v for k, v in materialization.tags.items()
-                        }
-
-        return tags_by_asset_key
-
-    def get_asset_tags(self, asset_key):
-        check.inst_param(asset_key, "asset_key", AssetKey)
-        if self.has_secondary_index(ASSET_KEY_INDEX_COLS):
-            query = (
-                db.select([AssetKeyTable.c.tags])
-                .where(AssetKeyTable.c.asset_key == asset_key.to_string())
-                .where(
-                    db.or_(
-                        AssetKeyTable.c.wipe_timestamp == None,
-                        AssetKeyTable.c.last_materialization_timestamp
-                        > AssetKeyTable.c.wipe_timestamp,
-                    )
-                )
-            )
-            with self.index_connection() as conn:
-                rows = conn.execute(query).fetchall()
-                if not rows or not rows[0] or not rows[0][0]:
-                    return {}
-
-                return seven.json.loads(rows[0][0])
-        else:
-            query = db.select([AssetKeyTable.c.last_materialization]).where(
-                AssetKeyTable.c.asset_key == asset_key.to_string()
-            )
-            with self.index_connection() as conn:
-                rows = conn.execute(query).fetchall()
-                if not rows or not rows[0] or not rows[0][0]:
-                    return {}
-
-                materialization = self._asset_materialization_from_json_column(rows[0][0])
-                return materialization.tags if materialization and materialization.tags else {}
-
     def wipe_asset(self, asset_key):
         check.inst_param(asset_key, "asset_key", AssetKey)
 
