@@ -2,13 +2,19 @@ import warnings
 from collections import namedtuple
 from datetime import datetime
 from enum import Enum
-from typing import NamedTuple
+from typing import Any, Dict, NamedTuple, Type
 
 from dagster import check
 from dagster.core.origin import PipelinePythonOrigin
 from dagster.core.storage.tags import PARENT_RUN_ID_TAG, ROOT_RUN_ID_TAG
 from dagster.core.utils import make_new_run_id
-from dagster.serdes import DefaultNamedTupleSerializer, whitelist_for_serdes
+from dagster.serdes import (
+    DefaultNamedTupleSerializer,
+    register_serdes_tuple_fallbacks,
+    unpack_inner_value,
+    whitelist_for_serdes,
+)
+from dagster.serdes.serdes import EnumSerializer, WhitelistMap, register_serdes_enum_fallbacks
 
 from .tags import (
     BACKFILL_ID_TAG,
@@ -20,8 +26,23 @@ from .tags import (
 )
 
 
-@whitelist_for_serdes
-class PipelineRunStatus(Enum):
+class DagsterRunStatusSerializer(EnumSerializer):
+    @classmethod
+    def value_from_storage_str(cls, storage_str: str, klass: Type) -> Enum:
+        return getattr(klass, storage_str)
+
+    @classmethod
+    def value_to_storage_str(
+        cls, value: Enum, whitelist_map: WhitelistMap, descent_path: str
+    ) -> str:
+        enum_value = value.value
+        # Store DagsterRunStatus with backcompat name PipelineRunStatus
+        backcompat_name = "PipelineRunStatus"
+        return ".".join([backcompat_name, enum_value])
+
+
+@whitelist_for_serdes(serializer=DagsterRunStatusSerializer)
+class DagsterRunStatus(Enum):
     """The status of pipeline execution."""
 
     QUEUED = "QUEUED"
@@ -35,6 +56,9 @@ class PipelineRunStatus(Enum):
     CANCELED = "CANCELED"
 
 
+PipelineRunStatus = DagsterRunStatus
+register_serdes_enum_fallbacks({"PipelineRunStatus": DagsterRunStatus})
+
 # These statuses that indicate a run may be using compute resources
 IN_PROGRESS_RUN_STATUSES = [
     PipelineRunStatus.STARTING,
@@ -90,11 +114,39 @@ class PipelineRunStatsSnapshot(
         )
 
 
-class PipelineRunSerializer(DefaultNamedTupleSerializer):
+class DagsterRunSerializer(DefaultNamedTupleSerializer):
     @classmethod
-    def value_from_storage_dict(cls, storage_dict, _klass):
+    def value_from_storage_dict(
+        cls,
+        storage_dict,
+        klass,
+        args_for_class,
+        whitelist_map,
+        descent_path,
+    ):
+        # unpack all stored fields
+        unpacked_dict = {
+            key: unpack_inner_value(value, whitelist_map, f"{descent_path}.{key}")
+            for key, value in storage_dict.items()
+        }
         # called by the serdes layer, delegates to helper method with expanded kwargs
-        return pipeline_run_from_storage(**storage_dict)
+        return pipeline_run_from_storage(**unpacked_dict)
+
+    @classmethod
+    def value_to_storage_dict(
+        cls,
+        value: NamedTuple,
+        whitelist_map: WhitelistMap,
+        descent_path: str,
+    ) -> Dict[str, Any]:
+        storage = super().value_to_storage_dict(
+            value,
+            whitelist_map,
+            descent_path,
+        )
+        # persist using legacy name PipelineRun
+        storage["__class__"] = "PipelineRun"
+        return storage
 
 
 def pipeline_run_from_storage(
@@ -184,7 +236,7 @@ def pipeline_run_from_storage(
             "Found unhandled arguments from stored PipelineRun: {args}".format(args=kwargs.keys())
         )
 
-    return PipelineRun(  # pylint: disable=redundant-keyword-arg
+    return DagsterRun(  # pylint: disable=redundant-keyword-arg
         pipeline_name=pipeline_name,
         run_id=run_id,
         run_config=run_config,
@@ -203,7 +255,6 @@ def pipeline_run_from_storage(
     )
 
 
-@whitelist_for_serdes(serializer=PipelineRunSerializer)
 class PipelineRun(
     namedtuple(
         "_PipelineRun",
@@ -254,7 +305,7 @@ class PipelineRun(
         # a list of solid queries provided by the user
         # possible to be None when only solids_to_execute is set by the user directly
         check.opt_list_param(solid_selection, "solid_selection", of_type=str)
-        check.opt_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
+        check.opt_nullable_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
 
         # Placing this with the other imports causes a cyclic import
         # https://github.com/dagster-io/dagster/issues/3181
@@ -268,10 +319,13 @@ class PipelineRun(
                 "external_pipeline_origin is required for queued runs",
             )
 
+        if run_id is None:
+            run_id = make_new_run_id()
+
         return super(PipelineRun, cls).__new__(
             cls,
             pipeline_name=check.opt_str_param(pipeline_name, "pipeline_name"),
-            run_id=check.opt_str_param(run_id, "run_id", default=make_new_run_id()),
+            run_id=check.str_param(run_id, "run_id"),
             run_config=check.opt_dict_param(run_config, "run_config", key_type=str),
             mode=check.opt_str_param(mode, "mode"),
             solid_selection=solid_selection,
@@ -363,6 +417,20 @@ class PipelineRun(
         return {PARTITION_NAME_TAG: partition.name, PARTITION_SET_TAG: partition_set.name}
 
 
+@whitelist_for_serdes(serializer=DagsterRunSerializer)
+class DagsterRun(PipelineRun):
+    """Serializable internal representation of a dagster run, as stored in a
+    :py:class:`~dagster.core.storage.runs.RunStorage`.
+
+    Subclasses PipelineRun for backcompat purposes. DagsterRun is the actual initialized class used throughout the system.
+    """
+
+
+# DagsterRun is serialized as PipelineRun so that it can be read by older (pre 0.13.x) version of
+# Dagster, but is read back in as a DagsterRun.
+register_serdes_tuple_fallbacks({"PipelineRun": DagsterRun})
+
+
 @whitelist_for_serdes
 class PipelineRunsFilter(
     namedtuple(
