@@ -17,7 +17,6 @@ import logging
 import os
 import sys
 import uuid
-import zlib
 from collections import namedtuple
 from functools import wraps
 from logging.handlers import RotatingFileHandler
@@ -38,20 +37,17 @@ TELEMETRY_STR = ".telemetry"
 INSTANCE_ID_STR = "instance_id"
 ENABLED_STR = "enabled"
 DAGSTER_HOME_FALLBACK = "~/.dagster"
-DAGSTER_TELEMETRY_URL = "http://telemetry.dagster.io/actions"
 MAX_BYTES = 10485760  # 10 MB = 10 * 1024 * 1024 bytes
 UPDATE_REPO_STATS = "update_repo_stats"
 START_DAGIT_WEBSERVER = "start_dagit_webserver"
 TELEMETRY_VERSION = "0.2"
 
-# start_TELEMETRY_WHITELISTED_FUNCTIONS
 TELEMETRY_WHITELISTED_FUNCTIONS = {
-    "_launch_scheduled_executions",
     "_logged_execute_pipeline",
     "execute_execute_command",
     "execute_launch_command",
+    "_daemon_run_command",
 }
-# end_TELEMETRY_WHITELISTED_FUNCTIONS
 
 
 def telemetry_wrapper(f):
@@ -327,7 +323,7 @@ def _set_telemetry_instance_id():
         with open(telemetry_id_path, "w") as telemetry_id_file:
             yaml.dump({INSTANCE_ID_STR: instance_id}, telemetry_id_file, default_flow_style=False)
         return instance_id
-    except Exception:  # pylint: disable=broad-except
+    except Exception:
         return "<<unable_to_write_instance_id>>"
 
 
@@ -406,14 +402,16 @@ def log_repo_stats(instance, source, pipeline=None, repo=None):
 
 
 def log_workspace_stats(instance, workspace_process_context):
-    from dagster.cli.workspace import WorkspaceProcessContext
+    from dagster.core.workspace import IWorkspaceProcessContext
 
     check.inst_param(instance, "instance", DagsterInstance)
     check.inst_param(
-        workspace_process_context, "workspace_process_context", WorkspaceProcessContext
+        workspace_process_context, "workspace_process_context", IWorkspaceProcessContext
     )
 
-    for repo_location in workspace_process_context.repository_locations:
+    request_context = workspace_process_context.create_request_context()
+
+    for repo_location in request_context.repository_locations:
         for external_repo in repo_location.get_repositories().values():
             log_external_repo_stats(instance, source="dagit", external_repo=external_repo)
 
@@ -464,108 +462,3 @@ SLACK_PROMPT = """
 """ % {
     "welcome": click.style("Welcome to Dagster!", bold=True)
 }
-
-
-def is_running_in_test():
-    return (
-        os.getenv("BUILDKITE") is not None
-        or os.getenv("TF_BUILD") is not None
-        or os.getenv("DAGSTER_DISABLE_TELEMETRY") is not None
-    )
-
-
-def upload_logs(stop_event, raise_errors=False):
-    """Upload logs to telemetry server every hour, or when log directory size is > 10MB"""
-
-    # We add a sanity check to ensure that no logs are uploaded in our
-    # buildkite/azure testing pipelines. The check is present at upload to
-    # allow for testing of logs being correctly written.
-    if is_running_in_test():
-        return
-
-    try:
-        last_run = datetime.datetime.now() - datetime.timedelta(minutes=120)
-        dagster_log_dir = get_dir_from_dagster_home("logs")
-        dagster_log_queue_dir = get_dir_from_dagster_home(".logs_queue")
-        in_progress = False
-        while not stop_event.is_set():
-            log_size = 0
-            if os.path.isdir(dagster_log_dir):
-                log_size = sum(
-                    os.path.getsize(os.path.join(dagster_log_dir, f))
-                    for f in os.listdir(dagster_log_dir)
-                    if os.path.isfile(os.path.join(dagster_log_dir, f))
-                )
-
-            log_queue_size = 0
-            if os.path.isdir(dagster_log_queue_dir):
-                log_queue_size = sum(
-                    os.path.getsize(os.path.join(dagster_log_queue_dir, f))
-                    for f in os.listdir(dagster_log_queue_dir)
-                    if os.path.isfile(os.path.join(dagster_log_queue_dir, f))
-                )
-
-            if log_size == 0 and log_queue_size == 0:
-                return
-
-            if not in_progress and (
-                datetime.datetime.now() - last_run > datetime.timedelta(minutes=60)
-                or log_size >= MAX_BYTES
-                or log_queue_size >= MAX_BYTES
-            ):
-                in_progress = True  # Prevent concurrent _upload_logs invocations
-                last_run = datetime.datetime.now()
-                dagster_log_dir = get_dir_from_dagster_home("logs")
-                dagster_log_queue_dir = get_dir_from_dagster_home(".logs_queue")
-                _upload_logs(
-                    dagster_log_dir, log_size, dagster_log_queue_dir, raise_errors=raise_errors
-                )
-                in_progress = False
-
-            stop_event.wait(600)  # Sleep for 10 minutes
-    except Exception:  # pylint: disable=broad-except
-        if raise_errors:
-            raise
-
-
-def _upload_logs(dagster_log_dir, log_size, dagster_log_queue_dir, raise_errors):
-    """Send POST request to telemetry server with the contents of $DAGSTER_HOME/logs/ directory """
-
-    try:
-        # lazy import for perf
-        import requests
-
-        if log_size > 0:
-            # Delete contents of dagster_log_queue_dir so that new logs can be copied over
-            for f in os.listdir(dagster_log_queue_dir):
-                # Todo: there is probably a way to try to upload these logs without introducing
-                # too much complexity...
-                os.remove(os.path.join(dagster_log_queue_dir, f))
-
-            os.rmdir(dagster_log_queue_dir)
-
-            os.rename(dagster_log_dir, dagster_log_queue_dir)
-
-        for curr_path in os.listdir(dagster_log_queue_dir):
-            curr_full_path = os.path.join(dagster_log_queue_dir, curr_path)
-            retry_num = 0
-            max_retries = 3
-            success = False
-
-            while not success and retry_num <= max_retries:
-                with open(curr_full_path, "rb") as curr_file:
-                    byte = curr_file.read()
-
-                    data = zlib.compress(byte, zlib.Z_BEST_COMPRESSION)
-                    headers = {"content-encoding": "gzip"}
-                    r = requests.post(DAGSTER_TELEMETRY_URL, data=data, headers=headers)
-                    if r.status_code == 200:
-                        success = True
-                    retry_num += 1
-
-            if success:
-                os.remove(curr_full_path)
-
-    except Exception:  # pylint: disable=broad-except
-        if raise_errors:
-            raise
