@@ -7,10 +7,10 @@ from contextlib import contextmanager
 import grpc
 from dagster import check, seven
 from dagster.core.events import EngineEventData
-from dagster.core.host_representation import ExternalRepositoryOrigin
+from dagster.core.host_representation.origin import ExternalRepositoryOrigin
 from dagster.core.instance import DagsterInstance
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
-from dagster.serdes import deserialize_json_to_dagster_namedtuple, serialize_dagster_namedtuple
+from dagster.serdes import serialize_dagster_namedtuple
 from dagster.utils.error import serializable_error_info_from_exc_info
 from grpc_health.v1 import health_pb2
 from grpc_health.v1.health_pb2_grpc import HealthStub
@@ -32,6 +32,17 @@ from .types import (
 
 CLIENT_HEARTBEAT_INTERVAL = 1
 
+DEFAULT_GRPC_TIMEOUT = 60
+
+
+def _max_rx_bytes():
+    env_set = os.getenv("DAGSTER_GRPC_MAX_RX_BYTES")
+    if env_set:
+        return int(env_set)
+
+    # default 50 MB
+    return 50 * (10 ** 6)
+
 
 def client_heartbeat_thread(client, shutdown_event):
     while True:
@@ -74,24 +85,36 @@ class DagsterGrpcClient:
 
     @contextmanager
     def _channel(self):
+        options = [
+            ("grpc.max_receive_message_length", _max_rx_bytes()),
+        ]
         with (
-            grpc.secure_channel(self._server_address, self._ssl_creds)
+            grpc.secure_channel(
+                self._server_address,
+                self._ssl_creds,
+                options=options,
+                compression=grpc.Compression.Gzip,
+            )
             if self._use_ssl
-            else grpc.insecure_channel(self._server_address)
+            else grpc.insecure_channel(
+                self._server_address,
+                options=options,
+                compression=grpc.Compression.Gzip,
+            )
         ) as channel:
             yield channel
 
-    def _query(self, method, request_type, timeout=None, **kwargs):
+    def _query(self, method, request_type, timeout=DEFAULT_GRPC_TIMEOUT, **kwargs):
         with self._channel() as channel:
             stub = DagsterApiStub(channel)
             response = getattr(stub, method)(request_type(**kwargs), timeout=timeout)
         # TODO need error handling here
         return response
 
-    def _streaming_query(self, method, request_type, **kwargs):
+    def _streaming_query(self, method, request_type, timeout=DEFAULT_GRPC_TIMEOUT, **kwargs):
         with self._channel() as channel:
             stub = DagsterApiStub(channel)
-            response_stream = getattr(stub, method)(request_type(**kwargs))
+            response_stream = getattr(stub, method)(request_type(**kwargs), timeout=timeout)
             yield from response_stream
 
     def ping(self, echo):
@@ -134,15 +157,11 @@ class DagsterGrpcClient:
                 execution_plan_snapshot_args
             ),
         )
-        return deserialize_json_to_dagster_namedtuple(res.serialized_execution_plan_snapshot)
+        return res.serialized_execution_plan_snapshot
 
     def list_repositories(self):
-
         res = self._query("ListRepositories", api_pb2.ListRepositoriesRequest)
-
-        return deserialize_json_to_dagster_namedtuple(
-            res.serialized_list_repositories_response_or_error
-        )
+        return res.serialized_list_repositories_response_or_error
 
     def external_partition_names(self, partition_names_args):
         check.inst_param(partition_names_args, "partition_names_args", PartitionNamesArgs)
@@ -153,9 +172,7 @@ class DagsterGrpcClient:
             serialized_partition_names_args=serialize_dagster_namedtuple(partition_names_args),
         )
 
-        return deserialize_json_to_dagster_namedtuple(
-            res.serialized_external_partition_names_or_external_partition_execution_error
-        )
+        return res.serialized_external_partition_names_or_external_partition_execution_error
 
     def external_partition_config(self, partition_args):
         check.inst_param(partition_args, "partition_args", PartitionArgs)
@@ -166,9 +183,7 @@ class DagsterGrpcClient:
             serialized_partition_args=serialize_dagster_namedtuple(partition_args),
         )
 
-        return deserialize_json_to_dagster_namedtuple(
-            res.serialized_external_partition_config_or_external_partition_execution_error
-        )
+        return res.serialized_external_partition_config_or_external_partition_execution_error
 
     def external_partition_tags(self, partition_args):
         check.inst_param(partition_args, "partition_args", PartitionArgs)
@@ -179,9 +194,7 @@ class DagsterGrpcClient:
             serialized_partition_args=serialize_dagster_namedtuple(partition_args),
         )
 
-        return deserialize_json_to_dagster_namedtuple(
-            res.serialized_external_partition_tags_or_external_partition_execution_error
-        )
+        return res.serialized_external_partition_tags_or_external_partition_execution_error
 
     def external_partition_set_execution_params(self, partition_set_execution_param_args):
         check.inst_param(
@@ -190,17 +203,17 @@ class DagsterGrpcClient:
             PartitionSetExecutionParamArgs,
         )
 
-        res = self._query(
-            "ExternalPartitionSetExecutionParams",
-            api_pb2.ExternalPartitionSetExecutionParamsRequest,
-            serialized_partition_set_execution_param_args=serialize_dagster_namedtuple(
-                partition_set_execution_param_args
-            ),
+        chunks = list(
+            self._streaming_query(
+                "ExternalPartitionSetExecutionParams",
+                api_pb2.ExternalPartitionSetExecutionParamsRequest,
+                serialized_partition_set_execution_param_args=serialize_dagster_namedtuple(
+                    partition_set_execution_param_args
+                ),
+            )
         )
 
-        return deserialize_json_to_dagster_namedtuple(
-            res.serialized_external_partition_set_execution_param_data_or_external_partition_execution_error
-        )
+        return "".join([chunk.serialized_chunk for chunk in chunks])
 
     def external_pipeline_subset(self, pipeline_subset_snapshot_args):
         check.inst_param(
@@ -217,9 +230,7 @@ class DagsterGrpcClient:
             ),
         )
 
-        return deserialize_json_to_dagster_namedtuple(
-            res.serialized_external_pipeline_subset_result
-        )
+        return res.serialized_external_pipeline_subset_result
 
     def external_repository(self, external_repository_origin):
         check.inst_param(
@@ -237,7 +248,7 @@ class DagsterGrpcClient:
             ),
         )
 
-        return deserialize_json_to_dagster_namedtuple(res.serialized_external_repository_data)
+        return res.serialized_external_repository_data
 
     def streaming_external_repository(self, external_repository_origin):
         for res in self._streaming_query(
@@ -260,40 +271,50 @@ class DagsterGrpcClient:
             ExternalScheduleExecutionArgs,
         )
 
-        res = self._query(
-            "ExternalScheduleExecution",
-            api_pb2.ExternalScheduleExecutionRequest,
-            serialized_external_schedule_execution_args=serialize_dagster_namedtuple(
-                external_schedule_execution_args
-            ),
+        chunks = list(
+            self._streaming_query(
+                "ExternalScheduleExecution",
+                api_pb2.ExternalScheduleExecutionRequest,
+                serialized_external_schedule_execution_args=serialize_dagster_namedtuple(
+                    external_schedule_execution_args
+                ),
+            )
         )
 
-        return deserialize_json_to_dagster_namedtuple(
-            res.serialized_external_schedule_execution_data_or_external_schedule_execution_error
-        )
+        return "".join([chunk.serialized_chunk for chunk in chunks])
 
-    def external_sensor_execution(self, sensor_execution_args):
+    def external_sensor_execution(self, sensor_execution_args, timeout=DEFAULT_GRPC_TIMEOUT):
         check.inst_param(
             sensor_execution_args,
             "sensor_execution_args",
             SensorExecutionArgs,
         )
 
-        res = self._query(
-            "ExternalSensorExecution",
-            api_pb2.ExternalSensorExecutionRequest,
-            serialized_external_sensor_execution_args=serialize_dagster_namedtuple(
-                sensor_execution_args
-            ),
+        chunks = list(
+            self._streaming_query(
+                "ExternalSensorExecution",
+                api_pb2.ExternalSensorExecutionRequest,
+                timeout=timeout,
+                serialized_external_sensor_execution_args=serialize_dagster_namedtuple(
+                    sensor_execution_args
+                ),
+            )
         )
 
-        return deserialize_json_to_dagster_namedtuple(
-            res.serialized_external_sensor_execution_data_or_external_sensor_execution_error
+        return "".join([chunk.serialized_chunk for chunk in chunks])
+
+    def external_notebook_data(self, notebook_path):
+        check.str_param(notebook_path, "notebook_path")
+        res = self._query(
+            "ExternalNotebookData",
+            api_pb2.ExternalNotebookDataRequest,
+            notebook_path=notebook_path,
         )
+        return res.content
 
     def shutdown_server(self, timeout=15):
         res = self._query("ShutdownServer", api_pb2.Empty, timeout=timeout)
-        return deserialize_json_to_dagster_namedtuple(res.serialized_shutdown_server_result)
+        return res.serialized_shutdown_server_result
 
     def cancel_execution(self, cancel_execution_request):
         check.inst_param(
@@ -310,7 +331,7 @@ class DagsterGrpcClient:
             ),
         )
 
-        return deserialize_json_to_dagster_namedtuple(res.serialized_cancel_execution_result)
+        return res.serialized_cancel_execution_result
 
     def can_cancel_execution(self, can_cancel_execution_request, timeout=None):
         check.inst_param(
@@ -328,7 +349,7 @@ class DagsterGrpcClient:
             ),
         )
 
-        return deserialize_json_to_dagster_namedtuple(res.serialized_can_cancel_execution_result)
+        return res.serialized_can_cancel_execution_result
 
     def start_run(self, execute_run_args):
         check.inst_param(execute_run_args, "execute_run_args", ExecuteExternalPipelineArgs)
@@ -340,9 +361,9 @@ class DagsterGrpcClient:
                     api_pb2.StartRunRequest,
                     serialized_execute_run_args=serialize_dagster_namedtuple(execute_run_args),
                 )
-                return deserialize_json_to_dagster_namedtuple(res.serialized_start_run_result)
+                return res.serialized_start_run_result
 
-            except Exception:  # pylint: disable=bare-except
+            except Exception:
                 pipeline_run = instance.get_run_by_id(execute_run_args.pipeline_run_id)
                 instance.report_engine_event(
                     message="Unexpected error in IPC client",
@@ -355,11 +376,11 @@ class DagsterGrpcClient:
 
     def get_current_image(self):
         res = self._query("GetCurrentImage", api_pb2.Empty)
-        return deserialize_json_to_dagster_namedtuple(res.serialized_current_image)
+        return res.serialized_current_image
 
     def health_check_query(self):
         try:
-            with grpc.insecure_channel(self._server_address) as channel:
+            with self._channel() as channel:
                 response = HealthStub(channel).Check(
                     health_pb2.HealthCheckRequest(service="DagsterApi")
                 )
@@ -419,6 +440,5 @@ def ephemeral_grpc_api_client(
         force_port=force_port,
         max_retries=max_retries,
         max_workers=max_workers,
-        lazy_load_user_code=True,
     ).create_ephemeral_client() as client:
         yield client
