@@ -3,26 +3,27 @@
 import os
 import sys
 
+import pendulum
 from dagster import check
-from dagster.core.definitions import ScheduleExecutionContext, TriggeredExecutionContext
+from dagster.core.definitions import ScheduleEvaluationContext
 from dagster.core.definitions.reconstructable import (
     ReconstructablePipeline,
     ReconstructableRepository,
 )
+from dagster.core.definitions.sensor_definition import SensorEvaluationContext
 from dagster.core.errors import (
+    DagsterExecutionInterruptedError,
     DagsterInvalidSubsetError,
-    DagsterSubprocessError,
+    DagsterRunNotFoundError,
     PartitionExecutionError,
     ScheduleExecutionError,
-    TriggeredExecutionError,
+    SensorExecutionError,
     user_code_error_boundary,
 )
 from dagster.core.events import EngineEventData
 from dagster.core.execution.api import create_execution_plan, execute_run_iterator
 from dagster.core.host_representation import external_pipeline_data_from_def
 from dagster.core.host_representation.external_data import (
-    ExternalExecutionParamsData,
-    ExternalExecutionParamsErrorData,
     ExternalPartitionConfigData,
     ExternalPartitionExecutionErrorData,
     ExternalPartitionExecutionParamData,
@@ -30,8 +31,8 @@ from dagster.core.host_representation.external_data import (
     ExternalPartitionSetExecutionParamData,
     ExternalPartitionTagsData,
     ExternalPipelineSubsetResult,
-    ExternalScheduleExecutionData,
     ExternalScheduleExecutionErrorData,
+    ExternalSensorExecutionErrorData,
 )
 from dagster.core.instance import DagsterInstance
 from dagster.core.snap.execution_plan_snapshot import (
@@ -39,21 +40,15 @@ from dagster.core.snap.execution_plan_snapshot import (
     snapshot_from_execution_plan,
 )
 from dagster.core.storage.pipeline_run import PipelineRun
-from dagster.grpc.types import (
-    ExecutionPlanSnapshotArgs,
-    ExternalScheduleExecutionArgs,
-    ExternalTriggeredExecutionArgs,
-    PartitionArgs,
-    PartitionNamesArgs,
-    ScheduleExecutionDataMode,
-)
+from dagster.grpc.types import ExecutionPlanSnapshotArgs
 from dagster.serdes import deserialize_json_to_dagster_namedtuple
 from dagster.serdes.ipc import IPCErrorMessage
+from dagster.seven import nullcontext
 from dagster.utils import start_termination_thread
 from dagster.utils.error import serializable_error_info_from_exc_info
-from dagster.utils.hosted_user_process import recon_repository_from_origin
+from dagster.utils.interrupts import capture_interrupts
 
-from .types import ExecuteRunArgs, ExternalScheduleExecutionArgs, PartitionSetExecutionParamArgs
+from .types import ExecuteExternalPipelineArgs
 
 
 class RunInSubprocessComplete:
@@ -64,33 +59,48 @@ class StartRunInSubprocessSuccessful:
     """Sentinel passed over multiprocessing Queue when launch is successful in subprocess."""
 
 
-def _core_execute_run(recon_pipeline, pipeline_run, instance):
+def _report_run_failed_if_not_finished(instance, pipeline_run_id):
+    check.inst_param(instance, "instance", DagsterInstance)
+    pipeline_run = instance.get_run_by_id(pipeline_run_id)
+    if pipeline_run and (not pipeline_run.is_finished):
+        yield instance.report_run_failed(pipeline_run)
+
+
+def core_execute_run(recon_pipeline, pipeline_run, instance, resume_from_failure=False):
     check.inst_param(recon_pipeline, "recon_pipeline", ReconstructablePipeline)
     check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
     check.inst_param(instance, "instance", DagsterInstance)
 
+    # try to load the pipeline definition early
     try:
-        for event in execute_run_iterator(recon_pipeline, pipeline_run, instance):
-            yield event
-    except DagsterSubprocessError as err:
-        if not all(
-            [err_info.cls_name == "KeyboardInterrupt" for err_info in err.subprocess_error_infos]
-        ):
-            yield instance.report_engine_event(
-                "An exception was thrown during execution that is likely a framework error, "
-                "rather than an error in user code.",
-                pipeline_run,
-                EngineEventData.engine_error(serializable_error_info_from_exc_info(sys.exc_info())),
-            )
-            instance.report_run_failed(pipeline_run)
-    except Exception:  # pylint: disable=broad-except
+        recon_pipeline.get_definition()
+    except Exception:
+        yield instance.report_engine_event(
+            "Could not load pipeline definition.",
+            pipeline_run,
+            EngineEventData.engine_error(serializable_error_info_from_exc_info(sys.exc_info())),
+        )
+        yield from _report_run_failed_if_not_finished(instance, pipeline_run.run_id)
+        return
+
+    try:
+        yield from execute_run_iterator(
+            recon_pipeline, pipeline_run, instance, resume_from_failure=resume_from_failure
+        )
+    except (KeyboardInterrupt, DagsterExecutionInterruptedError):
+        yield from _report_run_failed_if_not_finished(instance, pipeline_run.run_id)
+        yield instance.report_engine_event(
+            message="Run execution terminated by interrupt",
+            pipeline_run=pipeline_run,
+        )
+    except Exception:
         yield instance.report_engine_event(
             "An exception was thrown during execution that is likely a framework error, "
             "rather than an error in user code.",
             pipeline_run,
             EngineEventData.engine_error(serializable_error_info_from_exc_info(sys.exc_info())),
         )
-        instance.report_run_failed(pipeline_run)
+        yield from _report_run_failed_if_not_finished(instance, pipeline_run.run_id)
 
 
 def _run_in_subprocess(
@@ -104,29 +114,42 @@ def _run_in_subprocess(
     start_termination_thread(termination_event)
     try:
         execute_run_args = deserialize_json_to_dagster_namedtuple(serialized_execute_run_args)
-        check.inst_param(execute_run_args, "execute_run_args", ExecuteRunArgs)
+        check.inst_param(execute_run_args, "execute_run_args", ExecuteExternalPipelineArgs)
 
-        instance = DagsterInstance.from_ref(execute_run_args.instance_ref)
-        pipeline_run = instance.get_run_by_id(execute_run_args.pipeline_run_id)
+        with (
+            DagsterInstance.from_ref(execute_run_args.instance_ref)
+            if execute_run_args.instance_ref
+            else nullcontext()
+        ) as instance:
+            pipeline_run = instance.get_run_by_id(execute_run_args.pipeline_run_id)
 
-        pid = os.getpid()
+            if not pipeline_run:
+                raise DagsterRunNotFoundError(
+                    "gRPC server could not load run {run_id} in order to execute it. Make sure that the gRPC server has access to your run storage.".format(
+                        run_id=execute_run_args.pipeline_run_id
+                    ),
+                    invalid_run_id=execute_run_args.pipeline_run_id,
+                )
 
-    except:  # pylint: disable=bare-except
+            pid = os.getpid()
+
+    except:
+        serializable_error_info = serializable_error_info_from_exc_info(sys.exc_info())
         event = IPCErrorMessage(
-            serializable_error_info=serializable_error_info_from_exc_info(sys.exc_info()),
-            message="Error during RPC setup for ExecuteRun",
+            serializable_error_info=serializable_error_info,
+            message="Error during RPC setup for executing run: {message}".format(
+                message=serializable_error_info.message
+            ),
         )
         subprocess_status_handler(event)
         subprocess_status_handler(RunInSubprocessComplete())
-        if instance:
-            instance.dispose()
         return
 
     subprocess_status_handler(StartRunInSubprocessSuccessful())
 
     run_event_handler(
         instance.report_engine_event(
-            "Started process for pipeline (pid: {pid}).".format(pid=pid),
+            "Started process for run (pid: {pid}).".format(pid=pid),
             pipeline_run,
             EngineEventData.in_process(pid, marker_end="cli_api_subprocess_init"),
         )
@@ -136,15 +159,8 @@ def _run_in_subprocess(
     # https://amir.rachum.com/blog/2017/03/03/generator-cleanup/
     closed = False
     try:
-        for event in _core_execute_run(recon_pipeline, pipeline_run, instance):
+        for event in core_execute_run(recon_pipeline, pipeline_run, instance):
             run_event_handler(event)
-    except KeyboardInterrupt:
-        run_event_handler(
-            instance.report_engine_event(
-                message="Pipeline execution terminated by interrupt", pipeline_run=pipeline_run,
-            )
-        )
-        raise
     except GeneratorExit:
         closed = True
         raise
@@ -152,35 +168,25 @@ def _run_in_subprocess(
         if not closed:
             run_event_handler(
                 instance.report_engine_event(
-                    "Process for pipeline exited (pid: {pid}).".format(pid=pid), pipeline_run,
+                    "Process for run exited (pid: {pid}).".format(pid=pid),
+                    pipeline_run,
                 )
             )
         subprocess_status_handler(RunInSubprocessComplete())
         instance.dispose()
 
 
-def execute_run_in_subprocess(
-    serialized_execute_run_args, recon_pipeline, event_queue, termination_event
-):
-    _run_in_subprocess(
-        serialized_execute_run_args,
-        recon_pipeline,
-        termination_event,
-        subprocess_status_handler=event_queue.put,
-        run_event_handler=event_queue.put,
-    )
-
-
 def start_run_in_subprocess(
     serialized_execute_run_args, recon_pipeline, event_queue, termination_event
 ):
-    _run_in_subprocess(
-        serialized_execute_run_args,
-        recon_pipeline,
-        termination_event,
-        subprocess_status_handler=event_queue.put,
-        run_event_handler=lambda x: None,
-    )
+    with capture_interrupts():
+        _run_in_subprocess(
+            serialized_execute_run_args,
+            recon_pipeline,
+            termination_event,
+            subprocess_status_handler=event_queue.put,
+            run_event_handler=lambda x: None,
+        )
 
 
 def get_external_pipeline_subset_result(recon_pipeline, solid_selection):
@@ -201,117 +207,79 @@ def get_external_pipeline_subset_result(recon_pipeline, solid_selection):
     return ExternalPipelineSubsetResult(success=True, external_pipeline_data=external_pipeline_data)
 
 
-def get_external_schedule_execution(recon_repo, external_schedule_execution_args):
+def get_external_schedule_execution(
+    recon_repo,
+    instance_ref,
+    schedule_name,
+    scheduled_execution_timestamp,
+    scheduled_execution_timezone,
+):
     check.inst_param(
-        recon_repo, "recon_repo", ReconstructableRepository,
+        recon_repo,
+        "recon_repo",
+        ReconstructableRepository,
     )
-    check.inst_param(
-        external_schedule_execution_args,
-        "external_schedule_execution_args",
-        ExternalScheduleExecutionArgs,
-    )
-
     definition = recon_repo.get_definition()
-    schedule_def = definition.get_schedule_def(external_schedule_execution_args.schedule_name)
-    with DagsterInstance.from_ref(external_schedule_execution_args.instance_ref) as instance:
-        schedule_context = ScheduleExecutionContext(instance)
-        schedule_execution_data_mode = external_schedule_execution_args.schedule_execution_data_mode
+    schedule_def = definition.get_schedule_def(schedule_name)
+    scheduled_execution_time = (
+        pendulum.from_timestamp(
+            scheduled_execution_timestamp,
+            tz=scheduled_execution_timezone,
+        )
+        if scheduled_execution_timestamp
+        else None
+    )
 
+    with ScheduleEvaluationContext(instance_ref, scheduled_execution_time) as schedule_context:
         try:
             with user_code_error_boundary(
                 ScheduleExecutionError,
-                lambda: "Error occurred during the execution of should_execute for schedule "
+                lambda: "Error occurred during the execution function for schedule "
                 "{schedule_name}".format(schedule_name=schedule_def.name),
             ):
-                should_execute = None
-                if (
-                    schedule_execution_data_mode
-                    == ScheduleExecutionDataMode.LAUNCH_SCHEDULED_EXECUTION
-                ):
-                    should_execute = schedule_def.should_execute(schedule_context)
-                    if not should_execute:
-                        return ExternalScheduleExecutionData(
-                            should_execute=False, run_config=None, tags=None
-                        )
-
-            with user_code_error_boundary(
-                ScheduleExecutionError,
-                lambda: "Error occurred during the execution of run_config_fn for schedule "
-                "{schedule_name}".format(schedule_name=schedule_def.name),
-            ):
-                run_config = schedule_def.get_run_config(schedule_context)
-
-            with user_code_error_boundary(
-                ScheduleExecutionError,
-                lambda: "Error occurred during the execution of tags_fn for schedule "
-                "{schedule_name}".format(schedule_name=schedule_def.name),
-            ):
-                tags = schedule_def.get_tags(schedule_context)
-
-            return ExternalScheduleExecutionData(
-                run_config=run_config, tags=tags, should_execute=should_execute
-            )
+                return schedule_def.evaluate_tick(schedule_context)
         except ScheduleExecutionError:
             return ExternalScheduleExecutionErrorData(
                 serializable_error_info_from_exc_info(sys.exc_info())
             )
 
 
-def get_external_triggered_execution_params(recon_repo, external_triggered_execution_args):
-    check.inst_param(recon_repo, "recon_repo", ReconstructableRepository)
+def get_external_sensor_execution(
+    recon_repo, instance_ref, sensor_name, last_completion_timestamp, last_run_key, cursor
+):
     check.inst_param(
-        external_triggered_execution_args,
-        "external_triggered_execution_args",
-        ExternalTriggeredExecutionArgs,
+        recon_repo,
+        "recon_repo",
+        ReconstructableRepository,
     )
-    definition = recon_repo.get_definition()
-    triggered_execution_def = definition.get_triggered_execution_def(
-        external_triggered_execution_args.trigger_name
-    )
-    with DagsterInstance.from_ref(external_triggered_execution_args.instance_ref) as instance:
-        context = TriggeredExecutionContext(instance)
 
+    definition = recon_repo.get_definition()
+    sensor_def = definition.get_sensor_def(sensor_name)
+
+    with SensorEvaluationContext(
+        instance_ref,
+        last_completion_time=last_completion_timestamp,
+        last_run_key=last_run_key,
+        cursor=cursor,
+        repository_name=recon_repo.get_definition().name,
+    ) as sensor_context:
         try:
             with user_code_error_boundary(
-                TriggeredExecutionError,
-                lambda: "Error occured during the execution of should_execute_fn for triggered "
-                "execution {name}".format(name=triggered_execution_def.name),
+                SensorExecutionError,
+                lambda: "Error occurred during the execution of evaluation_fn for sensor "
+                "{sensor_name}".format(sensor_name=sensor_def.name),
             ):
-                should_execute = triggered_execution_def.should_execute(context)
-                if not should_execute:
-                    return ExternalExecutionParamsData(
-                        run_config=None, tags=None, should_execute=False
-                    )
-
-            with user_code_error_boundary(
-                TriggeredExecutionError,
-                lambda: "Error occured during the execution of run_config_fn for triggered "
-                "execution {name}".format(name=triggered_execution_def.name),
-            ):
-                run_config = triggered_execution_def.get_run_config(context)
-
-            with user_code_error_boundary(
-                TriggeredExecutionError,
-                lambda: "Error occured during the execution of tags_fn for triggered "
-                "execution {name}".format(name=triggered_execution_def.name),
-            ):
-                tags = triggered_execution_def.get_tags(context)
-
-            return ExternalExecutionParamsData(
-                run_config=run_config, tags=tags, should_execute=should_execute
-            )
-        except TriggeredExecutionError:
-            return ExternalExecutionParamsErrorData(
+                return sensor_def.evaluate_tick(sensor_context)
+        except SensorExecutionError:
+            return ExternalSensorExecutionErrorData(
                 serializable_error_info_from_exc_info(sys.exc_info())
             )
 
 
-def get_partition_config(args):
-    check.inst_param(args, "args", PartitionArgs)
-    recon_repo = recon_repository_from_origin(args.repository_origin)
+def get_partition_config(recon_repo, partition_set_name, partition_name):
     definition = recon_repo.get_definition()
-    partition_set_def = definition.get_partition_set_def(args.partition_set_name)
-    partition = partition_set_def.get_partition(args.partition_name)
+    partition_set_def = definition.get_partition_set_def(partition_set_name)
+    partition = partition_set_def.get_partition(partition_name)
     try:
         with user_code_error_boundary(
             PartitionExecutionError,
@@ -328,16 +296,21 @@ def get_partition_config(args):
         )
 
 
-def get_partition_names(args):
-    check.inst_param(args, "args", PartitionNamesArgs)
-    recon_repo = recon_repository_from_origin(args.repository_origin)
+def _get_target_for_partition_execution_error(partition_set_def):
+    if partition_set_def.job_name:
+        return f"partitioned config on job '{partition_set_def.job_name}'"
+    else:
+        return f"partition set '{partition_set_def.name}'"
+
+
+def get_partition_names(recon_repo, partition_set_name):
     definition = recon_repo.get_definition()
-    partition_set_def = definition.get_partition_set_def(args.partition_set_name)
+    partition_set_def = definition.get_partition_set_def(partition_set_name)
     try:
         with user_code_error_boundary(
             PartitionExecutionError,
-            lambda: "Error occurred during the execution of the partition generation function for "
-            "partition set {partition_set_name}".format(partition_set_name=partition_set_def.name),
+            lambda: f"Error occurred during the execution of the partition generation function for "
+            f"{_get_target_for_partition_execution_error(partition_set_def)}",
         ):
             return ExternalPartitionNamesData(
                 partition_names=partition_set_def.get_partition_names()
@@ -348,17 +321,15 @@ def get_partition_names(args):
         )
 
 
-def get_partition_tags(args):
-    check.inst_param(args, "args", PartitionArgs)
-    recon_repo = recon_repository_from_origin(args.repository_origin)
+def get_partition_tags(recon_repo, partition_set_name, partition_name):
     definition = recon_repo.get_definition()
-    partition_set_def = definition.get_partition_set_def(args.partition_set_name)
-    partition = partition_set_def.get_partition(args.partition_name)
+    partition_set_def = definition.get_partition_set_def(partition_set_name)
+    partition = partition_set_def.get_partition(partition_name)
     try:
         with user_code_error_boundary(
             PartitionExecutionError,
             lambda: "Error occurred during the evaluation of the `tags_for_partition` function for "
-            "partition set {partition_set_name}".format(partition_set_name=partition_set_def.name),
+            f"{_get_target_for_partition_execution_error(partition_set_def)}",
         ):
             tags = partition_set_def.tags_for_partition(partition)
             return ExternalPartitionTagsData(name=partition.name, tags=tags)
@@ -385,51 +356,51 @@ def get_external_execution_plan_snapshot(recon_pipeline, args):
                 run_config=args.run_config,
                 mode=args.mode,
                 step_keys_to_execute=args.step_keys_to_execute,
+                known_state=args.known_state,
+                instance_ref=args.instance_ref,
             ),
             args.pipeline_snapshot_id,
         )
-    except:  # pylint: disable=bare-except
+    except:
         return ExecutionPlanSnapshotErrorData(
             error=serializable_error_info_from_exc_info(sys.exc_info())
         )
 
 
-def get_partition_set_execution_param_data(args):
-    check.inst_param(args, "args", PartitionSetExecutionParamArgs)
-    recon_repo = recon_repository_from_origin(args.repository_origin)
+def get_partition_set_execution_param_data(recon_repo, partition_set_name, partition_names):
     repo_definition = recon_repo.get_definition()
-    partition_set_def = repo_definition.get_partition_set_def(args.partition_set_name)
+    partition_set_def = repo_definition.get_partition_set_def(partition_set_name)
     try:
         with user_code_error_boundary(
             PartitionExecutionError,
-            lambda: "Error occurred during the partition generation for partition set "
-            "{partition_set_name}".format(partition_set_name=partition_set_def.name),
+            lambda: "Error occurred during the partition generation for "
+            f"{_get_target_for_partition_execution_error(partition_set_def)}",
         ):
             all_partitions = partition_set_def.get_partitions()
         partitions = [
-            partition for partition in all_partitions if partition.name in args.partition_names
+            partition for partition in all_partitions if partition.name in partition_names
         ]
 
         partition_data = []
         for partition in partitions:
 
-            def _error_message_fn(partition_set_name, partition_name):
+            def _error_message_fn(partition_name):
                 return lambda: (
                     "Error occurred during the partition config and tag generation for "
-                    "partition set {partition_set_name}::{partition_name}".format(
-                        partition_set_name=partition_set_name, partition_name=partition_name
-                    )
+                    f"'{partition_name}' in {_get_target_for_partition_execution_error(partition_set_def)}"
                 )
 
             with user_code_error_boundary(
-                PartitionExecutionError, _error_message_fn(partition_set_def.name, partition.name)
+                PartitionExecutionError, _error_message_fn(partition.name)
             ):
                 run_config = partition_set_def.run_config_for_partition(partition)
                 tags = partition_set_def.tags_for_partition(partition)
 
             partition_data.append(
                 ExternalPartitionExecutionParamData(
-                    name=partition.name, tags=tags, run_config=run_config,
+                    name=partition.name,
+                    tags=tags,
+                    run_config=run_config,
                 )
             )
 
@@ -439,3 +410,16 @@ def get_partition_set_execution_param_data(args):
         return ExternalPartitionExecutionErrorData(
             serializable_error_info_from_exc_info(sys.exc_info())
         )
+
+
+def get_notebook_data(notebook_path):
+    check.str_param(notebook_path, "notebook_path")
+
+    if not notebook_path.endswith(".ipynb"):
+        raise Exception(
+            "unexpected file extension for notebooks. Please provide a path that ends with '.ipynb'."
+        )
+
+    with open(os.path.abspath(notebook_path), "rb") as f:
+        content = f.read()
+        return content
