@@ -2,13 +2,13 @@ import math
 import os
 import queue
 import sys
-import tempfile
 import threading
 import time
 import uuid
 from collections import namedtuple
 from concurrent.futures import ThreadPoolExecutor
 from threading import Event as ThreadingEventType
+from time import sleep
 
 import grpc
 from dagster import check, seven
@@ -17,9 +17,8 @@ from dagster.core.definitions.reconstructable import (
     ReconstructableRepository,
     repository_def_from_target_def,
 )
-from dagster.core.errors import DagsterUserCodeProcessError
-from dagster.core.host_representation import ExternalPipelineOrigin, ExternalRepositoryOrigin
 from dagster.core.host_representation.external_data import external_repository_data_from_def
+from dagster.core.host_representation.origin import ExternalPipelineOrigin, ExternalRepositoryOrigin
 from dagster.core.instance import DagsterInstance
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
 from dagster.serdes import (
@@ -27,12 +26,7 @@ from dagster.serdes import (
     serialize_dagster_namedtuple,
     whitelist_for_serdes,
 )
-from dagster.serdes.ipc import (
-    IPCErrorMessage,
-    ipc_write_stream,
-    open_ipc_subprocess,
-    read_unary_response,
-)
+from dagster.serdes.ipc import IPCErrorMessage, ipc_write_stream, open_ipc_subprocess
 from dagster.seven import multiprocessing
 from dagster.utils import find_free_port, safe_tempfile_path_unmanaged
 from dagster.utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
@@ -47,6 +41,7 @@ from .impl import (
     get_external_pipeline_subset_result,
     get_external_schedule_execution,
     get_external_sensor_execution,
+    get_notebook_data,
     get_partition_config,
     get_partition_names,
     get_partition_set_execution_param_data,
@@ -78,17 +73,14 @@ EVENT_QUEUE_POLL_INTERVAL = 0.1
 
 CLEANUP_TICK = 0.5
 
-STREAMING_EXTERNAL_REPOSITORY_CHUNK_SIZE = 4000000
+STREAMING_CHUNK_SIZE = 4000000
 
 
 class CouldNotBindGrpcServerToAddress(Exception):
     pass
 
 
-class LazyRepositorySymbolsAndCodePointers:
-    """Enables lazily loading user code at RPC-time so that it doesn't interrupt startup and
-    we can gracefully handle user code errors."""
-
+class RepositorySymbolsAndCodePointers:
     def __init__(self, loadable_target_origin):
         self._loadable_target_origin = loadable_target_origin
         self._loadable_repository_symbols = None
@@ -104,16 +96,10 @@ class LazyRepositorySymbolsAndCodePointers:
 
     @property
     def loadable_repository_symbols(self):
-        if self._loadable_repository_symbols is None:
-            self.load()
-
         return self._loadable_repository_symbols
 
     @property
     def code_pointers_by_repo_name(self):
-        if self._code_pointers_by_repo_name is None:
-            self.load()
-
         return self._code_pointers_by_repo_name
 
 
@@ -210,11 +196,17 @@ class DagsterApiServer(DagsterApiServicer):
         self._termination_times = {}
         self._execution_lock = threading.Lock()
 
-        self._repository_symbols_and_code_pointers = LazyRepositorySymbolsAndCodePointers(
+        self._serializable_load_error = None
+
+        self._repository_symbols_and_code_pointers = RepositorySymbolsAndCodePointers(
             loadable_target_origin
         )
-        if not lazy_load_user_code:
+        try:
             self._repository_symbols_and_code_pointers.load()
+        except Exception:
+            if not lazy_load_user_code:
+                raise
+            self._serializable_load_error = serializable_error_info_from_exc_info(sys.exc_info())
 
         self.__last_heartbeat_time = time.time()
         if heartbeat:
@@ -271,10 +263,10 @@ class DagsterApiServer(DagsterApiServicer):
 
                         # the process died in an unexpected manner. inform the system
                         message = (
-                            "Pipeline execution process for {run_id} unexpectedly exited.".format(
-                                run_id=run.run_id
-                            )
+                            f"Run execution process for {run.run_id} unexpectedly "
+                            f"exited with exit code {process.exitcode}."
                         )
+
                         instance.report_engine_event(message, run, cls=self.__class__)
                         instance.report_run_failed(run)
 
@@ -306,6 +298,7 @@ class DagsterApiServer(DagsterApiServicer):
                 external_repository_origin.repository_name
             ],
             self._get_current_image(),
+            sys.executable,
         )
 
     def _recon_pipeline_from_origin(self, external_pipeline_origin):
@@ -352,18 +345,22 @@ class DagsterApiServer(DagsterApiServicer):
         )
 
     def ListRepositories(self, request, _context):
-        try:
-            response = ListRepositoriesResponse(
-                self._repository_symbols_and_code_pointers.loadable_repository_symbols,
-                executable_path=self._loadable_target_origin.executable_path
-                if self._loadable_target_origin
-                else None,
-                repository_code_pointer_dict=(
-                    self._repository_symbols_and_code_pointers.code_pointers_by_repo_name
-                ),
+        if self._serializable_load_error:
+            return api_pb2.ListRepositoriesReply(
+                serialized_list_repositories_response_or_error=serialize_dagster_namedtuple(
+                    self._serializable_load_error
+                )
             )
-        except Exception:  # pylint: disable=broad-except
-            response = serializable_error_info_from_exc_info(sys.exc_info())
+
+        response = ListRepositoriesResponse(
+            self._repository_symbols_and_code_pointers.loadable_repository_symbols,
+            executable_path=self._loadable_target_origin.executable_path
+            if self._loadable_target_origin
+            else None,
+            repository_code_pointer_dict=(
+                self._repository_symbols_and_code_pointers.code_pointers_by_repo_name
+            ),
+        )
 
         return api_pb2.ListRepositoriesReply(
             serialized_list_repositories_response_or_error=serialize_dagster_namedtuple(response)
@@ -387,6 +384,11 @@ class DagsterApiServer(DagsterApiServicer):
             )
         )
 
+    def ExternalNotebookData(self, request, _context):
+        notebook_path = request.notebook_path
+        check.str_param(notebook_path, "notebook_path")
+        return api_pb2.ExternalNotebookDataReply(content=get_notebook_data(notebook_path))
+
     def ExternalPartitionSetExecutionParams(self, request, _context):
         args = deserialize_json_to_dagster_namedtuple(
             request.serialized_partition_set_execution_param_args
@@ -399,17 +401,16 @@ class DagsterApiServer(DagsterApiServicer):
         )
 
         recon_repo = self._recon_repository_from_origin(args.repository_origin)
-
-        return api_pb2.ExternalPartitionSetExecutionParamsReply(
-            serialized_external_partition_set_execution_param_data_or_external_partition_execution_error=serialize_dagster_namedtuple(
-                get_partition_set_execution_param_data(
-                    recon_repo=recon_repo,
-                    partition_set_name=args.partition_set_name,
-                    partition_names=args.partition_names,
-                )
+        serialized_data = serialize_dagster_namedtuple(
+            get_partition_set_execution_param_data(
+                recon_repo=recon_repo,
+                partition_set_name=args.partition_set_name,
+                partition_names=args.partition_names,
             )
         )
 
+        yield from self._split_serialized_data_into_chunk_events(serialized_data)
+
     def ExternalPartitionConfig(self, request, _context):
         args = deserialize_json_to_dagster_namedtuple(request.serialized_partition_args)
 
@@ -479,16 +480,13 @@ class DagsterApiServer(DagsterApiServicer):
         serialized_external_repository_data = self._get_serialized_external_repository_data(request)
 
         num_chunks = int(
-            math.ceil(
-                float(len(serialized_external_repository_data))
-                / STREAMING_EXTERNAL_REPOSITORY_CHUNK_SIZE
-            )
+            math.ceil(float(len(serialized_external_repository_data)) / STREAMING_CHUNK_SIZE)
         )
 
         for i in range(num_chunks):
-            start_index = i * STREAMING_EXTERNAL_REPOSITORY_CHUNK_SIZE
+            start_index = i * STREAMING_CHUNK_SIZE
             end_index = min(
-                (i + 1) * STREAMING_EXTERNAL_REPOSITORY_CHUNK_SIZE,
+                (i + 1) * STREAMING_CHUNK_SIZE,
                 len(serialized_external_repository_data),
             )
 
@@ -499,6 +497,20 @@ class DagsterApiServer(DagsterApiServicer):
                 ],
             )
 
+    def _split_serialized_data_into_chunk_events(self, serialized_data):
+        num_chunks = int(math.ceil(float(len(serialized_data)) / STREAMING_CHUNK_SIZE))
+        for i in range(num_chunks):
+            start_index = i * STREAMING_CHUNK_SIZE
+            end_index = min(
+                (i + 1) * STREAMING_CHUNK_SIZE,
+                len(serialized_data),
+            )
+
+            yield api_pb2.StreamingChunkEvent(
+                sequence_number=i,
+                serialized_chunk=serialized_data[start_index:end_index],
+            )
+
     def ExternalScheduleExecution(self, request, _context):
         args = deserialize_json_to_dagster_namedtuple(
             request.serialized_external_schedule_execution_args
@@ -511,19 +523,18 @@ class DagsterApiServer(DagsterApiServicer):
         )
 
         recon_repo = self._recon_repository_from_origin(args.repository_origin)
-
-        return api_pb2.ExternalScheduleExecutionReply(
-            serialized_external_schedule_execution_data_or_external_schedule_execution_error=serialize_dagster_namedtuple(
-                get_external_schedule_execution(
-                    recon_repo,
-                    args.instance_ref,
-                    args.schedule_name,
-                    args.scheduled_execution_timestamp,
-                    args.scheduled_execution_timezone,
-                )
+        serialized_schedule_data = serialize_dagster_namedtuple(
+            get_external_schedule_execution(
+                recon_repo,
+                args.instance_ref,
+                args.schedule_name,
+                args.scheduled_execution_timestamp,
+                args.scheduled_execution_timezone,
             )
         )
 
+        yield from self._split_serialized_data_into_chunk_events(serialized_schedule_data)
+
     def ExternalSensorExecution(self, request, _context):
         args = deserialize_json_to_dagster_namedtuple(
             request.serialized_external_sensor_execution_args
@@ -532,19 +543,19 @@ class DagsterApiServer(DagsterApiServicer):
         check.inst_param(args, "args", SensorExecutionArgs)
 
         recon_repo = self._recon_repository_from_origin(args.repository_origin)
-
-        return api_pb2.ExternalSensorExecutionReply(
-            serialized_external_sensor_execution_data_or_external_sensor_execution_error=serialize_dagster_namedtuple(
-                get_external_sensor_execution(
-                    recon_repo,
-                    args.instance_ref,
-                    args.sensor_name,
-                    args.last_completion_time,
-                    args.last_run_key,
-                )
+        serialized_sensor_data = serialize_dagster_namedtuple(
+            get_external_sensor_execution(
+                recon_repo,
+                args.instance_ref,
+                args.sensor_name,
+                args.last_completion_time,
+                args.last_run_key,
+                args.cursor,
             )
         )
 
+        yield from self._split_serialized_data_into_chunk_events(serialized_sensor_data)
+
     def ShutdownServer(self, request, _context):
         try:
             self._shutdown_once_executions_finish_event.set()
@@ -553,7 +564,7 @@ class DagsterApiServer(DagsterApiServicer):
                     ShutdownServerResult(success=True, serializable_error_info=None)
                 )
             )
-        except:  # pylint: disable=bare-except
+        except:
             return api_pb2.ShutdownServerReply(
                 serialized_shutdown_server_result=serialize_dagster_namedtuple(
                     ShutdownServerResult(
@@ -580,7 +591,7 @@ class DagsterApiServer(DagsterApiServicer):
                     self._termination_times[cancel_execution_request.run_id] = time.time()
                     success = True
 
-        except:  # pylint: disable=bare-except
+        except:
             serializable_error_info = serializable_error_info_from_exc_info(sys.exc_info())
 
         return api_pb2.CancelExecutionReply(
@@ -630,7 +641,7 @@ class DagsterApiServer(DagsterApiServicer):
             run_id = execute_run_args.pipeline_run_id
             recon_pipeline = self._recon_pipeline_from_origin(execute_run_args.pipeline_origin)
 
-        except:  # pylint: disable=bare-except
+        except:
             return api_pb2.StartRunReply(
                 serialized_start_run_result=serialize_dagster_namedtuple(
                     StartRunResult(
@@ -668,7 +679,7 @@ class DagsterApiServer(DagsterApiServicer):
         serializable_error_info = None
 
         while success is None:
-            time.sleep(EVENT_QUEUE_POLL_INTERVAL)
+            sleep(EVENT_QUEUE_POLL_INTERVAL)
             # We use `get_nowait()` instead of `get()` so that we can handle the case where the
             # execution process has died unexpectedly -- `get()` would hang forever in that case
             try:
@@ -798,7 +809,10 @@ class DagsterGrpcServer:
             "If set to None, the server will use the gRPC default.",
         )
 
-        self.server = grpc.server(ThreadPoolExecutor(max_workers=max_workers))
+        self.server = grpc.server(
+            ThreadPoolExecutor(max_workers=max_workers),
+            compression=grpc.Compression.Gzip,
+        )
         self._server_termination_event = threading.Event()
 
         try:
@@ -909,21 +923,29 @@ class CouldNotStartServerProcess(Exception):
         )
 
 
-def wait_for_grpc_server(server_process, ipc_output_file, timeout=60):
-    event = read_unary_response(ipc_output_file, timeout=timeout, ipc_process=server_process)
+def wait_for_grpc_server(server_process, client, subprocess_args, timeout=60):
+    start_time = time.time()
 
-    if isinstance(event, GrpcServerFailedToBindEvent):
-        raise CouldNotBindGrpcServerToAddress()
-    elif isinstance(event, GrpcServerLoadErrorEvent):
-        raise DagsterUserCodeProcessError(
-            event.error_info.to_string(), user_code_process_error_infos=[event.error_info]
-        )
-    elif isinstance(event, GrpcServerStartedEvent):
-        return True
-    else:
-        raise Exception(
-            "Received unexpected IPC event from gRPC Server: {event}".format(event=event)
-        )
+    last_error = None
+
+    while True:
+        try:
+            client.ping("")
+            return
+        except grpc._channel._InactiveRpcError:  # pylint: disable=protected-access
+            last_error = serializable_error_info_from_exc_info(sys.exc_info())
+
+        if time.time() - start_time > timeout:
+            raise Exception(
+                f"Timed out waiting for gRPC server to start with arguments: \"{' '.join(subprocess_args)}\". Most recent connection error: {str(last_error)}"
+            )
+
+        if server_process.poll() != None:
+            raise Exception(
+                f"gRPC server exited with return code {server_process.returncode} while starting up with the command: \"{' '.join(subprocess_args)}\""
+            )
+
+        sleep(0.1)
 
 
 def open_server_process(
@@ -933,8 +955,8 @@ def open_server_process(
     max_workers=None,
     heartbeat=False,
     heartbeat_timeout=30,
-    lazy_load_user_code=False,
     fixed_server_id=None,
+    startup_timeout=20,
 ):
     check.invariant((port or socket) and not (port and socket), "Set only port or socket")
     check.opt_inst_param(loadable_target_origin, "loadable_target_origin", LoadableTargetOrigin)
@@ -942,49 +964,48 @@ def open_server_process(
 
     from dagster.core.test_utils import get_mocked_system_timezone
 
-    with tempfile.TemporaryDirectory() as temp_dir:
-        output_file = os.path.join(
-            temp_dir, "grpc-server-startup-{uuid}".format(uuid=uuid.uuid4().hex)
-        )
+    mocked_system_timezone = get_mocked_system_timezone()
 
-        mocked_system_timezone = get_mocked_system_timezone()
+    subprocess_args = (
+        [
+            loadable_target_origin.executable_path
+            if loadable_target_origin and loadable_target_origin.executable_path
+            else sys.executable,
+            "-m",
+            "dagster.grpc",
+        ]
+        + ["--lazy-load-user-code"]
+        + (["--port", str(port)] if port else [])
+        + (["--socket", socket] if socket else [])
+        + (["-n", str(max_workers)] if max_workers else [])
+        + (["--heartbeat"] if heartbeat else [])
+        + (["--heartbeat-timeout", str(heartbeat_timeout)] if heartbeat_timeout else [])
+        + (["--fixed-server-id", fixed_server_id] if fixed_server_id else [])
+        + (["--override-system-timezone", mocked_system_timezone] if mocked_system_timezone else [])
+        + (["--log-level", "WARNING"])  # don't log INFO messages for automatically spun up servers
+    )
 
-        subprocess_args = (
-            [
-                loadable_target_origin.executable_path
-                if loadable_target_origin and loadable_target_origin.executable_path
-                else sys.executable,
-                "-m",
-                "dagster.grpc",
-            ]
-            + (["--port", str(port)] if port else [])
-            + (["--socket", socket] if socket else [])
-            + (["-n", str(max_workers)] if max_workers else [])
-            + (["--heartbeat"] if heartbeat else [])
-            + (["--heartbeat-timeout", str(heartbeat_timeout)] if heartbeat_timeout else [])
-            + (["--lazy-load-user-code"] if lazy_load_user_code else [])
-            + (["--ipc-output-file", output_file])
-            + (["--fixed-server-id", fixed_server_id] if fixed_server_id else [])
-            + (
-                ["--override-system-timezone", mocked_system_timezone]
-                if mocked_system_timezone
-                else []
-            )
-        )
+    if loadable_target_origin:
+        subprocess_args += loadable_target_origin.get_cli_args()
 
-        if loadable_target_origin:
-            subprocess_args += loadable_target_origin.get_cli_args()
+    server_process = open_ipc_subprocess(subprocess_args)
 
-        server_process = open_ipc_subprocess(subprocess_args)
+    from dagster.grpc.client import DagsterGrpcClient
 
-        try:
-            wait_for_grpc_server(server_process, output_file)
-        except:
-            if server_process.poll() is None:
-                server_process.terminate()
-            raise
+    client = DagsterGrpcClient(
+        port=port,
+        socket=socket,
+        host="localhost",
+    )
 
-        return server_process
+    try:
+        wait_for_grpc_server(server_process, client, subprocess_args, timeout=startup_timeout)
+    except:
+        if server_process.poll() is None:
+            server_process.terminate()
+        raise
+
+    return server_process
 
 
 def open_server_process_on_dynamic_port(
@@ -993,8 +1014,8 @@ def open_server_process_on_dynamic_port(
     max_workers=None,
     heartbeat=False,
     heartbeat_timeout=30,
-    lazy_load_user_code=False,
     fixed_server_id=None,
+    startup_timeout=20,
 ):
     server_process = None
     retries = 0
@@ -1008,8 +1029,8 @@ def open_server_process_on_dynamic_port(
                 max_workers=max_workers,
                 heartbeat=heartbeat,
                 heartbeat_timeout=heartbeat_timeout,
-                lazy_load_user_code=lazy_load_user_code,
                 fixed_server_id=fixed_server_id,
+                startup_timeout=startup_timeout,
             )
         except CouldNotBindGrpcServerToAddress:
             pass
@@ -1028,22 +1049,24 @@ class GrpcServerProcess:
         max_workers=None,
         heartbeat=False,
         heartbeat_timeout=30,
-        lazy_load_user_code=False,
         fixed_server_id=None,
+        startup_timeout=20,
     ):
         self.port = None
         self.socket = None
         self.server_process = None
 
-        check.opt_inst_param(loadable_target_origin, "loadable_target_origin", LoadableTargetOrigin)
+        self.loadable_target_origin = check.opt_inst_param(
+            loadable_target_origin, "loadable_target_origin", LoadableTargetOrigin
+        )
         check.bool_param(force_port, "force_port")
         check.int_param(max_retries, "max_retries")
         check.opt_int_param(max_workers, "max_workers")
         check.bool_param(heartbeat, "heartbeat")
         check.int_param(heartbeat_timeout, "heartbeat_timeout")
         check.invariant(heartbeat_timeout > 0, "heartbeat_timeout must be greater than 0")
-        check.bool_param(lazy_load_user_code, "lazy_load_user_code")
         check.opt_str_param(fixed_server_id, "fixed_server_id")
+        check.int_param(startup_timeout, "startup_timeout")
         check.invariant(
             max_workers is None or max_workers > 1 if heartbeat else True,
             "max_workers must be greater than 1 or set to None if heartbeat is True. "
@@ -1057,8 +1080,8 @@ class GrpcServerProcess:
                 max_workers=max_workers,
                 heartbeat=heartbeat,
                 heartbeat_timeout=heartbeat_timeout,
-                lazy_load_user_code=lazy_load_user_code,
                 fixed_server_id=fixed_server_id,
+                startup_timeout=startup_timeout,
             )
         else:
             self.socket = safe_tempfile_path_unmanaged()
@@ -1070,8 +1093,8 @@ class GrpcServerProcess:
                 max_workers=max_workers,
                 heartbeat=heartbeat,
                 heartbeat_timeout=heartbeat_timeout,
-                lazy_load_user_code=lazy_load_user_code,
                 fixed_server_id=fixed_server_id,
+                startup_timeout=startup_timeout,
             )
 
         if self.server_process is None:
