@@ -16,6 +16,7 @@ from dagster.core.workspace import IWorkspace
 from dagster.seven.compat.pendulum import to_timezone
 from dagster.utils import merge_dicts
 from dagster.utils.error import serializable_error_info_from_exc_info
+from dagster.utils.log import default_date_format_string
 
 
 class _ScheduleLaunchContext:
@@ -45,9 +46,6 @@ class _ScheduleLaunchContext:
         self._write()
 
 
-_SCHEDULER_DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S%z"
-
-
 def execute_scheduler_iteration(instance, workspace, logger, max_catchup_runs):
     end_datetime_utc = pendulum.now("UTC")
     yield from launch_scheduled_runs(
@@ -89,12 +87,13 @@ def launch_scheduled_runs(
                 instance,
                 logger,
                 schedule_state,
+                workspace,
                 repo_location,
                 end_datetime_utc,
                 max_catchup_runs,
                 (debug_crash_flags.get(schedule_state.job_name) if debug_crash_flags else None),
             )
-        except Exception:  # pylint: disable=broad-except
+        except Exception:
             error_info = serializable_error_info_from_exc_info(sys.exc_info())
             logger.error(
                 f"Scheduler caught an error for schedule {schedule_state.job_name} : {error_info.to_string()}"
@@ -106,6 +105,7 @@ def launch_scheduled_runs_for_schedule(
     instance,
     logger,
     schedule_state,
+    workspace,
     repo_location,
     end_datetime_utc,
     max_catchup_runs,
@@ -118,13 +118,14 @@ def launch_scheduled_runs_for_schedule(
 
     latest_tick = instance.get_latest_job_tick(schedule_state.job_origin_id)
 
-    if not latest_tick:
-        start_timestamp_utc = schedule_state.job_specific_data.start_timestamp
-    elif latest_tick.status == JobTickStatus.STARTED:
-        # Scheduler was interrupted while performing this tick, re-do it
-        start_timestamp_utc = latest_tick.timestamp
-    else:
-        start_timestamp_utc = latest_tick.timestamp + 1
+    start_timestamp_utc = schedule_state.job_specific_data.start_timestamp
+
+    if latest_tick:
+        if latest_tick.status == JobTickStatus.STARTED:
+            # Scheduler was interrupted while performing this tick, re-do it
+            start_timestamp_utc = max(start_timestamp_utc, latest_tick.timestamp)
+        else:
+            start_timestamp_utc = max(start_timestamp_utc, latest_tick.timestamp + 1)
 
     schedule_name = schedule_state.job_name
     repo_name = schedule_state.origin.external_repository_origin.repository_name
@@ -173,10 +174,10 @@ def launch_scheduled_runs_for_schedule(
         tick_times = tick_times[-max_catchup_runs:]
 
     if len(tick_times) == 1:
-        tick_time = tick_times[0].strftime(_SCHEDULER_DATETIME_FORMAT)
+        tick_time = tick_times[0].strftime(default_date_format_string())
         logger.info(f"Evaluating schedule `{schedule_name}` at {tick_time}")
     else:
-        times = ", ".join([time.strftime(_SCHEDULER_DATETIME_FORMAT) for time in tick_times])
+        times = ", ".join([time.strftime(default_date_format_string()) for time in tick_times])
         logger.info(f"Evaluating schedule `{schedule_name}` at the following times: {times}")
 
     for schedule_time in tick_times:
@@ -204,6 +205,7 @@ def launch_scheduled_runs_for_schedule(
             yield from _schedule_runs_at_time(
                 instance,
                 logger,
+                workspace,
                 repo_location,
                 external_repo,
                 external_schedule,
@@ -229,6 +231,7 @@ def _check_for_debug_crash(debug_crash_flags, key):
 def _schedule_runs_at_time(
     instance,
     logger,
+    workspace,
     repo_location,
     external_repo,
     external_schedule,
@@ -301,9 +304,9 @@ def _schedule_runs_at_time(
 
         if run.status != PipelineRunStatus.FAILURE:
             try:
-                instance.submit_run(run.run_id, external_pipeline)
+                instance.submit_run(run.run_id, workspace)
                 logger.info(f"Completed scheduled launch of run {run.run_id} for {schedule_name}")
-            except Exception:  # pylint: disable=broad-except
+            except Exception:
                 error_info = serializable_error_info_from_exc_info(sys.exc_info())
                 logger.error(
                     f"Run {run.run_id} created successfully but failed to launch: {str(serializable_error_info_from_exc_info(sys.exc_info()))}"
@@ -359,7 +362,7 @@ def _create_scheduler_run(
             known_state=None,
         )
         execution_plan_snapshot = external_execution_plan.execution_plan_snapshot
-    except Exception:  # pylint: disable=broad-except
+    except Exception:
         execution_plan_errors.append(serializable_error_info_from_exc_info(sys.exc_info()))
 
     pipeline_tags = external_pipeline.tags or {}
@@ -392,6 +395,7 @@ def _create_scheduler_run(
         execution_plan_snapshot=execution_plan_snapshot,
         parent_pipeline_snapshot=external_pipeline.parent_pipeline_snapshot,
         external_pipeline_origin=external_pipeline.get_external_origin(),
+        pipeline_code_origin=external_pipeline.get_python_origin(),
     )
 
     if len(execution_plan_errors) > 0:
