@@ -14,12 +14,12 @@ Why not pickle?
   (in memory, not human readable, etc) just handle the json case effectively.
 """
 
-
 from abc import ABC, abstractmethod
 from enum import Enum
 from inspect import Parameter, isclass, signature
 from typing import (
     Any,
+    Callable,
     Dict,
     List,
     Mapping,
@@ -29,8 +29,8 @@ from typing import (
     Tuple,
     Type,
     TypeVar,
-    Union,
     cast,
+    overload,
 )
 
 from dagster import check, seven
@@ -76,10 +76,11 @@ class WhitelistMap(NamedTuple):
 
     def register_enum(
         self,
+        name: str,
         enum: Type[Enum],
         serializer: Optional[Type["EnumSerializer"]],
     ):
-        self.enums[enum.__name__] = (enum, serializer or DefaultEnumSerializer)
+        self.enums[name] = (enum, serializer or DefaultEnumSerializer)
 
     def has_enum_entry(self, name: str) -> bool:
         return name in self.enums
@@ -95,7 +96,17 @@ class WhitelistMap(NamedTuple):
 _WHITELIST_MAP = WhitelistMap.create()
 
 
-def whitelist_for_serdes(serializer: Union[Type, Type["Serializer"]]):
+@overload
+def whitelist_for_serdes(serializer: Type) -> Type:
+    ...
+
+
+@overload
+def whitelist_for_serdes(serializer: Type["Serializer"]) -> Callable[[Type], Type]:
+    ...
+
+
+def whitelist_for_serdes(serializer):
     """
     Decorator to whitelist a named tuple or enum to be serializable.
 
@@ -115,13 +126,19 @@ def whitelist_for_serdes(serializer: Union[Type, Type["Serializer"]]):
 def _whitelist_for_serdes(
     whitelist_map: WhitelistMap, serializer: Optional[Type["Serializer"]] = None
 ):
-    def __whitelist_for_serdes(klass):
-        if issubclass(klass, Enum):
-            whitelist_map.register_enum(klass, serializer)
-        elif issubclass(klass, tuple):
+    def __whitelist_for_serdes(klass: Type):
+        if issubclass(klass, Enum) and (
+            serializer is None or issubclass(serializer, EnumSerializer)
+        ):
+            whitelist_map.register_enum(klass.__name__, klass, serializer)
+        elif issubclass(klass, tuple) and (
+            serializer is None or issubclass(serializer, NamedTupleSerializer)
+        ):
             sig_params = signature(klass.__new__).parameters
             _check_serdes_tuple_class_invariants(klass, sig_params)
-            whitelist_map.register_tuple(klass.__name__, klass, serializer, sig_params)
+            whitelist_map.register_tuple(
+                klass.__name__, cast(Type[NamedTuple], klass), serializer, sig_params
+            )
         else:
             raise SerdesUsageError(f"Can not whitelist class {klass} for serializer {serializer}")
 
@@ -140,12 +157,25 @@ class EnumSerializer(Serializer):
     def value_from_storage_str(cls, storage_str: str, klass: Type) -> Enum:
         raise NotImplementedError()
 
+    @classmethod
+    @abstractmethod
+    def value_to_storage_str(
+        cls, value: Enum, whitelist_map: WhitelistMap, descent_path: str
+    ) -> str:
+        raise NotImplementedError()
+
 
 class DefaultEnumSerializer(EnumSerializer):
     @classmethod
     def value_from_storage_str(cls, storage_str: str, klass: Type) -> Enum:
         return getattr(klass, storage_str)
 
+    @classmethod
+    def value_to_storage_str(
+        cls, value: Enum, whitelist_map: WhitelistMap, descent_path: str
+    ) -> str:
+        return str(value)
+
 
 class NamedTupleSerializer(Serializer):
     @classmethod
@@ -155,12 +185,18 @@ class NamedTupleSerializer(Serializer):
         storage_dict: Dict[str, Any],
         klass: Type,
         args_for_class: Mapping[str, Parameter],
+        whitelist_map: WhitelistMap,
+        descent_path: str,
     ) -> NamedTuple:
         """
+        Load the target value from the object tree output of json parsing.
+
         Args:
             storage_dict: The parsed json object to hydrate
             klass: The namedtuple class object
             args_for_class: the inspect.signature paramaters for __new__
+            whitelist_map: current map of whitelisted serdes objects
+            descent_path: the path to the current node from the root
         """
         raise NotImplementedError()
 
@@ -172,6 +208,15 @@ class NamedTupleSerializer(Serializer):
         whitelist_map: WhitelistMap,
         descent_path: str,
     ) -> Dict[str, Any]:
+        """
+        Transform the object in to a form that can be json serialized.
+
+        Args:
+            value: the instance of the object
+            whitelist_map: current map of whitelisted serdes objects
+            descent_path: the path to the current node from the root
+        """
+
         raise NotImplementedError()
 
 
@@ -192,12 +237,26 @@ class DefaultNamedTupleSerializer(NamedTupleSerializer):
         storage_dict: Dict[str, Any],
         klass: Type,
         args_for_class: Mapping[str, Parameter],
+        whitelist_map: WhitelistMap,
+        descent_path: str,
     ) -> NamedTuple:
         # Naively implements backwards compatibility by filtering arguments that aren't present in
         # the constructor. If a property is present in the serialized object, but doesn't exist in
         # the version of the class loaded into memory, that property will be completely ignored.
-        filtered_val = {k: v for k, v in storage_dict.items() if k in args_for_class}
-        return klass(**filtered_val)
+        unpacked_dict = {
+            key: unpack_inner_value(value, whitelist_map, f"{descent_path}.{key}")
+            for key, value in storage_dict.items()
+            if key in args_for_class
+        }
+        return cls.value_from_unpacked(unpacked_dict, klass)
+
+    @classmethod
+    def value_from_unpacked(
+        cls,
+        unpacked_dict: Dict[str, Any],
+        klass: Type,
+    ):
+        return klass(**unpacked_dict)
 
     @classmethod
     def value_to_storage_dict(
@@ -212,7 +271,7 @@ class DefaultNamedTupleSerializer(NamedTupleSerializer):
         for key, inner_value in value._asdict().items():
             if key in skip_when_empty_fields and inner_value in EMPTY_VALUES_TO_SKIP:
                 continue
-            base_dict[key] = _pack_value(inner_value, whitelist_map, f"{descent_path}.{key}")
+            base_dict[key] = pack_inner_value(inner_value, whitelist_map, f"{descent_path}.{key}")
 
         base_dict["__class__"] = value.__class__.__name__
         return base_dict
@@ -230,12 +289,14 @@ def serialize_dagster_namedtuple(nt: tuple, **json_kwargs) -> str:
 
 
 def _serialize_dagster_namedtuple(nt: tuple, whitelist_map: WhitelistMap, **json_kwargs) -> str:
-    return seven.json.dumps(_pack_value(nt, whitelist_map, _root(nt)), **json_kwargs)
+    return seven.json.dumps(pack_inner_value(nt, whitelist_map, _root(nt)), **json_kwargs)
 
 
-def serialize_value(val: Any) -> str:
+def serialize_value(val: Any, whitelist_map: WhitelistMap = _WHITELIST_MAP) -> str:
     """Serialize a value to a json encoded string."""
-    return seven.json.dumps(_pack_value(val, whitelist_map=_WHITELIST_MAP, descent_path=_root(val)))
+    return seven.json.dumps(
+        pack_inner_value(val, whitelist_map=whitelist_map, descent_path=_root(val))
+    )
 
 
 def pack_value(val: Any) -> Any:
@@ -246,13 +307,13 @@ def pack_value(val: Any) -> Any:
         * set
         * frozenset
     """
-    return _pack_value(val, whitelist_map=_WHITELIST_MAP, descent_path=_root(val))
+    return pack_inner_value(val, whitelist_map=_WHITELIST_MAP, descent_path=_root(val))
 
 
-def _pack_value(val: Any, whitelist_map: WhitelistMap, descent_path: str) -> Any:
+def pack_inner_value(val: Any, whitelist_map: WhitelistMap, descent_path: str) -> Any:
     if isinstance(val, list):
         return [
-            _pack_value(item, whitelist_map, f"{descent_path}[{idx}]")
+            pack_inner_value(item, whitelist_map, f"{descent_path}[{idx}]")
             for idx, item in enumerate(val)
         ]
     if isinstance(val, tuple):
@@ -270,25 +331,27 @@ def _pack_value(val: Any, whitelist_map: WhitelistMap, descent_path: str) -> Any
             raise SerializationError(
                 f"Can only serialize whitelisted Enums, received {klass_name}.{_path_msg(descent_path)}",
             )
-        return {"__enum__": str(val)}
+        _, enum_serializer = whitelist_map.get_enum_entry(klass_name)
+        return {"__enum__": enum_serializer.value_to_storage_str(val, whitelist_map, descent_path)}
     if isinstance(val, set):
         set_path = descent_path + "{}"
         return {
             "__set__": [
-                _pack_value(item, whitelist_map, set_path) for item in sorted(list(val), key=str)
+                pack_inner_value(item, whitelist_map, set_path)
+                for item in sorted(list(val), key=str)
             ]
         }
     if isinstance(val, frozenset):
         frz_set_path = descent_path + "{}"
         return {
             "__frozenset__": [
-                _pack_value(item, whitelist_map, frz_set_path)
+                pack_inner_value(item, whitelist_map, frz_set_path)
                 for item in sorted(list(val), key=str)
             ]
         }
     if isinstance(val, dict):
         return {
-            key: _pack_value(value, whitelist_map, f"{descent_path}.{key}")
+            key: pack_inner_value(value, whitelist_map, f"{descent_path}.{key}")
             for key, value in val.items()
         }
 
@@ -326,33 +389,38 @@ def deserialize_as(json_str: str, cls: Type[T]) -> T:
     check.failed(f"Deserialized object was not expected target type {cls}, got {type(val)}")
 
 
+def opt_deserialize_as(json_str: Optional[str], cls: Type[T]) -> Optional[T]:
+    """Optionally deserialize a json encoded string to a specific namedtuple class."""
+    return deserialize_as(json_str, cls) if json_str else None
+
+
 def _deserialize_json(json_str: str, whitelist_map: WhitelistMap):
     value = seven.json.loads(json_str)
-    return _unpack_value(value, whitelist_map=whitelist_map, descent_path=_root(value))
+    return unpack_inner_value(value, whitelist_map=whitelist_map, descent_path=_root(value))
 
 
-def deserialize_value(val: str) -> Any:
+def deserialize_value(val: str, whitelist_map: WhitelistMap = _WHITELIST_MAP) -> Any:
     """Deserialize a json encoded string in to its original value"""
-    return _unpack_value(
+    return unpack_inner_value(
         seven.json.loads(check.str_param(val, "val")),
-        whitelist_map=_WHITELIST_MAP,
+        whitelist_map=whitelist_map,
         descent_path="",
     )
 
 
 def unpack_value(val: Any) -> Any:
     """Convert a packed value in to its original form"""
-    return _unpack_value(
+    return unpack_inner_value(
         val,
         whitelist_map=_WHITELIST_MAP,
         descent_path="",
     )
 
 
-def _unpack_value(val: Any, whitelist_map: WhitelistMap, descent_path: str) -> Any:
+def unpack_inner_value(val: Any, whitelist_map: WhitelistMap, descent_path: str) -> Any:
     if isinstance(val, list):
         return [
-            _unpack_value(item, whitelist_map, f"{descent_path}[{idx}]")
+            unpack_inner_value(item, whitelist_map, f"{descent_path}[{idx}]")
             for idx, item in enumerate(val)
         ]
     if isinstance(val, dict) and val.get("__class__"):
@@ -364,17 +432,15 @@ def _unpack_value(val: Any, whitelist_map: WhitelistMap, descent_path: str) -> A
                 f"expected versions.{_path_msg(descent_path)}"
             )
 
-        klass, serializer, sig_params = whitelist_map.get_tuple_entry(klass_name)
+        klass, serializer, args_for_class = whitelist_map.get_tuple_entry(klass_name)
 
         # Target class being set to none, likely by
         if klass is None:
             return None
 
-        unpacked_val = {
-            key: _unpack_value(value, whitelist_map, f"{descent_path}.{key}")
-            for key, value in val.items()
-        }
-        return serializer.value_from_storage_dict(unpacked_val, klass, sig_params)
+        return serializer.value_from_storage_dict(
+            val, klass, args_for_class, whitelist_map, descent_path
+        )
     if isinstance(val, dict) and val.get("__enum__"):
         name, member = val["__enum__"].split(".")
         if not whitelist_map.has_enum_entry(name):
@@ -387,15 +453,15 @@ def _unpack_value(val: Any, whitelist_map: WhitelistMap, descent_path: str) -> A
         return enum_serializer.value_from_storage_str(member, enum_class)
     if isinstance(val, dict) and val.get("__set__") is not None:
         set_path = descent_path + "{}"
-        return set([_unpack_value(item, whitelist_map, set_path) for item in val["__set__"]])
+        return set([unpack_inner_value(item, whitelist_map, set_path) for item in val["__set__"]])
     if isinstance(val, dict) and val.get("__frozenset__") is not None:
         frz_set_path = descent_path + "{}"
         return frozenset(
-            [_unpack_value(item, whitelist_map, frz_set_path) for item in val["__frozenset__"]]
+            [unpack_inner_value(item, whitelist_map, frz_set_path) for item in val["__frozenset__"]]
         )
     if isinstance(val, dict):
         return {
-            key: _unpack_value(value, whitelist_map, f"{descent_path}.{key}")
+            key: unpack_inner_value(value, whitelist_map, f"{descent_path}.{key}")
             for key, value in val.items()
         }
 
@@ -410,7 +476,7 @@ def _unpack_value(val: Any, whitelist_map: WhitelistMap, descent_path: str) -> A
 def register_serdes_tuple_fallbacks(
     fallback_map: Dict[str, Optional[Type]],
     whitelist_map: WhitelistMap = _WHITELIST_MAP,
-):
+) -> None:
     """
     Manually provide remappings for named tuples.
     Used to manage loading previously types that no longer exist.
@@ -425,6 +491,26 @@ def register_serdes_tuple_fallbacks(
         )
 
 
+def register_serdes_enum_fallbacks(
+    fallback_map: Dict[str, Optional[Type[Enum]]],
+    whitelist_map: WhitelistMap = _WHITELIST_MAP,
+) -> None:
+    """
+    Manually provide remappings for named tuples.
+    Used to manage loading previously types that no longer exist.
+    """
+
+    serializer: Type[EnumSerializer] = DefaultEnumSerializer
+    for class_name, klass in fallback_map.items():
+        if not klass or not issubclass(klass, Enum):
+            raise SerdesUsageError(
+                f"Cannot register {klass} as an enum, because it does not have enum type."
+            )
+        if klass and whitelist_map.has_enum_entry(klass.__name__):
+            _, serializer = whitelist_map.get_enum_entry(klass.__name__)
+        whitelist_map.register_enum(class_name, klass, serializer)
+
+
 ###################################################################################################
 # Validation
 ###################################################################################################
