@@ -12,8 +12,21 @@ import sys
 import tempfile
 import threading
 from collections import namedtuple
+from datetime import timezone
 from enum import Enum
-from typing import Iterator
+from typing import (
+    TYPE_CHECKING,
+    Callable,
+    ContextManager,
+    Generator,
+    Generic,
+    Iterator,
+    Optional,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 from warnings import warn
 
 import _thread as thread
@@ -23,6 +36,7 @@ from dagster.core.errors import DagsterExecutionInterruptedError, DagsterInvaria
 from dagster.seven import IS_WINDOWS, multiprocessing
 from dagster.seven.abc import Mapping
 
+from .alert import make_email_on_pipeline_failure_sensor, make_email_on_run_failure_sensor
 from .merger import merge_dicts
 from .yaml_utils import load_yaml_from_glob_list, load_yaml_from_globs, load_yaml_from_path
 
@@ -31,6 +45,10 @@ if sys.version_info > (3,):
 else:
     from pathlib2 import Path  # pylint: disable=import-error
 
+if TYPE_CHECKING:
+    from dagster.core.events import DagsterEvent
+
+
 EPOCH = datetime.datetime.utcfromtimestamp(0)
 
 PICKLE_PROTOCOL = 4
@@ -351,12 +369,16 @@ def start_termination_thread(termination_event):
     int_thread.start()
 
 
+T = TypeVar("T")
+
 # Executes the next() function within an instance of the supplied context manager class
 # (leaving the context before yielding each result)
-def iterate_with_context(context, iterator):
+def iterate_with_context(
+    context_fn: Callable[[], ContextManager], iterator: Iterator[T]
+) -> Iterator[T]:
     while True:
         # Allow interrupts during user code so that we can terminate slow/hanging steps
-        with context():
+        with context_fn():
             try:
                 next_output = next(iterator)
             except StopIteration:
@@ -388,7 +410,10 @@ class frozentags(frozendict):
         return frozentags(updated)
 
 
-class EventGenerationManager:
+GeneratedContext = TypeVar("GeneratedContext")
+
+
+class EventGenerationManager(Generic[GeneratedContext]):
     """Utility class that wraps an event generator function, that also yields a single instance of
     a typed object.  All events yielded before the typed object are yielded through the method
     `generate_setup_events` and all events yielded after the typed object are yielded through the
@@ -402,15 +427,20 @@ class EventGenerationManager:
     get the typed object.
     """
 
-    def __init__(self, generator, object_cls, require_object=True):
+    def __init__(
+        self,
+        generator: Generator[Union["DagsterEvent", GeneratedContext], None, None],
+        object_cls: Type[GeneratedContext],
+        require_object: Optional[bool] = True,
+    ):
         self.generator = check.generator(generator)
-        self.object_cls = check.type_param(object_cls, "object_cls")
+        self.object_cls: Type[GeneratedContext] = check.type_param(object_cls, "object_cls")
         self.require_object = check.bool_param(require_object, "require_object")
-        self.object = None
+        self.object: Optional[GeneratedContext] = None
         self.did_setup = False
         self.did_teardown = False
 
-    def generate_setup_events(self):
+    def generate_setup_events(self) -> Iterator["DagsterEvent"]:
         self.did_setup = True
         try:
             while self.object is None:
@@ -428,35 +458,24 @@ class EventGenerationManager:
                     "generator never yielded object of type {}".format(self.object_cls.__name__),
                 )
 
-    def get_object(self):
+    def get_object(self) -> GeneratedContext:
         if not self.did_setup:
             check.failed("Called `get_object` before `generate_setup_events`")
-        return self.object
+        return cast(GeneratedContext, self.object)
 
-    def generate_teardown_events(self):
+    def generate_teardown_events(self) -> Iterator["DagsterEvent"]:
         self.did_teardown = True
         if self.object:
             yield from self.generator
 
 
-def utc_timezone():
-    if sys.version_info.major >= 3 and sys.version_info.minor >= 2:
-        from datetime import timezone
-
-        return timezone.utc
-    else:
-        import pytz
-
-        return pytz.utc
-
-
 def utc_datetime_from_timestamp(timestamp):
-    tz = utc_timezone()
+    tz = timezone.utc
     return datetime.datetime.fromtimestamp(timestamp, tz=tz)
 
 
 def utc_datetime_from_naive(dt):
-    tz = utc_timezone()
+    tz = timezone.utc
     return dt.replace(tzinfo=tz)
 
 
