@@ -1,12 +1,13 @@
 import datetime
+from typing import Callable, Union
 
 import pendulum
 from dagster import check
 from dagster.core.definitions.partition import Partition, PartitionSetDefinition
-from dagster.core.definitions.schedule import ScheduleExecutionContext
+from dagster.core.definitions.run_request import SkipReason
+from dagster.core.definitions.schedule_definition import ScheduleEvaluationContext
 from dagster.core.errors import DagsterInvariantViolationError
-from dagster.seven import PendulumDateTime, to_timezone
-from dagster.utils.schedules import schedule_execution_time_iterator
+from dagster.seven.compat.pendulum import PendulumDateTime, to_timezone
 
 DEFAULT_MONTHLY_FORMAT = "%Y-%m"
 DEFAULT_DATE_FORMAT = "%Y-%m-%d"
@@ -14,73 +15,6 @@ DEFAULT_HOURLY_FORMAT_WITHOUT_TIMEZONE = "%Y-%m-%d-%H:%M"
 DEFAULT_HOURLY_FORMAT_WITH_TIMEZONE = DEFAULT_HOURLY_FORMAT_WITHOUT_TIMEZONE + "%z"
 
 
-def schedule_partition_range(
-    start,
-    end,
-    cron_schedule,
-    fmt,
-    timezone,
-    execution_time_to_partition_fn,
-    inclusive=False,
-):
-    check.inst_param(start, "start", datetime.datetime)
-    check.opt_inst_param(end, "end", datetime.datetime)
-    check.str_param(cron_schedule, "cron_schedule")
-    check.str_param(fmt, "fmt")
-    check.opt_str_param(timezone, "timezone")
-    check.callable_param(execution_time_to_partition_fn, "execution_time_to_partition_fn")
-    check.opt_bool_param(inclusive, "inclusive")
-
-    if end and start > end:
-        raise DagsterInvariantViolationError(
-            'Selected date range start "{start}" is after date range end "{end}'.format(
-                start=start.strftime(fmt),
-                end=end.strftime(fmt),
-            )
-        )
-
-    def get_schedule_range_partitions(current_time=None):
-        check.opt_inst_param(current_time, "current_time", datetime.datetime)
-        tz = timezone if timezone else "UTC"
-        _start = (
-            to_timezone(start, tz)
-            if isinstance(start, PendulumDateTime)
-            else pendulum.instance(start, tz=tz)
-        )
-
-        if end:
-            _end = end
-        elif current_time:
-            _end = current_time
-        else:
-            _end = pendulum.now(tz)
-
-        # coerce to the definition timezone
-        if isinstance(_end, PendulumDateTime):
-            _end = to_timezone(_end, tz)
-        else:
-            _end = pendulum.instance(_end, tz=tz)
-
-        end_timestamp = _end.timestamp()
-
-        partitions = []
-        for next_time in schedule_execution_time_iterator(_start.timestamp(), cron_schedule, tz):
-
-            partition_time = execution_time_to_partition_fn(next_time)
-
-            if partition_time.timestamp() > end_timestamp:
-                break
-
-            if partition_time.timestamp() < _start.timestamp():
-                continue
-
-            partitions.append(Partition(value=partition_time, name=partition_time.strftime(fmt)))
-
-        return partitions if inclusive else partitions[:-1]
-
-    return get_schedule_range_partitions
-
-
 def date_partition_range(
     start,
     end=None,
@@ -168,7 +102,7 @@ def date_partition_range(
 
 def identity_partition_selector(context, partition_set_def):
     """Utility function for supplying a partition selector when creating a schedule from a
-    partition set made of ``datetime``s that assumes the schedule always executes at the
+    partition set made of ``datetime`` objects that assumes the schedule always executes at the
     partition time.
 
     It's important that the cron string passed into ``create_schedule_definition`` match
@@ -199,13 +133,15 @@ def identity_partition_selector(context, partition_set_def):
     return create_offset_partition_selector(lambda d: d)(context, partition_set_def)
 
 
-def create_offset_partition_selector(execution_time_to_partition_fn):
+def create_offset_partition_selector(
+    execution_time_to_partition_fn,
+) -> Callable[[ScheduleEvaluationContext, PartitionSetDefinition], Union[Partition, SkipReason]]:
     """Utility function for supplying a partition selector when creating a schedule from a
-    partition set made of `datetime`s that assumes a fixed time offset between the partition
-    time and the time at which the schedule executes.
+    partition set made of ``datetime`` objects that assumes a fixed time offset between the
+    partition time and the time at which the schedule executes.
 
     It's important to keep the cron string that's supplied to
-    `PartitionSetDefinition.create_schedule_definition` in sync with the offset that's
+    ``PartitionSetDefinition.create_schedule_definition`` in sync with the offset that's
     supplied to this function. For example, a schedule created from a partition set with
     partitions for each day at midnight that fills in the partition for day N at day N+1 at
     10:00AM would create the partition selector as follows:
@@ -232,32 +168,50 @@ def create_offset_partition_selector(execution_time_to_partition_fn):
 
     Args:
         execution_time_to_partition_fn (Callable[[datetime.datetime], datetime.datetime]): A
-        function that maps the execution time of the schedule to the partition time.
+            function that maps the execution time of the schedule to the partition time.
     """
 
     check.callable_param(execution_time_to_partition_fn, "execution_time_to_partition_fn")
 
-    def offset_partition_selector(context, partition_set_def):
-        check.inst_param(context, "context", ScheduleExecutionContext)
-        check.inst_param(partition_set_def, "partition_set_def", PartitionSetDefinition)
+    def offset_partition_selector(
+        context: ScheduleEvaluationContext, partition_set_def: PartitionSetDefinition
+    ) -> Union[Partition, SkipReason]:
+        no_partitions_skip_reason = SkipReason(
+            "Partition selector did not return a partition. Make sure that the timezone "
+            "on your partition set matches your execution timezone."
+        )
+
+        earliest_possible_partition = next(iter(partition_set_def.get_partitions(None)), None)
+        if not earliest_possible_partition:
+            return no_partitions_skip_reason
+
+        valid_partitions = partition_set_def.get_partitions(context.scheduled_execution_time)
 
         if not context.scheduled_execution_time:
-            partitions = partition_set_def.get_partitions()
-            if not partitions:
-                return None
-            return partitions[-1]
+            if not valid_partitions:
+                return no_partitions_skip_reason
+            return valid_partitions[-1]
 
         partition_time = execution_time_to_partition_fn(context.scheduled_execution_time)
 
-        for partition in reversed(
-            partition_set_def.get_partitions(context.scheduled_execution_time)
-        ):
+        if partition_time < earliest_possible_partition.value:
+            return SkipReason(
+                f"Your partition ({partition_time.isoformat()}) is before the beginning of "
+                f"the partition set ({earliest_possible_partition.value.isoformat()}). "
+                "Verify your schedule's start_date is correct."
+            )
+
+        if partition_time > valid_partitions[-1].value:
+            return SkipReason(
+                f"Your partition ({partition_time.isoformat()}) is after the end of "
+                f"the partition set ({valid_partitions[-1].value.isoformat()}). "
+                "Verify your schedule's end_date is correct."
+            )
+
+        for partition in valid_partitions:
             if partition.value.isoformat() == partition_time.isoformat():
                 return partition
 
-            if partition.value < partition_time:
-                break
-
-        return None
+        return no_partitions_skip_reason
 
     return offset_partition_selector
