@@ -6,32 +6,38 @@ from threading import Thread
 
 import pytest
 from dagster import (
+    DynamicOut,
+    DynamicOutput,
     Field,
     ModeDefinition,
     RetryRequested,
     String,
     execute_pipeline,
     execute_pipeline_iterator,
+    fs_io_manager,
+    job,
+    op,
     pipeline,
     reconstructable,
     resource,
     solid,
 )
 from dagster.core.definitions.no_step_launcher import no_step_launcher
-from dagster.core.errors import DagsterExecutionInterruptedError
 from dagster.core.events import DagsterEventType
 from dagster.core.execution.api import create_execution_plan
-from dagster.core.execution.context_creation_pipeline import PipelineExecutionContextManager
+from dagster.core.execution.context_creation_pipeline import PlanExecutionContextManager
 from dagster.core.execution.plan.external_step import (
     LocalExternalStepLauncher,
     local_external_step_launcher,
     step_context_to_step_run_ref,
     step_run_ref_to_step_context,
 )
+from dagster.core.execution.retries import RetryMode
 from dagster.core.instance import DagsterInstance
 from dagster.core.storage.pipeline_run import PipelineRun
+from dagster.core.test_utils import instance_for_test
 from dagster.utils import safe_tempfile_path, send_interrupt
-from dagster.utils.merger import deep_merge_dicts
+from dagster.utils.merger import deep_merge_dicts, merge_dicts
 
 RUN_CONFIG_BASE = {"solids": {"return_two": {"config": {"a": "b"}}}}
 
@@ -44,11 +50,13 @@ def make_run_config(scratch_dir, mode):
     return deep_merge_dicts(
         RUN_CONFIG_BASE,
         {
-            "resources": {
-                step_launcher_resource_key: {"config": {"scratch_dir": scratch_dir}}
-                for step_launcher_resource_key in step_launcher_resource_keys
-            },
-            "intermediate_storage": {"filesystem": {"config": {"base_dir": scratch_dir}}},
+            "resources": merge_dicts(
+                {"io_manager": {"config": {"base_dir": scratch_dir}}},
+                {
+                    step_launcher_resource_key: {"config": {"scratch_dir": scratch_dir}}
+                    for step_launcher_resource_key in step_launcher_resource_keys
+                },
+            ),
         },
     )
 
@@ -68,6 +76,36 @@ def request_retry_local_external_step_launcher(context):
     return RequestRetryLocalExternalStepLauncher(**context.resource_config)
 
 
+def define_dynamic_job():
+    from typing import List
+
+    @op(required_resource_keys={"first_step_launcher"}, out=DynamicOut(int))
+    def dynamic_outs():
+        for i in range(0, 3):
+            yield DynamicOutput(value=i, mapping_key=f"num_{i}")
+
+    @op(required_resource_keys={"second_step_launcher"})
+    def increment(i):
+        return i + 1
+
+    @op
+    def total(ins: List[int]):
+        return sum(ins)
+
+    @job(
+        resource_defs={
+            "first_step_launcher": local_external_step_launcher,
+            "second_step_launcher": local_external_step_launcher,
+            "io_manager": fs_io_manager,
+        }
+    )
+    def my_job():
+        all_incs = dynamic_outs().map(increment)
+        total(all_incs.collect())
+
+    return my_job
+
+
 def define_basic_pipeline():
     @solid(required_resource_keys=set(["first_step_launcher"]), config_schema={"a": Field(str)})
     def return_two(_):
@@ -84,6 +122,7 @@ def define_basic_pipeline():
                 resource_defs={
                     "first_step_launcher": local_external_step_launcher,
                     "second_step_launcher": local_external_step_launcher,
+                    "io_manager": fs_io_manager,
                 },
             ),
             ModeDefinition(
@@ -91,6 +130,7 @@ def define_basic_pipeline():
                 resource_defs={
                     "first_step_launcher": no_step_launcher,
                     "second_step_launcher": local_external_step_launcher,
+                    "io_manager": fs_io_manager,
                 },
             ),
             ModeDefinition(
@@ -98,6 +138,7 @@ def define_basic_pipeline():
                 resource_defs={
                     "first_step_launcher": request_retry_local_external_step_launcher,
                     "second_step_launcher": request_retry_local_external_step_launcher,
+                    "io_manager": fs_io_manager,
                 },
             ),
         ]
@@ -128,6 +169,7 @@ def define_sleepy_pipeline():
                 "external",
                 resource_defs={
                     "first_step_launcher": local_external_step_launcher,
+                    "io_manager": fs_io_manager,
                 },
             ),
         ]
@@ -146,15 +188,17 @@ def initialize_step_context(scratch_dir, instance):
         mode="external",
     )
 
-    plan = create_execution_plan(
-        reconstructable(define_basic_pipeline), pipeline_run.run_config, mode="external"
-    )
+    recon_pipeline = reconstructable(define_basic_pipeline)
 
-    initialization_manager = PipelineExecutionContextManager(
-        plan,
-        pipeline_run.run_config,
-        pipeline_run,
-        instance,
+    plan = create_execution_plan(recon_pipeline, pipeline_run.run_config, mode="external")
+
+    initialization_manager = PlanExecutionContextManager(
+        pipeline=recon_pipeline,
+        execution_plan=plan,
+        run_config=pipeline_run.run_config,
+        pipeline_run=pipeline_run,
+        instance=instance,
+        retry_mode=RetryMode.DISABLED,
     )
     for _ in initialization_manager.prepare_context():
         pass
@@ -173,7 +217,6 @@ def test_step_context_to_step_run_ref():
         assert step_run_ref.run_id == step_context.pipeline_run.run_id
 
         rehydrated_step_context = step_run_ref_to_step_context(step_run_ref, instance)
-        assert rehydrated_step_context.required_resource_keys == step_context.required_resource_keys
         rehydrated_step = rehydrated_step_context.step
         assert rehydrated_step.pipeline_name == step.pipeline_name
         assert rehydrated_step.step_inputs == step.step_inputs
@@ -209,6 +252,27 @@ def test_pipeline(mode):
         assert result.result_for_solid("add_one").output_value() == 3
 
 
+def test_dynamic_job():
+    with tempfile.TemporaryDirectory() as tmpdir:
+        with instance_for_test() as instance:
+            result = execute_pipeline(
+                pipeline=reconstructable(define_dynamic_job),
+                run_config={
+                    "resources": {
+                        "first_step_launcher": {
+                            "config": {"scratch_dir": tmpdir},
+                        },
+                        "second_step_launcher": {
+                            "config": {"scratch_dir": tmpdir},
+                        },
+                        "io_manager": {"config": {"base_dir": tmpdir}},
+                    }
+                },
+                instance=instance,
+            )
+            assert result.result_for_solid("total").output_value() == 6
+
+
 def test_launcher_requests_retry():
     mode = "request_retry"
     with tempfile.TemporaryDirectory() as tmpdir:
@@ -238,8 +302,12 @@ def test_interrupt_step_launcher(mode):
     with tempfile.TemporaryDirectory() as tmpdir:
         with safe_tempfile_path() as success_tempfile:
             sleepy_run_config = {
-                "resources": {"first_step_launcher": {"config": {"scratch_dir": tmpdir}}},
-                "intermediate_storage": {"filesystem": {"config": {"base_dir": tmpdir}}},
+                "resources": {
+                    "first_step_launcher": {
+                        "config": {"scratch_dir": tmpdir},
+                    },
+                    "io_manager": {"config": {"base_dir": tmpdir}},
+                },
                 "solids": {"sleepy_solid": {"config": {"tempfile": success_tempfile}}},
             }
 
@@ -249,19 +317,12 @@ def test_interrupt_step_launcher(mode):
 
             results = []
 
-            received_interrupt = False
-
-            try:
-                for result in execute_pipeline_iterator(
-                    pipeline=reconstructable(define_sleepy_pipeline),
-                    mode=mode,
-                    run_config=sleepy_run_config,
-                ):
-                    results.append(result.event_type)
-            except DagsterExecutionInterruptedError:
-                received_interrupt = True
-
-            assert received_interrupt
+            for result in execute_pipeline_iterator(
+                pipeline=reconstructable(define_sleepy_pipeline),
+                mode=mode,
+                run_config=sleepy_run_config,
+            ):
+                results.append(result.event_type)
 
             assert DagsterEventType.STEP_FAILURE in results
             assert DagsterEventType.PIPELINE_FAILURE in results
