@@ -9,14 +9,14 @@ snapshots = Snapshot()
 snapshots['test_create_execution_plan_with_dep 1'] = '''{
   "__class__": "ExecutionPlanSnapshot",
   "artifacts_persisted": true,
+  "executor_name": "in_process",
   "initial_known_state": null,
-  "pipeline_snapshot_id": "1d7f214f0584150df209c62a02074780527c95bb",
+  "pipeline_snapshot_id": "588501f8b7b5a1a7dd65fc0aa091bab26670e5c2",
   "snapshot_version": 1,
   "step_keys_to_execute": [
     "solid_one",
     "solid_two"
   ],
-  "step_output_versions": [],
   "steps": [
     {
       "__class__": "ExecutionStepSnap",
@@ -48,6 +48,7 @@ snapshots['test_create_execution_plan_with_dep 1'] = '''{
       "solid_handle_id": "solid_one",
       "step_handle": {
         "__class__": "StepHandle",
+        "key": "solid_one",
         "solid_handle": {
           "__class__": "SolidHandle",
           "name": "solid_one",
@@ -116,6 +117,7 @@ snapshots['test_create_execution_plan_with_dep 1'] = '''{
       "solid_handle_id": "solid_two",
       "step_handle": {
         "__class__": "StepHandle",
+        "key": "solid_two",
         "solid_handle": {
           "__class__": "SolidHandle",
           "name": "solid_two",
@@ -130,13 +132,13 @@ snapshots['test_create_execution_plan_with_dep 1'] = '''{
 snapshots['test_create_noop_execution_plan 1'] = '''{
   "__class__": "ExecutionPlanSnapshot",
   "artifacts_persisted": true,
+  "executor_name": "in_process",
   "initial_known_state": null,
-  "pipeline_snapshot_id": "0965b76124e758660317760c7e9bbc66282f33b0",
+  "pipeline_snapshot_id": "2290b3c558988d5ac3b68b109c7a6da33a237696",
   "snapshot_version": 1,
   "step_keys_to_execute": [
     "noop_solid"
   ],
-  "step_output_versions": [],
   "steps": [
     {
       "__class__": "ExecutionStepSnap",
@@ -168,6 +170,7 @@ snapshots['test_create_noop_execution_plan 1'] = '''{
       "solid_handle_id": "noop_solid",
       "step_handle": {
         "__class__": "StepHandle",
+        "key": "noop_solid",
         "solid_handle": {
           "__class__": "SolidHandle",
           "name": "noop_solid",
@@ -182,13 +185,13 @@ snapshots['test_create_noop_execution_plan 1'] = '''{
 snapshots['test_create_noop_execution_plan_with_tags 1'] = '''{
   "__class__": "ExecutionPlanSnapshot",
   "artifacts_persisted": true,
+  "executor_name": "in_process",
   "initial_known_state": null,
-  "pipeline_snapshot_id": "3248f027c9045c80d4d5124967cad459be986f0c",
+  "pipeline_snapshot_id": "bb5c233d49c4c6832472f7eea29ba6ec92b78463",
   "snapshot_version": 1,
   "step_keys_to_execute": [
     "noop_solid"
   ],
-  "step_output_versions": [],
   "steps": [
     {
       "__class__": "ExecutionStepSnap",
@@ -231,6 +234,7 @@ snapshots['test_create_noop_execution_plan_with_tags 1'] = '''{
       "solid_handle_id": "noop_solid",
       "step_handle": {
         "__class__": "StepHandle",
+        "key": "noop_solid",
         "solid_handle": {
           "__class__": "SolidHandle",
           "name": "noop_solid",
@@ -248,8 +252,9 @@ snapshots['test_create_noop_execution_plan_with_tags 1'] = '''{
 snapshots['test_create_with_composite 1'] = '''{
   "__class__": "ExecutionPlanSnapshot",
   "artifacts_persisted": true,
+  "executor_name": "in_process",
   "initial_known_state": null,
-  "pipeline_snapshot_id": "574214af42cfa7ee95c5720ce8cc862e2b440a04",
+  "pipeline_snapshot_id": "3aad6c74f16d0c802e9fc139412d5f9e1766f54a",
   "snapshot_version": 1,
   "step_keys_to_execute": [
     "comp_1.return_one",
@@ -258,7 +263,6 @@ snapshots['test_create_with_composite 1'] = '''{
     "comp_2.add_one",
     "add"
   ],
-  "step_output_versions": [],
   "steps": [
     {
       "__class__": "ExecutionStepSnap",
@@ -349,6 +353,7 @@ snapshots['test_create_with_composite 1'] = '''{
       "solid_handle_id": "add",
       "step_handle": {
         "__class__": "StepHandle",
+        "key": "add",
         "solid_handle": {
           "__class__": "SolidHandle",
           "name": "add",
@@ -425,6 +430,7 @@ snapshots['test_create_with_composite 1'] = '''{
       "solid_handle_id": "comp_1.add_one",
       "step_handle": {
         "__class__": "StepHandle",
+        "key": "comp_1.add_one",
         "solid_handle": {
           "__class__": "SolidHandle",
           "name": "add_one",
@@ -471,6 +477,7 @@ snapshots['test_create_with_composite 1'] = '''{
       "solid_handle_id": "comp_1.return_one",
       "step_handle": {
         "__class__": "StepHandle",
+        "key": "comp_1.return_one",
         "solid_handle": {
           "__class__": "SolidHandle",
           "name": "return_one",
@@ -551,6 +558,7 @@ snapshots['test_create_with_composite 1'] = '''{
       "solid_handle_id": "comp_2.add_one",
       "step_handle": {
         "__class__": "StepHandle",
+        "key": "comp_2.add_one",
         "solid_handle": {
           "__class__": "SolidHandle",
           "name": "add_one",
@@ -597,6 +605,7 @@ snapshots['test_create_with_composite 1'] = '''{
       "solid_handle_id": "comp_2.return_one",
       "step_handle": {
         "__class__": "StepHandle",
+        "key": "comp_2.return_one",
         "solid_handle": {
           "__class__": "SolidHandle",
           "name": "return_one",
