@@ -1,16 +1,20 @@
 import pendulum
 import pytest
-from dagster.core.definitions.job import JobType
-from dagster.core.host_representation import RepositoryLocationHandleManager
-from dagster.core.host_representation.grpc_server_registry import ProcessGrpcServerRegistry
+from dagster.core.definitions.run_request import JobType
 from dagster.core.instance import DagsterInstance
 from dagster.core.scheduler.job import JobState, JobStatus, JobTickStatus
 from dagster.core.storage.pipeline_run import PipelineRunStatus
 from dagster.core.storage.tags import RUN_KEY_TAG, SENSOR_NAME_TAG
-from dagster.core.test_utils import cleanup_test_instance, get_crash_signals
+from dagster.core.test_utils import (
+    cleanup_test_instance,
+    create_test_daemon_workspace,
+    get_crash_signals,
+    get_logger_output_from_capfd,
+)
 from dagster.daemon import get_default_daemon_logger
 from dagster.daemon.sensor import execute_sensor_iteration
-from dagster.seven import IS_WINDOWS, create_pendulum_time, multiprocessing, to_timezone
+from dagster.seven import IS_WINDOWS, multiprocessing
+from dagster.seven.compat.pendulum import create_pendulum_time, to_timezone
 
 from .test_sensor_run import instance_with_sensors, repos, wait_for_all_runs_to_start
 
@@ -18,18 +22,15 @@ from .test_sensor_run import instance_with_sensors, repos, wait_for_all_runs_to_
 def _test_launch_sensor_runs_in_subprocess(instance_ref, execution_datetime, debug_crash_flags):
     with DagsterInstance.from_ref(instance_ref) as instance:
         try:
-            with pendulum.test(execution_datetime), ProcessGrpcServerRegistry(
-                wait_for_processes_on_exit=True
-            ) as grpc_server_registry:
-                with RepositoryLocationHandleManager(grpc_server_registry) as handle_manager:
-                    list(
-                        execute_sensor_iteration(
-                            instance,
-                            get_default_daemon_logger("SensorDaemon"),
-                            handle_manager,
-                            debug_crash_flags=debug_crash_flags,
-                        )
+            with pendulum.test(execution_datetime), create_test_daemon_workspace() as workspace:
+                list(
+                    execute_sensor_iteration(
+                        instance,
+                        get_default_daemon_logger("SensorDaemon"),
+                        workspace,
+                        debug_crash_flags=debug_crash_flags,
                     )
+                )
         finally:
             cleanup_test_instance(instance)
 
@@ -66,7 +67,7 @@ def test_failure_before_run_created(external_repo_context, crash_location, crash
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 1
             assert ticks[0].status == JobTickStatus.SKIPPED
-            captured = capfd.readouterr()
+            capfd.readouterr()
 
             # create a starting tick, but crash
             debug_crash_flags = {external_sensor.name: {crash_location: crash_signal}}
@@ -79,7 +80,7 @@ def test_failure_before_run_created(external_repo_context, crash_location, crash
 
             assert launch_process.exitcode != 0
 
-            captured = capfd.readouterr()
+            capfd.readouterr()
 
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 2
@@ -101,13 +102,11 @@ def test_failure_before_run_created(external_repo_context, crash_location, crash
 
             assert instance.get_runs_count() == 1
             run = instance.get_runs()[0]
-            captured = capfd.readouterr()
             assert (
-                captured.out.replace("\r\n", "\n")
-                == f"""2019-02-27 18:01:03 - SensorDaemon - INFO - Checking for new runs for sensor: simple_sensor
-2019-02-27 18:01:03 - SensorDaemon - INFO - Launching run for simple_sensor
-2019-02-27 18:01:03 - SensorDaemon - INFO - Completed launch of run {run.run_id} for simple_sensor
-"""
+                get_logger_output_from_capfd(capfd, "SensorDaemon")
+                == f"""2019-02-27 18:01:03 -0600 - SensorDaemon - INFO - Checking for new runs for sensor: simple_sensor
+2019-02-27 18:01:03 -0600 - SensorDaemon - INFO - Launching run for simple_sensor
+2019-02-27 18:01:03 -0600 - SensorDaemon - INFO - Completed launch of run {run.run_id} for simple_sensor"""
             )
 
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
