@@ -23,10 +23,16 @@ from dagster import (
     solid,
 )
 from dagster.core.definitions.decorators.sensor import asset_sensor, sensor
-from dagster.core.definitions.pipeline_sensor import run_status_sensor
 from dagster.core.definitions.reconstructable import ReconstructableRepository
 from dagster.core.definitions.run_request import JobType
-from dagster.core.definitions.sensor import DEFAULT_SENSOR_DAEMON_INTERVAL, RunRequest, SkipReason
+from dagster.core.definitions.run_status_sensor_definition import run_status_sensor
+from dagster.core.definitions.sensor_definition import (
+    DEFAULT_SENSOR_DAEMON_INTERVAL,
+    RunRequest,
+    SkipReason,
+)
+from dagster.core.events import DagsterEvent, DagsterEventType
+from dagster.core.events.log import EventLogEntry
 from dagster.core.execution.api import execute_pipeline
 from dagster.core.host_representation import (
     ExternalJobOrigin,
@@ -34,13 +40,16 @@ from dagster.core.host_representation import (
     InProcessRepositoryLocationOrigin,
     ManagedGrpcPythonEnvRepositoryLocationOrigin,
 )
-from dagster.core.host_representation.grpc_server_registry import ProcessGrpcServerRegistry
 from dagster.core.instance import DagsterInstance
 from dagster.core.scheduler.job import JobState, JobStatus, JobTickStatus
+from dagster.core.storage.event_log.base import EventRecordsFilter
 from dagster.core.storage.pipeline_run import PipelineRunStatus
-from dagster.core.test_utils import instance_for_test
+from dagster.core.test_utils import (
+    create_test_daemon_workspace,
+    get_logger_output_from_capfd,
+    instance_for_test,
+)
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
-from dagster.core.workspace.dynamic_workspace import DynamicWorkspace
 from dagster.daemon import get_default_daemon_logger
 from dagster.daemon.controller import (
     DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS,
@@ -61,7 +70,7 @@ def the_pipeline():
     the_solid()
 
 
-@graph
+@graph()
 def the_graph():
     the_solid()
 
@@ -79,7 +88,7 @@ def config_pipeline():
     config_solid()
 
 
-@graph
+@graph()
 def config_graph():
     config_solid()
 
@@ -119,7 +128,7 @@ def failure_pipeline():
     failure_solid()
 
 
-@graph
+@graph()
 def failure_graph():
     failure_solid()
 
@@ -306,10 +315,9 @@ def the_other_repo():
 @contextmanager
 def instance_with_sensors(external_repo_context, overrides=None):
     with instance_for_test(overrides) as instance:
-        with ProcessGrpcServerRegistry() as grpc_server_registry:
-            with DynamicWorkspace(grpc_server_registry) as workspace:
-                with external_repo_context() as external_repo:
-                    yield (instance, workspace, external_repo)
+        with create_test_daemon_workspace() as workspace:
+            with external_repo_context() as external_repo:
+                yield (instance, workspace, external_repo)
 
 
 @contextmanager
@@ -439,12 +447,10 @@ def test_simple_sensor(external_repo_context, capfd):
                 JobTickStatus.SKIPPED,
             )
 
-            captured = capfd.readouterr()
             assert (
-                captured.out
-                == """2019-02-27 17:59:59 - SensorDaemon - INFO - Checking for new runs for sensor: simple_sensor
-2019-02-27 17:59:59 - SensorDaemon - INFO - Sensor returned false for simple_sensor, skipping
-"""
+                get_logger_output_from_capfd(capfd, "SensorDaemon")
+                == """2019-02-27 17:59:59 -0600 - SensorDaemon - INFO - Checking for new runs for sensor: simple_sensor
+2019-02-27 17:59:59 -0600 - SensorDaemon - INFO - Sensor returned false for simple_sensor, skipping"""
             )
 
             freeze_datetime = freeze_datetime.add(seconds=30)
@@ -469,13 +475,11 @@ def test_simple_sensor(external_repo_context, capfd):
                 [run.run_id],
             )
 
-            captured = capfd.readouterr()
             assert (
-                captured.out
-                == """2019-02-27 18:00:29 - SensorDaemon - INFO - Checking for new runs for sensor: simple_sensor
-2019-02-27 18:00:29 - SensorDaemon - INFO - Launching run for simple_sensor
-2019-02-27 18:00:29 - SensorDaemon - INFO - Completed launch of run {run_id} for simple_sensor
-""".format(
+                get_logger_output_from_capfd(capfd, "SensorDaemon")
+                == """2019-02-27 18:00:29 -0600 - SensorDaemon - INFO - Checking for new runs for sensor: simple_sensor
+2019-02-27 18:00:29 -0600 - SensorDaemon - INFO - Launching run for simple_sensor
+2019-02-27 18:00:29 -0600 - SensorDaemon - INFO - Completed launch of run {run_id} for simple_sensor""".format(
                     run_id=run.run_id
                 )
             )
@@ -643,11 +647,11 @@ def test_wrong_config_sensor(external_repo_context, capfd):
                 freeze_datetime,
                 JobTickStatus.FAILURE,
                 [],
-                "Error in config for pipeline the_pipeline",
+                "Error in config for pipeline",
             )
 
             captured = capfd.readouterr()
-            assert ("Error in config for pipeline the_pipeline") in captured.out
+            assert ("Error in config for pipeline") in captured.out
 
             # Error repeats on subsequent ticks
 
@@ -662,11 +666,11 @@ def test_wrong_config_sensor(external_repo_context, capfd):
                 freeze_datetime,
                 JobTickStatus.FAILURE,
                 [],
-                "Error in config for pipeline the_pipeline",
+                "Error in config for pipeline",
             )
 
             captured = capfd.readouterr()
-            assert ("Error in config for pipeline the_pipeline") in captured.out
+            assert ("Error in config for pipeline") in captured.out
 
 
 @pytest.mark.parametrize("external_repo_context", repos())
@@ -1441,6 +1445,16 @@ def default_storage_config_fn(_):
     return {}
 
 
+def sql_event_log_storage_config_fn(temp_dir):
+    return {
+        "event_log_storage": {
+            "module": "dagster.core.storage.event_log",
+            "class": "ConsolidatedSqliteEventLogStorage",
+            "config": {"base_dir": temp_dir},
+        },
+    }
+
+
 @pytest.mark.parametrize("external_repo_context", repos())
 @pytest.mark.parametrize(
     "storage_config_fn",
@@ -1543,6 +1557,76 @@ def test_run_status_sensor_interleave(external_repo_context, storage_config_fn):
                 assert ticks[0].origin_run_ids[0] == run1.run_id
 
 
+@pytest.mark.parametrize("external_repo_context", repos())
+@pytest.mark.parametrize("storage_config_fn", [sql_event_log_storage_config_fn])
+def test_pipeline_failure_sensor_empty_run_records(external_repo_context, storage_config_fn):
+    freeze_datetime = pendulum.now()
+    with tempfile.TemporaryDirectory() as temp_dir:
+
+        with instance_with_sensors(
+            external_repo_context, overrides=storage_config_fn(temp_dir)
+        ) as (
+            instance,
+            workspace,
+            external_repo,
+        ):
+
+            with pendulum.test(freeze_datetime):
+                failure_sensor = external_repo.get_external_sensor("my_pipeline_failure_sensor")
+                instance.start_sensor(failure_sensor)
+
+                evaluate_sensors(instance, workspace)
+
+                ticks = instance.get_job_ticks(failure_sensor.get_external_origin_id())
+                assert len(ticks) == 1
+                validate_tick(
+                    ticks[0],
+                    failure_sensor,
+                    freeze_datetime,
+                    JobTickStatus.SKIPPED,
+                )
+
+                freeze_datetime = freeze_datetime.add(seconds=60)
+                time.sleep(1)
+
+            with pendulum.test(freeze_datetime):
+                # create a mismatch between event storage and run storage
+                instance.event_log_storage.store_event(
+                    EventLogEntry(
+                        None,
+                        "fake failure event",
+                        "debug",
+                        "",
+                        "fake_run_id",
+                        time.time(),
+                        dagster_event=DagsterEvent(
+                            DagsterEventType.PIPELINE_FAILURE.value,
+                            "foo",
+                        ),
+                    )
+                )
+                runs = instance.get_runs()
+                assert len(runs) == 0
+                failure_events = instance.get_event_records(
+                    EventRecordsFilter(event_type=DagsterEventType.PIPELINE_FAILURE)
+                )
+                assert len(failure_events) == 1
+                freeze_datetime = freeze_datetime.add(seconds=60)
+
+            with pendulum.test(freeze_datetime):
+                # shouldn't fire the failure sensor due to the mismatch
+                evaluate_sensors(instance, workspace)
+
+                ticks = instance.get_job_ticks(failure_sensor.get_external_origin_id())
+                assert len(ticks) == 2
+                validate_tick(
+                    ticks[0],
+                    failure_sensor,
+                    freeze_datetime,
+                    JobTickStatus.SKIPPED,
+                )
+
+
 @pytest.mark.parametrize("external_repo_context", repos())
 def test_multi_job_sensor(external_repo_context):
     freeze_datetime = to_timezone(
