@@ -3,6 +3,7 @@ import os
 import re
 import sqlite3
 from collections import namedtuple
+from enum import Enum
 from gzip import GzipFile
 
 import pytest
@@ -24,12 +25,14 @@ from dagster.core.events.log import EventLogEntry
 from dagster.core.instance import DagsterInstance, InstanceRef
 from dagster.core.storage.event_log.migration import migrate_event_log_data
 from dagster.core.storage.event_log.sql_event_log import SqlEventLogStorage
+from dagster.core.storage.pipeline_run import DagsterRun, DagsterRunStatus
 from dagster.serdes.serdes import (
     WhitelistMap,
     _deserialize_json,
     _whitelist_for_serdes,
     deserialize_json_to_dagster_namedtuple,
     serialize_dagster_namedtuple,
+    serialize_value,
 )
 from dagster.utils.test import copy_directory
 
@@ -393,10 +396,6 @@ def test_0_10_0_schedule_wipe():
         assert "jobs" not in get_sqlite3_tables(db_path)
         assert "job_ticks" not in get_sqlite3_tables(db_path)
 
-        with pytest.raises(DagsterInstanceMigrationRequired):
-            with DagsterInstance.from_ref(InstanceRef.from_dir(test_dir)) as instance:
-                instance.optimize_for_dagit(statement_timeout=500)
-
         with DagsterInstance.from_ref(InstanceRef.from_dir(test_dir)) as instance:
             instance.upgrade()
 
@@ -434,7 +433,6 @@ def test_0_11_0_add_asset_columns():
             assert "last_materialization" in set(get_sqlite3_columns(db_path, "asset_keys"))
             assert "last_run_id" in set(get_sqlite3_columns(db_path, "asset_keys"))
             assert "asset_details" in set(get_sqlite3_columns(db_path, "asset_keys"))
-            instance.get_asset_tags(AssetKey("model"))
 
 
 def test_rename_event_log_entry():
@@ -484,7 +482,6 @@ def test_0_12_0_extract_asset_index_cols():
 
             execute_pipeline(asset_pipeline, instance=instance)
             assert storage.has_asset_key(AssetKey(["a"]))
-            old_tags = storage.get_asset_tags(AssetKey(["a"]))
 
             # wipe and leave asset wiped
             storage.wipe_asset(AssetKey(["b"]))
@@ -503,9 +500,7 @@ def test_0_12_0_extract_asset_index_cols():
             assert storage.has_asset_key(AssetKey(["a"]))
             assert not storage.has_asset_key(AssetKey(["b"]))
 
-            new_tags = storage.get_asset_tags(AssetKey(["a"]))
             new_keys = storage.all_asset_keys()
-            assert set(old_tags) == set(new_tags)
             assert set(old_keys) == set(new_keys)
 
             # make sure that storing assets still works
@@ -531,3 +526,50 @@ def test_solid_handle_node_handle():
     result = _deserialize_json(test_str, legacy_env)
     assert isinstance(result, SolidHandle)
     assert result.name == test_handle.name
+
+
+def test_pipeline_run_dagster_run():
+    # serialize in current code
+    test_run = DagsterRun(pipeline_name="test")
+    test_str = serialize_dagster_namedtuple(test_run)
+
+    # deserialize in "legacy" code
+    legacy_env = WhitelistMap.create()
+
+    @_whitelist_for_serdes(legacy_env)
+    class PipelineRun(
+        namedtuple(
+            "_PipelineRun",
+            "pipeline_name run_id run_config mode solid_selection solids_to_execute "
+            "step_keys_to_execute status tags root_run_id parent_run_id "
+            "pipeline_snapshot_id execution_plan_snapshot_id external_pipeline_origin "
+            "pipeline_code_origin",
+        )
+    ):
+        pass
+
+    @_whitelist_for_serdes(legacy_env)  # pylint: disable=unused-variable
+    class PipelineRunStatus(Enum):
+        QUEUED = "QUEUED"
+        NOT_STARTED = "NOT_STARTED"
+
+    result = _deserialize_json(test_str, legacy_env)
+    assert isinstance(result, PipelineRun)
+    assert result.pipeline_name == test_run.pipeline_name
+
+
+def test_pipeline_run_status_dagster_run_status():
+    # serialize in current code
+    test_status = DagsterRunStatus("QUEUED")
+    test_str = serialize_value(test_status)
+
+    # deserialize in "legacy" code
+    legacy_env = WhitelistMap.create()
+
+    @_whitelist_for_serdes(legacy_env)
+    class PipelineRunStatus(Enum):
+        QUEUED = "QUEUED"
+
+    result = _deserialize_json(test_str, legacy_env)
+    assert isinstance(result, PipelineRunStatus)
+    assert result.value == test_status.value
