@@ -1,13 +1,19 @@
 import pendulum
 import pytest
-from dagster.core.host_representation.grpc_server_registry import ProcessGrpcServerRegistry
 from dagster.core.instance import DagsterInstance
 from dagster.core.scheduler.job import JobTickStatus
 from dagster.core.storage.pipeline_run import PipelineRunStatus
 from dagster.core.storage.tags import PARTITION_NAME_TAG, SCHEDULED_EXECUTION_TIME_TAG
-from dagster.core.test_utils import cleanup_test_instance, get_crash_signals, get_terminate_signal
+from dagster.core.test_utils import (
+    cleanup_test_instance,
+    create_test_daemon_workspace,
+    get_crash_signals,
+    get_logger_output_from_capfd,
+    get_terminate_signal,
+)
 from dagster.scheduler.scheduler import launch_scheduled_runs
-from dagster.seven import IS_WINDOWS, create_pendulum_time, multiprocessing, to_timezone
+from dagster.seven import IS_WINDOWS, multiprocessing
+from dagster.seven.compat.pendulum import create_pendulum_time, to_timezone
 
 from .test_scheduler_run import (
     instance_with_schedules,
@@ -21,20 +27,20 @@ from .test_scheduler_run import (
 
 def _test_launch_scheduled_runs_in_subprocess(instance_ref, execution_datetime, debug_crash_flags):
     with DagsterInstance.from_ref(instance_ref) as instance:
-        with ProcessGrpcServerRegistry(wait_for_processes_on_exit=True) as grpc_server_registry:
-            try:
+        try:
+            with create_test_daemon_workspace() as workspace:
                 with pendulum.test(execution_datetime):
                     list(
                         launch_scheduled_runs(
                             instance,
-                            grpc_server_registry,
+                            workspace,
                             logger(),
                             pendulum.now("UTC"),
                             debug_crash_flags=debug_crash_flags,
                         )
                     )
-            finally:
-                cleanup_test_instance(instance)
+        finally:
+            cleanup_test_instance(instance)
 
 
 @pytest.mark.skipif(
@@ -75,12 +81,10 @@ def test_failure_recovery_before_run_created(
 
             assert scheduler_process.exitcode != 0
 
-            captured = capfd.readouterr()
             assert (
-                captured.out.replace("\r\n", "\n")
-                == """2019-02-26 18:00:00 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
-2019-02-26 18:00:00 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-27 00:00:00+0000
-"""
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-02-26 18:00:00 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
+2019-02-26 18:00:00 -0600 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-27 00:00:00 +0000"""
             )
 
             ticks = instance.get_job_ticks(external_schedule.get_external_origin_id())
@@ -116,14 +120,12 @@ def test_failure_recovery_before_run_created(
                 JobTickStatus.SUCCESS,
                 [instance.get_runs()[0].run_id],
             )
-            captured = capfd.readouterr()
             assert (
-                captured.out.replace("\r\n", "\n")
-                == """2019-02-26 18:05:00 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
-2019-02-26 18:05:00 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-27 00:00:00+0000
-2019-02-26 18:05:00 - SchedulerDaemon - INFO - Resuming previously interrupted schedule execution
-2019-02-26 18:05:00 - SchedulerDaemon - INFO - Completed scheduled launch of run {run_id} for simple_schedule
-""".format(
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-02-26 18:05:00 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
+2019-02-26 18:05:00 -0600 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-27 00:00:00 +0000
+2019-02-26 18:05:00 -0600 - SchedulerDaemon - INFO - Resuming previously interrupted schedule execution
+2019-02-26 18:05:00 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {run_id} for simple_schedule""".format(
                     run_id=instance.get_runs()[0].run_id
                 )
             )
