@@ -3,7 +3,6 @@ import os
 import random
 import string
 import sys
-import tempfile
 import time
 from contextlib import contextmanager
 
@@ -18,32 +17,42 @@ from dagster import (
     ScheduleDefinition,
     daily_schedule,
     hourly_schedule,
+    job,
+    op,
     pipeline,
     repository,
     schedule,
-    seven,
     solid,
 )
 from dagster.core.definitions.reconstructable import ReconstructableRepository
 from dagster.core.definitions.run_request import RunRequest
-from dagster.core.errors import DagsterScheduleWipeRequired
 from dagster.core.host_representation import (
     ExternalJobOrigin,
     ExternalRepositoryOrigin,
     InProcessRepositoryLocationOrigin,
     ManagedGrpcPythonEnvRepositoryLocationOrigin,
 )
-from dagster.core.host_representation.grpc_server_registry import ProcessGrpcServerRegistry
-from dagster.core.scheduler.job import JobState, JobStatus, JobTickStatus, JobType, ScheduleJobData
-from dagster.core.storage.pipeline_run import PipelineRunStatus, PipelineRunsFilter
+from dagster.core.scheduler.job import (
+    JobState,
+    JobStatus,
+    JobTickData,
+    JobTickStatus,
+    JobType,
+    ScheduleJobData,
+)
+from dagster.core.storage.pipeline_run import (
+    IN_PROGRESS_RUN_STATUSES,
+    PipelineRunStatus,
+    PipelineRunsFilter,
+)
 from dagster.core.storage.tags import PARTITION_NAME_TAG, SCHEDULED_EXECUTION_TIME_TAG
 from dagster.core.test_utils import (
+    create_test_daemon_workspace,
+    get_logger_output_from_capfd,
     instance_for_test,
-    instance_for_test_tempdir,
     mock_system_timezone,
 )
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
-from dagster.core.workspace.dynamic_workspace import DynamicWorkspace
 from dagster.daemon import get_default_daemon_logger
 from dagster.scheduler.scheduler import launch_scheduled_runs
 from dagster.seven.compat.pendulum import create_pendulum_time, to_timezone
@@ -326,7 +335,7 @@ manual_partition = PartitionSetDefinition(
     pipeline_name="two_step_pipeline",
     # selects only second step
     solid_selection=["end"],
-    partition_fn=lambda: ["one"],
+    partition_fn=lambda: [Partition("one")],
     # includes config for first step - test that it is ignored
     run_config_fn_for_partition=lambda _: {"solids": {"start": {"inputs": {"x": {"value": 4}}}}},
 )
@@ -336,6 +345,23 @@ manual_partition_schedule = manual_partition.create_schedule_definition(
 )
 
 
+def define_default_config_job():
+    @op(config_schema=str)
+    def my_op(context):
+        assert context.op_config == "foo"
+
+    @job(config={"ops": {"my_op": {"config": "foo"}}})
+    def default_config_job():
+        my_op()
+
+    return default_config_job
+
+
+default_config_schedule = ScheduleDefinition(
+    name="default_config_schedule", cron_schedule="* * * * *", job=define_default_config_job()
+)
+
+
 @repository
 def the_repo():
     return [
@@ -362,6 +388,7 @@ def the_repo():
         large_schedule,
         two_step_pipeline,
         manual_partition_schedule,
+        default_config_schedule,
     ]
 
 
@@ -386,10 +413,9 @@ def logger():
 @contextmanager
 def instance_with_schedules(external_repo_context, overrides=None):
     with schedule_instance(overrides) as instance:
-        with ProcessGrpcServerRegistry() as grpc_server_registry:
-            with DynamicWorkspace(grpc_server_registry) as workspace:
-                with external_repo_context() as external_repo:
-                    yield (instance, workspace, external_repo)
+        with create_test_daemon_workspace() as workspace:
+            with external_repo_context() as external_repo:
+                yield (instance, workspace, external_repo)
 
 
 @contextmanager
@@ -504,13 +530,10 @@ def test_simple_schedule(external_repo_context, capfd):
             ticks = instance.get_job_ticks(schedule_origin.get_id())
             assert len(ticks) == 0
 
-            captured = capfd.readouterr()
-
             assert (
-                captured.out
-                == """2019-02-27 17:59:59 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
-2019-02-27 17:59:59 - SchedulerDaemon - INFO - No new runs for simple_schedule
-"""
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-02-27 17:59:59 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
+2019-02-27 17:59:59 -0600 - SchedulerDaemon - INFO - No new runs for simple_schedule"""
             )
 
         freeze_datetime = freeze_datetime.add(seconds=2)
@@ -545,14 +568,11 @@ def test_simple_schedule(external_repo_context, capfd):
                 partition_time=create_pendulum_time(2019, 2, 27),
             )
 
-            captured = capfd.readouterr()
-
             assert (
-                captured.out
-                == """2019-02-27 18:00:01 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
-2019-02-27 18:00:01 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-28 00:00:00+0000
-2019-02-27 18:00:01 - SchedulerDaemon - INFO - Completed scheduled launch of run {run_id} for simple_schedule
-""".format(
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
+2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-28 00:00:00 +0000
+2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {run_id} for simple_schedule""".format(
                     run_id=instance.get_runs()[0].run_id
                 )
             )
@@ -610,13 +630,12 @@ def test_simple_schedule(external_repo_context, capfd):
             assert "2019-02-28" in runs_by_partition
             assert "2019-03-01" in runs_by_partition
 
-            captured = capfd.readouterr()
-
-            assert captured.out == """2019-03-01 18:00:03 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
-2019-03-01 18:00:03 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at the following times: 2019-03-01 00:00:00+0000, 2019-03-02 00:00:00+0000
-2019-03-01 18:00:03 - SchedulerDaemon - INFO - Completed scheduled launch of run {first_run_id} for simple_schedule
-2019-03-01 18:00:03 - SchedulerDaemon - INFO - Completed scheduled launch of run {second_run_id} for simple_schedule
-""".format(
+            assert get_logger_output_from_capfd(
+                capfd, "SchedulerDaemon"
+            ) == """2019-03-01 18:00:03 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
+2019-03-01 18:00:03 -0600 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at the following times: 2019-03-01 00:00:00 +0000, 2019-03-02 00:00:00 +0000
+2019-03-01 18:00:03 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {first_run_id} for simple_schedule
+2019-03-01 18:00:03 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {second_run_id} for simple_schedule""".format(
                 first_run_id=instance.get_runs()[1].run_id,
                 second_run_id=instance.get_runs()[0].run_id,
             )
@@ -628,6 +647,53 @@ def test_simple_schedule(external_repo_context, capfd):
             assert len(ticks) == 3
 
 
+@pytest.mark.parametrize("external_repo_context", repos())
+def test_old_tick_schedule(external_repo_context):
+    freeze_datetime = to_timezone(
+        create_pendulum_time(year=2019, month=2, day=27, hour=23, minute=59, second=59, tz="UTC"),
+        "US/Central",
+    )
+    with instance_with_schedules(external_repo_context) as (
+        instance,
+        workspace,
+        external_repo,
+    ):
+        with pendulum.test(freeze_datetime):
+
+            external_schedule = external_repo.get_external_schedule("simple_schedule")
+
+            # Create an old tick from several days ago
+            instance.create_job_tick(
+                JobTickData(
+                    job_origin_id=external_schedule.get_external_origin_id(),
+                    job_name="simple_schedule",
+                    job_type=JobType.SCHEDULE,
+                    status=JobTickStatus.STARTED,
+                    timestamp=pendulum.now("UTC").subtract(days=3).timestamp(),
+                )
+            )
+
+            schedule_origin = external_schedule.get_external_origin()
+
+            # the start time is what determines the number of runs, not the last tick
+            instance.start_schedule_and_update_storage_state(external_schedule)
+
+        freeze_datetime = freeze_datetime.add(seconds=2)
+        with pendulum.test(freeze_datetime):
+            list(
+                launch_scheduled_runs(
+                    instance,
+                    workspace,
+                    logger(),
+                    pendulum.now("UTC"),
+                )
+            )
+
+            assert instance.get_runs_count() == 1
+            ticks = instance.get_job_ticks(schedule_origin.get_id())
+            assert len(ticks) == 2
+
+
 @pytest.mark.parametrize("external_repo_context", repos())
 def test_no_started_schedules(external_repo_context, capfd):
     with instance_with_schedules(external_repo_context) as (
@@ -820,13 +886,11 @@ def test_skip(external_repo_context, capfd):
                 [run.run_id for run in instance.get_runs()],
             )
 
-            captured = capfd.readouterr()
             assert (
-                captured.out
-                == """2019-02-26 18:00:00 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: skip_schedule
-2019-02-26 18:00:00 - SchedulerDaemon - INFO - Evaluating schedule `skip_schedule` at 2019-02-27 00:00:00+0000
-2019-02-26 18:00:00 - SchedulerDaemon - INFO - No run requests returned for skip_schedule, skipping
-"""
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-02-26 18:00:00 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: skip_schedule
+2019-02-26 18:00:00 -0600 - SchedulerDaemon - INFO - Evaluating schedule `skip_schedule` at 2019-02-27 00:00:00 +0000
+2019-02-26 18:00:00 -0600 - SchedulerDaemon - INFO - No run requests returned for skip_schedule, skipping"""
             )
 
 
@@ -889,10 +953,59 @@ def test_wrong_config(external_repo_context, capfd):
             captured = capfd.readouterr()
 
             assert "Failed to fetch execution plan for wrong_config_schedule" in captured.out
-            assert "Error in config for pipeline the_pipeline" in captured.out
+            assert "Error in config for pipeline" in captured.out
             assert 'Missing required config entry "solids" at the root.' in captured.out
 
 
+@pytest.mark.parametrize("external_repo_context", repos())
+def test_schedule_run_default_config(external_repo_context):
+    with instance_with_schedules(external_repo_context) as (
+        instance,
+        workspace,
+        external_repo,
+    ):
+        external_schedule = external_repo.get_external_schedule("default_config_schedule")
+        schedule_origin = external_schedule.get_external_origin()
+        initial_datetime = create_pendulum_time(
+            year=2019, month=2, day=27, hour=0, minute=0, second=0
+        )
+        with pendulum.test(initial_datetime):
+            instance.start_schedule_and_update_storage_state(external_schedule)
+
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
+
+            assert instance.get_runs_count() == 1
+
+            wait_for_all_runs_to_start(instance)
+
+            run = instance.get_runs()[0]
+
+            validate_run_started(
+                run,
+                execution_time=initial_datetime,
+                expected_success=True,
+            )
+
+            ticks = instance.get_job_ticks(schedule_origin.get_id())
+            assert len(ticks) == 1
+            validate_tick(
+                ticks[0],
+                external_schedule,
+                initial_datetime,
+                JobTickStatus.SUCCESS,
+                [run.run_id for run in instance.get_runs()],
+            )
+
+            # wait for run to complete
+            run = instance.get_run_by_id(run.run_id)
+
+            while run.status in IN_PROGRESS_RUN_STATUSES:
+                time.sleep(1)
+                run = instance.get_run_by_id(run.run_id)
+
+            assert run.status == PipelineRunStatus.SUCCESS
+
+
 def _get_unloadable_schedule_origin():
     working_directory = os.path.dirname(__file__)
     recon_repo = ReconstructableRepository.for_file(__file__, "doesnt_exist", working_directory)
@@ -1162,48 +1275,47 @@ def test_bad_load_schedule(external_repo_context, capfd):
 
 
 def test_bad_load_repository_location(capfd):
-    with schedule_instance() as instance, ProcessGrpcServerRegistry() as grpc_server_registry:
-        with DynamicWorkspace(grpc_server_registry) as workspace:
-            fake_origin = _get_unloadable_schedule_origin()
-            initial_datetime = create_pendulum_time(
-                year=2019,
-                month=2,
-                day=27,
-                hour=23,
-                minute=59,
-                second=59,
+    with schedule_instance() as instance, create_test_daemon_workspace() as workspace:
+        fake_origin = _get_unloadable_schedule_origin()
+        initial_datetime = create_pendulum_time(
+            year=2019,
+            month=2,
+            day=27,
+            hour=23,
+            minute=59,
+            second=59,
+        )
+        with pendulum.test(initial_datetime):
+            schedule_state = JobState(
+                fake_origin,
+                JobType.SCHEDULE,
+                JobStatus.RUNNING,
+                ScheduleJobData(
+                    "0 0 * * *", pendulum.now("UTC").timestamp(), "DagsterDaemonScheduler"
+                ),
             )
-            with pendulum.test(initial_datetime):
-                schedule_state = JobState(
-                    fake_origin,
-                    JobType.SCHEDULE,
-                    JobStatus.RUNNING,
-                    ScheduleJobData(
-                        "0 0 * * *", pendulum.now("UTC").timestamp(), "DagsterDaemonScheduler"
-                    ),
-                )
-                instance.add_job_state(schedule_state)
+            instance.add_job_state(schedule_state)
 
-            initial_datetime = initial_datetime.add(seconds=1)
-            with pendulum.test(initial_datetime):
-                list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
+        initial_datetime = initial_datetime.add(seconds=1)
+        with pendulum.test(initial_datetime):
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
-                assert instance.get_runs_count() == 0
+            assert instance.get_runs_count() == 0
 
-                ticks = instance.get_job_ticks(fake_origin.get_id())
+            ticks = instance.get_job_ticks(fake_origin.get_id())
 
-                assert len(ticks) == 0
+            assert len(ticks) == 0
 
-                captured = capfd.readouterr()
-                assert "Scheduler caught an error for schedule doesnt_exist" in captured.out
-                assert "doesnt_exist not found at module scope" in captured.out
+            captured = capfd.readouterr()
+            assert "Scheduler caught an error for schedule doesnt_exist" in captured.out
+            assert "doesnt_exist not found at module scope" in captured.out
 
-            initial_datetime = initial_datetime.add(days=1)
-            with pendulum.test(initial_datetime):
-                list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
-                assert instance.get_runs_count() == 0
-                ticks = instance.get_job_ticks(fake_origin.get_id())
-                assert len(ticks) == 0
+        initial_datetime = initial_datetime.add(days=1)
+        with pendulum.test(initial_datetime):
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
+            assert instance.get_runs_count() == 0
+            ticks = instance.get_job_ticks(fake_origin.get_id())
+            assert len(ticks) == 0
 
 
 @pytest.mark.parametrize("external_repo_context", repos())
@@ -1245,14 +1357,13 @@ def test_multiple_schedules_on_different_time_ranges(external_repo_context, capf
             assert len(hourly_ticks) == 1
             assert hourly_ticks[0].status == JobTickStatus.SUCCESS
 
-            captured = capfd.readouterr()
-
-            assert captured.out == """2019-02-27 18:00:01 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule, simple_hourly_schedule
-2019-02-27 18:00:01 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-28 00:00:00+0000
-2019-02-27 18:00:01 - SchedulerDaemon - INFO - Completed scheduled launch of run {first_run_id} for simple_schedule
-2019-02-27 18:00:01 - SchedulerDaemon - INFO - Evaluating schedule `simple_hourly_schedule` at 2019-02-28 00:00:00+0000
-2019-02-27 18:00:01 - SchedulerDaemon - INFO - Completed scheduled launch of run {second_run_id} for simple_hourly_schedule
-""".format(
+            assert get_logger_output_from_capfd(
+                capfd, "SchedulerDaemon"
+            ) == """2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule, simple_hourly_schedule
+2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-28 00:00:00 +0000
+2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {first_run_id} for simple_schedule
+2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Evaluating schedule `simple_hourly_schedule` at 2019-02-28 00:00:00 +0000
+2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {second_run_id} for simple_hourly_schedule""".format(
                 first_run_id=instance.get_runs()[1].run_id,
                 second_run_id=instance.get_runs()[0].run_id,
             )
@@ -1271,14 +1382,12 @@ def test_multiple_schedules_on_different_time_ranges(external_repo_context, capf
             assert len(hourly_ticks) == 2
             assert len([tick for tick in hourly_ticks if tick.status == JobTickStatus.SUCCESS]) == 2
 
-            captured = capfd.readouterr()
             assert (
-                captured.out
-                == """2019-02-27 19:00:01 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule, simple_hourly_schedule
-2019-02-27 19:00:01 - SchedulerDaemon - INFO - No new runs for simple_schedule
-2019-02-27 19:00:01 - SchedulerDaemon - INFO - Evaluating schedule `simple_hourly_schedule` at 2019-02-28 01:00:00+0000
-2019-02-27 19:00:01 - SchedulerDaemon - INFO - Completed scheduled launch of run {third_run_id} for simple_hourly_schedule
-""".format(
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-02-27 19:00:01 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule, simple_hourly_schedule
+2019-02-27 19:00:01 -0600 - SchedulerDaemon - INFO - No new runs for simple_schedule
+2019-02-27 19:00:01 -0600 - SchedulerDaemon - INFO - Evaluating schedule `simple_hourly_schedule` at 2019-02-28 01:00:00 +0000
+2019-02-27 19:00:01 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {third_run_id} for simple_hourly_schedule""".format(
                     third_run_id=instance.get_runs()[0].run_id
                 )
             )
@@ -1329,17 +1438,18 @@ def test_launch_failure(external_repo_context, capfd):
                 [run.run_id for run in instance.get_runs()],
             )
 
-            captured = capfd.readouterr()
+            logger_output = get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+
             assert (
-                """2019-02-26 18:00:00 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
-2019-02-26 18:00:00 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-27 00:00:00+0000
-2019-02-26 18:00:00 - SchedulerDaemon - ERROR - Run {run_id} created successfully but failed to launch:""".format(
+                """2019-02-26 18:00:00 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
+2019-02-26 18:00:00 -0600 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at 2019-02-27 00:00:00 +0000
+2019-02-26 18:00:00 -0600 - SchedulerDaemon - ERROR - Run {run_id} created successfully but failed to launch:""".format(
                     run_id=instance.get_runs()[0].run_id
                 )
-                in captured.out
+                in logger_output
             )
 
-            assert "The entire purpose of this is to throw on launch" in captured.out
+            assert "The entire purpose of this is to throw on launch" in logger_output
 
 
 def test_partitionless_schedule(capfd):
@@ -1375,15 +1485,12 @@ def test_partitionless_schedule(capfd):
                 partition_time=None,
             )
 
-            captured = capfd.readouterr()
-
             assert (
-                captured.out
-                == """2019-03-04 00:00:00 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: partitionless_schedule
-2019-03-04 00:00:00 - SchedulerDaemon - WARNING - partitionless_schedule has no partition set, so not trying to catch up
-2019-03-04 00:00:00 - SchedulerDaemon - INFO - Evaluating schedule `partitionless_schedule` at 2019-03-04 00:00:00-0600
-2019-03-04 00:00:00 - SchedulerDaemon - INFO - Completed scheduled launch of run {run_id} for partitionless_schedule
-""".format(
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-03-04 00:00:00 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: partitionless_schedule
+2019-03-04 00:00:00 -0600 - SchedulerDaemon - WARNING - partitionless_schedule has no partition set, so not trying to catch up
+2019-03-04 00:00:00 -0600 - SchedulerDaemon - INFO - Evaluating schedule `partitionless_schedule` at 2019-03-04 00:00:00 -0600
+2019-03-04 00:00:00 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {run_id} for partitionless_schedule""".format(
                     run_id=instance.get_runs()[0].run_id
                 )
             )
@@ -1450,13 +1557,13 @@ def test_max_catchup_runs(capfd):
                 partition_time=create_pendulum_time(2019, 3, 2),
             )
 
-            captured = capfd.readouterr()
-            assert captured.out == """2019-03-04 17:59:59 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
-2019-03-04 17:59:59 - SchedulerDaemon - WARNING - simple_schedule has fallen behind, only launching 2 runs
-2019-03-04 17:59:59 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at the following times: 2019-03-03 00:00:00+0000, 2019-03-04 00:00:00+0000
-2019-03-04 17:59:59 - SchedulerDaemon - INFO - Completed scheduled launch of run {first_run_id} for simple_schedule
-2019-03-04 17:59:59 - SchedulerDaemon - INFO - Completed scheduled launch of run {second_run_id} for simple_schedule
-""".format(
+            assert get_logger_output_from_capfd(
+                capfd, "SchedulerDaemon"
+            ) == """2019-03-04 17:59:59 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: simple_schedule
+2019-03-04 17:59:59 -0600 - SchedulerDaemon - WARNING - simple_schedule has fallen behind, only launching 2 runs
+2019-03-04 17:59:59 -0600 - SchedulerDaemon - INFO - Evaluating schedule `simple_schedule` at the following times: 2019-03-03 00:00:00 +0000, 2019-03-04 00:00:00 +0000
+2019-03-04 17:59:59 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {first_run_id} for simple_schedule
+2019-03-04 17:59:59 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {second_run_id} for simple_schedule""".format(
                 first_run_id=instance.get_runs()[1].run_id,
                 second_run_id=instance.get_runs()[0].run_id,
             )
@@ -1496,13 +1603,10 @@ def test_multi_runs(external_repo_context, capfd):
             ticks = instance.get_job_ticks(schedule_origin.get_id())
             assert len(ticks) == 0
 
-            captured = capfd.readouterr()
-
             assert (
-                captured.out
-                == """2019-02-27 17:59:59 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: multi_run_schedule
-2019-02-27 17:59:59 - SchedulerDaemon - INFO - No new runs for multi_run_schedule
-"""
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-02-27 17:59:59 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: multi_run_schedule
+2019-02-27 17:59:59 -0600 - SchedulerDaemon - INFO - No new runs for multi_run_schedule"""
             )
 
         freeze_datetime = freeze_datetime.add(seconds=2)
@@ -1528,15 +1632,12 @@ def test_multi_runs(external_repo_context, capfd):
             validate_run_started(runs[0], execution_time=create_pendulum_time(2019, 2, 28))
             validate_run_started(runs[1], execution_time=create_pendulum_time(2019, 2, 28))
 
-            captured = capfd.readouterr()
-
             assert (
-                captured.out
-                == f"""2019-02-27 18:00:01 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: multi_run_schedule
-2019-02-27 18:00:01 - SchedulerDaemon - INFO - Evaluating schedule `multi_run_schedule` at 2019-02-28 00:00:00+0000
-2019-02-27 18:00:01 - SchedulerDaemon - INFO - Completed scheduled launch of run {runs[1].run_id} for multi_run_schedule
-2019-02-27 18:00:01 - SchedulerDaemon - INFO - Completed scheduled launch of run {runs[0].run_id} for multi_run_schedule
-"""
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == f"""2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: multi_run_schedule
+2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Evaluating schedule `multi_run_schedule` at 2019-02-28 00:00:00 +0000
+2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {runs[1].run_id} for multi_run_schedule
+2019-02-27 18:00:01 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {runs[0].run_id} for multi_run_schedule"""
             )
 
             # Verify idempotence
@@ -1558,15 +1659,12 @@ def test_multi_runs(external_repo_context, capfd):
             assert len([tick for tick in ticks if tick.status == JobTickStatus.SUCCESS]) == 2
             runs = instance.get_runs()
 
-            captured = capfd.readouterr()
-
             assert (
-                captured.out
-                == f"""2019-02-28 18:00:01 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: multi_run_schedule
-2019-02-28 18:00:01 - SchedulerDaemon - INFO - Evaluating schedule `multi_run_schedule` at 2019-03-01 00:00:00+0000
-2019-02-28 18:00:01 - SchedulerDaemon - INFO - Completed scheduled launch of run {runs[1].run_id} for multi_run_schedule
-2019-02-28 18:00:01 - SchedulerDaemon - INFO - Completed scheduled launch of run {runs[0].run_id} for multi_run_schedule
-"""
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == f"""2019-02-28 18:00:01 -0600 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: multi_run_schedule
+2019-02-28 18:00:01 -0600 - SchedulerDaemon - INFO - Evaluating schedule `multi_run_schedule` at 2019-03-01 00:00:00 +0000
+2019-02-28 18:00:01 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {runs[1].run_id} for multi_run_schedule
+2019-02-28 18:00:01 -0600 - SchedulerDaemon - INFO - Completed scheduled launch of run {runs[0].run_id} for multi_run_schedule"""
             )
 
 
@@ -1615,58 +1713,6 @@ def test_multi_runs_missing_run_key(external_repo_context, capfd):
             )
 
 
-@pytest.mark.skipif(seven.IS_WINDOWS, reason="Cron doesn't work on windows")
-def test_run_with_hanging_cron_schedules():
-    # Verify that the system will prompt you to wipe your schedules with the SystemCronScheduler
-    # before you can switch to DagsterDaemonScheduler
-
-    with tempfile.TemporaryDirectory() as temp_dir:
-        with instance_for_test_tempdir(
-            temp_dir,
-            overrides={"scheduler": {"module": "dagster_cron", "class": "SystemCronScheduler"}},
-        ) as cron_instance:
-            with default_repo() as external_repo:
-                cron_instance.start_schedule_and_update_storage_state(
-                    external_repo.get_external_schedule("simple_schedule")
-                )
-
-        # Can't change scheduler to DagsterDaemonScheduler, warns you to wipe
-        with pytest.raises(DagsterScheduleWipeRequired):
-            with instance_for_test_tempdir(
-                temp_dir,
-                overrides={
-                    "scheduler": {
-                        "module": "dagster.core.scheduler",
-                        "class": "DagsterDaemonScheduler",
-                    },
-                },
-            ) as instance:
-                instance.optimize_for_dagit(statement_timeout=500)
-
-        with instance_for_test_tempdir(
-            temp_dir,
-            overrides={
-                "scheduler": {
-                    "module": "dagster_cron",
-                    "class": "SystemCronScheduler",
-                },
-            },
-        ) as cron_instance:
-            cron_instance.wipe_all_schedules()
-
-        # After wiping, now can change the scheduler to DagsterDaemonScheduler
-        with instance_for_test_tempdir(
-            temp_dir,
-            overrides={
-                "scheduler": {
-                    "module": "dagster.core.scheduler",
-                    "class": "DagsterDaemonScheduler",
-                },
-            },
-        ):
-            pass
-
-
 @pytest.mark.parametrize("external_repo_context", repos())
 def test_large_schedule(external_repo_context):
     freeze_datetime = to_timezone(
