@@ -1,8 +1,9 @@
 import pendulum
 import pytest
 from dagster.core.scheduler.job import JobTickStatus
+from dagster.core.test_utils import get_logger_output_from_capfd
 from dagster.scheduler.scheduler import launch_scheduled_runs
-from dagster.seven import create_pendulum_time, to_timezone
+from dagster.seven.compat.pendulum import create_pendulum_time, to_timezone
 from dagster.utils.partitions import DEFAULT_HOURLY_FORMAT_WITH_TIMEZONE
 
 from .test_scheduler_run import (
@@ -21,7 +22,7 @@ def test_non_utc_timezone_run(external_repo_context, capfd):
     # Verify that schedule runs at the expected time in a non-UTC timezone
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         freeze_datetime = to_timezone(
@@ -41,7 +42,7 @@ def test_non_utc_timezone_run(external_repo_context, capfd):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -50,20 +51,17 @@ def test_non_utc_timezone_run(external_repo_context, capfd):
             ticks = instance.get_job_ticks(schedule_origin.get_id())
             assert len(ticks) == 0
 
-            captured = capfd.readouterr()
-
             assert (
-                captured.out
-                == """2019-02-27 21:59:59 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: daily_central_time_schedule
-2019-02-27 21:59:59 - SchedulerDaemon - INFO - No new runs for daily_central_time_schedule
-"""
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-02-27 21:59:59 -0800 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: daily_central_time_schedule
+2019-02-27 21:59:59 -0800 - SchedulerDaemon - INFO - No new runs for daily_central_time_schedule"""
             )
         freeze_datetime = freeze_datetime.add(seconds=2)
         with pendulum.test(freeze_datetime):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -92,14 +90,11 @@ def test_non_utc_timezone_run(external_repo_context, capfd):
                 create_pendulum_time(2019, 2, 27, tz="US/Central"),
             )
 
-            captured = capfd.readouterr()
-
             assert (
-                captured.out
-                == """2019-02-27 22:00:01 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: daily_central_time_schedule
-2019-02-27 22:00:01 - SchedulerDaemon - INFO - Evaluating schedule `daily_central_time_schedule` at 2019-02-28 00:00:00-0600
-2019-02-27 22:00:01 - SchedulerDaemon - INFO - Completed scheduled launch of run {run_id} for daily_central_time_schedule
-""".format(
+                get_logger_output_from_capfd(capfd, "SchedulerDaemon")
+                == """2019-02-27 22:00:01 -0800 - SchedulerDaemon - INFO - Checking for new runs for the following schedules: daily_central_time_schedule
+2019-02-27 22:00:01 -0800 - SchedulerDaemon - INFO - Evaluating schedule `daily_central_time_schedule` at 2019-02-28 00:00:00 -0600
+2019-02-27 22:00:01 -0800 - SchedulerDaemon - INFO - Completed scheduled launch of run {run_id} for daily_central_time_schedule""".format(
                     run_id=instance.get_runs()[0].run_id
                 )
             )
@@ -108,7 +103,7 @@ def test_non_utc_timezone_run(external_repo_context, capfd):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -124,7 +119,7 @@ def test_differing_timezones(external_repo_context):
     # Two schedules, one using US/Central, the other on US/Eastern
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         freeze_datetime = to_timezone(
@@ -152,7 +147,7 @@ def test_differing_timezones(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -170,7 +165,7 @@ def test_differing_timezones(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -209,7 +204,7 @@ def test_differing_timezones(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -245,7 +240,7 @@ def test_differing_timezones(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -266,7 +261,7 @@ def test_differing_timezones(external_repo_context):
 def test_different_days_in_different_timezones(external_repo_context):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         freeze_datetime = to_timezone(
@@ -285,7 +280,7 @@ def test_different_days_in_different_timezones(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -299,7 +294,7 @@ def test_different_days_in_different_timezones(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -332,7 +327,7 @@ def test_different_days_in_different_timezones(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -348,7 +343,7 @@ def test_hourly_dst_spring_forward(external_repo_context):
     # Verify that an hourly schedule still runs hourly during the spring DST transition
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         # 1AM CST
@@ -373,7 +368,7 @@ def test_hourly_dst_spring_forward(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -413,7 +408,7 @@ def test_hourly_dst_spring_forward(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -428,7 +423,7 @@ def test_hourly_dst_fall_back(external_repo_context):
     # Verify that an hourly schedule still runs hourly during the fall DST transition
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         # 12:30 AM CST
@@ -453,7 +448,7 @@ def test_hourly_dst_fall_back(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -506,7 +501,7 @@ def test_hourly_dst_fall_back(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -521,7 +516,7 @@ def test_daily_dst_spring_forward(external_repo_context):
     # Verify that a daily schedule still runs once per day during the spring DST transition
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         # Night before DST
@@ -544,7 +539,7 @@ def test_daily_dst_spring_forward(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -589,7 +584,7 @@ def test_daily_dst_spring_forward(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -604,7 +599,7 @@ def test_daily_dst_fall_back(external_repo_context):
     # Verify that a daily schedule still runs once per day during the fall DST transition
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         # Night before DST
@@ -627,7 +622,7 @@ def test_daily_dst_fall_back(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -672,7 +667,7 @@ def test_daily_dst_fall_back(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -688,7 +683,7 @@ def test_execute_during_dst_transition_spring_forward(external_repo_context):
     # by the DST transition does not execute for that day
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         # Day before DST
@@ -713,7 +708,7 @@ def test_execute_during_dst_transition_spring_forward(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -765,7 +760,7 @@ def test_execute_during_dst_transition_spring_forward(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -779,7 +774,7 @@ def test_execute_during_dst_transition_spring_forward(external_repo_context):
 def test_execute_during_dst_transition_fall_back(external_repo_context):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         # A schedule that runs daily during a time that occurs twice during a fall DST transition
@@ -805,7 +800,7 @@ def test_execute_during_dst_transition_fall_back(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -848,7 +843,7 @@ def test_execute_during_dst_transition_fall_back(external_repo_context):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
